/******************************Includes************************************/
#include <linux/module.h>
#include <linux/errno.h>
#include <mach/am_regs.h>

#include <mach/am_regs.h>

#include <linux/amlogic/tvin/tvin.h>
#include "../tvin_global.h"
#include "../tvin_format_table.h"
#include "tvafe_regs.h"
#include "tvafe_adc.h"
#include "tvafe_cvd.h"
#include "tvafe_general.h"
/***************************Local defines**********************************/
/* calibration validation funtion enable/disable */
// #define TVAFE_ADC_CAL_VALIDATION

/* edid config reg value */
#define TVAFE_EDID_CONFIG           0x03804050//0x03800050

/* calibration defines */
#define TVAFE_ADC_CAL_STEP_GAIN     1
#define TVAFE_ADC_CAL_STEP_CLAMP    1
#define TVAFE_ADC_CAL_STEP_SHIFT    1
#define TVAFE_ADC_CAL_STEP_STAGE    (TVAFE_ADC_CAL_STEP_GAIN*16 + TVAFE_ADC_CAL_STEP_SHIFT)

/***************************local Variables**********************************/
struct tvafe_adc_cal_s cal_std_value_component = {
	.a_analog_clamp	   = 73,
	.a_analog_gain 	   = 112,
	.a_digital_gain	   = 975, // 0dB
	.a_digital_offset1 = 1,
	.a_digital_offset2 = 28,
	.b_analog_clamp	   = 65,
	.b_analog_gain 	   = 122,
	.b_digital_gain	   = 899, // 0dB
	.b_digital_offset1 = 3,
	.b_digital_offset2 = 64,
	.c_analog_clamp	   = 62,
	.c_analog_gain 	   = 121,
	.c_digital_gain	   = 899, // 0dB
	.c_digital_offset1 = 2,
	.c_digital_offset2 = 64,
};

struct tvafe_adc_cal_s cal_std_value_vga = {
	.a_analog_clamp    = 66,
	.a_analog_gain     = 153,
	.a_digital_gain    = 1024, // 0dB
	.a_digital_offset1 = 0,
	.a_digital_offset2 = 0,
	.b_analog_clamp    = 71,
	.b_analog_gain     = 162,
	.b_digital_gain    = 1024, // 0dB
	.b_digital_offset1 = 0,
	.b_digital_offset2 = 0,
	.c_analog_clamp    = 65,
	.c_analog_gain     = 162,
	.c_digital_gain    = 1024,// 0dB
	.c_digital_offset1 = 0,
	.c_digital_offset2 = 0,
};

static int threshold_value = 32;
#if (MESON_CPU_TYPE < MESON_CPU_TYPE_MESONG9TV)
static enum tvafe_adc_pin_e tvafe_default_cvbs_out = TVAFE_ADC_PIN_A_PGA_0;
#endif
const signed short tvafe_comp_hs_patch[TVIN_SIG_FMT_COMP_MAX - TVIN_SIG_FMT_COMP_480P_60HZ_D000] =
{
	//VGA
#if 0
	0,  // TVIN_SIG_FMT_VGA_560X384P_60D147,
	0,  // TVIN_SIG_FMT_VGA_640X200P_59D924,
	0,  // TVIN_SIG_FMT_VGA_640X350P_85D080,
	0,  // TVIN_SIG_FMT_VGA_640X400P_59D940,
	0,  // TVIN_SIG_FMT_VGA_640X400P_85D080,
	0,  // TVIN_SIG_FMT_VGA_640X400P_59D638,
	0,  // TVIN_SIG_FMT_VGA_640X400P_56D416,
	0,  // TVIN_SIG_FMT_VGA_640X480P_66D619,
	0,  // TVIN_SIG_FMT_VGA_640X480P_66D667,   // 10
	0,  // TVIN_SIG_FMT_VGA_640X480P_59D940,
	0,  // TVIN_SIG_FMT_VGA_640X480P_60D000,
	0,  // TVIN_SIG_FMT_VGA_640X480P_72D809,
	0,  // TVIN_SIG_FMT_VGA_640X480P_75D000_A,
	0,  // TVIN_SIG_FMT_VGA_640X480P_85D008,
	0,  // TVIN_SIG_FMT_VGA_640X480P_59D638,
	0,  // TVIN_SIG_FMT_VGA_640X480P_75D000_B,
	0,  // TVIN_SIG_FMT_VGA_640X870P_75D000,
	0,  // TVIN_SIG_FMT_VGA_720X350P_70D086,
	0,  // TVIN_SIG_FMT_VGA_720X400P_85D039,   // 20
	0,  // TVIN_SIG_FMT_VGA_720X400P_70D086,
	0,  // TVIN_SIG_FMT_VGA_720X400P_87D849,
	0,  // TVIN_SIG_FMT_VGA_720X400P_59D940,
	0,  // TVIN_SIG_FMT_VGA_720X480P_59D940,
	0,  // TVIN_SIG_FMT_VGA_768X480P_59D896,
	0,  // TVIN_SIG_FMT_VGA_800X600P_56D250,
	0,  // TVIN_SIG_FMT_VGA_800X600P_60D317,
	0,  // TVIN_SIG_FMT_VGA_800X600P_72D188,
	0,  // TVIN_SIG_FMT_VGA_800X600P_75D000,
	0,  // TVIN_SIG_FMT_VGA_800X600P_85D061,   // 30
	0,  // TVIN_SIG_FMT_VGA_832X624P_75D087,
	0,  // TVIN_SIG_FMT_VGA_848X480P_84D751,
	0,  // TVIN_SIG_FMT_VGA_960X600P_59D635,
	0,  // TVIN_SIG_FMT_VGA_1024X768P_59D278,
	0,  // TVIN_SIG_FMT_VGA_1024X768P_74D927,
	0,  // TVIN_SIG_FMT_VGA_1024X768P_60D004,
	0,  // TVIN_SIG_FMT_VGA_1024X768P_70D069,
	0,  // TVIN_SIG_FMT_VGA_1024X768P_75D029,
	0,  // TVIN_SIG_FMT_VGA_1024X768P_84D997,
	0,  // TVIN_SIG_FMT_VGA_1024X768P_60D000,   // 40
	0,  // TVIN_SIG_FMT_VGA_1024X768P_74D925,
	0,  // TVIN_SIG_FMT_VGA_1024X768P_75D020,
	0,  // TVIN_SIG_FMT_VGA_1024X768P_70D008,
	0,  // TVIN_SIG_FMT_VGA_1024X768P_75D782,
	0,  // TVIN_SIG_FMT_VGA_1024X768P_77D069,
	0,  // TVIN_SIG_FMT_VGA_1024X768P_71D799,
	0,  // TVIN_SIG_FMT_VGA_1024X1024P_60D000,
	0,  // TVIN_SIG_FMT_VGA_1152X864P_70D012,
	0,  // TVIN_SIG_FMT_VGA_1152X864P_75D000,
	0,  // TVIN_SIG_FMT_VGA_1152X864P_84D999,   // 50
	0,  // TVIN_SIG_FMT_VGA_1152X870P_75D062,
	0,  // TVIN_SIG_FMT_VGA_1152X900P_65D950,
	0,  // TVIN_SIG_FMT_VGA_1152X900P_66D004,
	0,  // TVIN_SIG_FMT_VGA_1152X900P_76D047,
	0,  // TVIN_SIG_FMT_VGA_1152X900P_76D149,
	0,  // TVIN_SIG_FMT_VGA_1280X720P_59D855,
	0,  // TVIN_SIG_FMT_VGA_1280X768P_59D870,
	0,  // TVIN_SIG_FMT_VGA_1280X768P_59D995,
	0,  // TVIN_SIG_FMT_VGA_1280X768P_60D100,
	0,  // TVIN_SIG_FMT_VGA_1280X768P_74D893,   // 60
	0,  // TVIN_SIG_FMT_VGA_1280X768P_84D837,
	0,  // TVIN_SIG_FMT_VGA_1280X800P_59D810,
	0,  // TVIN_SIG_FMT_VGA_1280X960P_60D000,
	0,  // TVIN_SIG_FMT_VGA_1280X960P_75D000,
	0,  // TVIN_SIG_FMT_VGA_1280X960P_85D002,
	0,  // TVIN_SIG_FMT_VGA_1280X1024P_60D020,
	0,  // TVIN_SIG_FMT_VGA_1280X1024P_75D025,
	0,  // TVIN_SIG_FMT_VGA_1280X1024P_85D024,
	0,  // TVIN_SIG_FMT_VGA_1280X1024P_59D979,
	0,  // TVIN_SIG_FMT_VGA_1280X1024P_72D005,   // 70
	0,  // TVIN_SIG_FMT_VGA_1280X1024P_60D002,
	0,  // TVIN_SIG_FMT_VGA_1280X1024P_67D003,
	0,  // TVIN_SIG_FMT_VGA_1280X1024P_74D112,
	0,  // TVIN_SIG_FMT_VGA_1280X1024P_76D179,
	0,  // TVIN_SIG_FMT_VGA_1280X1024P_66D718,
	0,  // TVIN_SIG_FMT_VGA_1280X1024P_66D677,
	0,  // TVIN_SIG_FMT_VGA_1280X1024P_76D107,
	0,  // TVIN_SIG_FMT_VGA_1280X1024P_59D996,
	0,  // TVIN_SIG_FMT_VGA_1360X768P_59D799,
	0,  // TVIN_SIG_FMT_VGA_1440X1080P_60D000,   // 80
	0,  // TVIN_SIG_FMT_VGA_1600X1200P_60D000,
	0,  // TVIN_SIG_FMT_VGA_1600X1200P_65D000,
	0,  // TVIN_SIG_FMT_VGA_1600X1200P_70D000,
	0,  // TVIN_SIG_FMT_VGA_1680X1080P_60D000,
	0,  // TVIN_SIG_FMT_VGA_1920X1080P_59D963,
	0,  // TVIN_SIG_FMT_VGA_1920X1080P_60D000,
	0,  // TVIN_SIG_FMT_VGA_1920X1200P_59D950,
	0,  // TVIN_SIG_FMT_VGA_MAX,
#endif
	//Component format
#ifdef CONFIG_ADC_DOUBLE_SAMPLING_FOR_SD
	-47,   	// TVIN_SIG_FMT_COMPONENT_480P_60D000,
	17,  	// TVIN_SIG_FMT_COMPONENT_480I_59D940,
	-44,   	// TVIN_SIG_FMT_COMPONENT_576P_50D000,
	7,  	//TVIN_SIG_FMT_COMPONENT_576I_50D000,
#else
	-47,   // TVIN_SIG_FMT_COMPONENT_480P_60D000,
	6,   // TVIN_SIG_FMT_COMPONENT_480I_59D940,
	-47,   // TVIN_SIG_FMT_COMPONENT_576P_50D000,
	3,   // TVIN_SIG_FMT_COMPONENT_576I_50D000,
#endif
	-29,   // TVIN_SIG_FMT_COMPONENT_720P_59D940,
	-29,   // TVIN_SIG_FMT_COMPONENT_720P_50D000,
	0,   // TVIN_SIG_FMT_COMPONENT_1080P_23D976,
	0,   // TVIN_SIG_FMT_COMPONENT_1080P_24D000,
	14,   // TVIN_SIG_FMT_COMPONENT_1080P_25D000,    // 90
	-33,   // TVIN_SIG_FMT_COMPONENT_1080P_30D000,
	-29,   // TVIN_SIG_FMT_COMPONENT_1080P_50D000,
	-30,   // TVIN_SIG_FMT_COMPONENT_1080P_60D000,
	0,   // TVIN_SIG_FMT_COMPONENT_1080I_47D952,
	0,   // TVIN_SIG_FMT_COMPONENT_1080I_48D000,
	-29,   // TVIN_SIG_FMT_COMPONENT_1080I_50D000_A,
	0,   // TVIN_SIG_FMT_COMPONENT_1080I_50D000_B,
	0,   // TVIN_SIG_FMT_COMPONENT_1080I_50D000_C,
	-28,   // TVIN_SIG_FMT_COMPONENT_1080I_60D000,    //99
};

/* TOP */ //TVIN_SIG_FMT_VGA_800X600P_60D317
const static  int vga_top_reg_default[][2] = {
	{TVFE_DVSS_MUXCTRL             , 0x017000008,} ,// TVFE_DVSS_MUXCTRL
	{TVFE_DVSS_MUXVS_REF           , 0x00000000,} ,// TVFE_DVSS_MUXVS_REF
	{TVFE_DVSS_MUXCOAST_V          , 0x0200000c,} ,// TVFE_DVSS_MUXCOAST_V
	{TVFE_DVSS_SEP_HVWIDTH         , 0x000a0073,} ,// TVFE_DVSS_SEP_HVWIDTH
	{TVFE_DVSS_SEP_HPARA           , 0x026b0343,} ,// TVFE_DVSS_SEP_HPARA
	{TVFE_DVSS_SEP_VINTEG          , 0x0fff0100,} ,// TVFE_DVSS_SEP_VINTEG
	{TVFE_DVSS_SEP_H_THR           , 0x00005002,} ,// TVFE_DVSS_SEP_H_THR
	{TVFE_DVSS_SEP_CTRL            , 0x40000008,} ,// TVFE_DVSS_SEP_CTRL
	{TVFE_DVSS_GEN_WIDTH           , 0x00080008,} ,// TVFE_DVSS_GEN_WIDTH
	{TVFE_DVSS_GEN_PRD             , 0x020d0359,} ,// TVFE_DVSS_GEN_PRD
	{TVFE_DVSS_GEN_COAST           , 0x01cc001c,} ,// TVFE_DVSS_GEN_COAST
	{TVFE_DVSS_NOSIG_PARA          , 0x00000009,} ,// TVFE_DVSS_NOSIG_PARA
	{TVFE_DVSS_NOSIG_PLS_TH        , 0x05000010,} ,// TVFE_DVSS_NOSIG_PLS_TH
	{TVFE_DVSS_GATE_H              , 0x00260010,} ,// TVFE_DVSS_GATE_H
	{TVFE_DVSS_GATE_V              , 0x00000000,} ,// TVFE_DVSS_GATE_V
	{TVFE_DVSS_INDICATOR1          , 0x00000000,} ,// TVFE_DVSS_INDICATOR1
	{TVFE_DVSS_INDICATOR2          , 0x00000000,} ,// TVFE_DVSS_INDICATOR2
	{TVFE_DVSS_MVDET_CTRL1         , 0x00000000,} ,// TVFE_DVSS_MVDET_CTRL1
	{TVFE_DVSS_MVDET_CTRL2         , 0x00000000,} ,// TVFE_DVSS_MVDET_CTRL2
	{TVFE_DVSS_MVDET_CTRL3         , 0x00000000,} ,// TVFE_DVSS_MVDET_CTRL3
	{TVFE_DVSS_MVDET_CTRL4         , 0x00000000,} ,// TVFE_DVSS_MVDET_CTRL4
	{TVFE_DVSS_MVDET_CTRL5         , 0x00000000,} ,// TVFE_DVSS_MVDET_CTRL5
	{TVFE_DVSS_MVDET_CTRL6         , 0x00000000,} ,// TVFE_DVSS_MVDET_CTRL6
	{TVFE_DVSS_MVDET_CTRL7         , 0x00000000,} ,// TVFE_DVSS_MVDET_CTRL7
	{TVFE_SYNCTOP_SPOL_MUXCTRL     , 0x0002f009,} ,// TVFE_SYNCTOP_SPOL_MUXCTRL
	{TVFE_SYNCTOP_INDICATOR1_HCNT  , 0x00000000,} ,// TVFE_SYNCTOP_INDICATOR1_HCNT
	{TVFE_SYNCTOP_INDICATOR2_VCNT  , 0x00000000,} ,// TVFE_SYNCTOP_INDICATOR2_VCNT
	{TVFE_SYNCTOP_INDICATOR3       , 0x00000000,} ,// TVFE_SYNCTOP_INDICATOR3
	{TVFE_SYNCTOP_SFG_MUXCTRL1     , 0x81315107,} ,// TVFE_SYNCTOP_SFG_MUXCTRL1
	{TVFE_SYNCTOP_SFG_MUXCTRL2     , 0x01330000,} ,// TVFE_SYNCTOP_SFG_MUXCTRL2
	{TVFE_SYNCTOP_INDICATOR4       , 0x00000000,} ,// TVFE_SYNCTOP_INDICATOR4
	{TVFE_SYNCTOP_SAM_MUXCTRL      , 0x00082001,} ,// TVFE_SYNCTOP_SAM_MUXCTRL
	{TVFE_MISC_WSS1_MUXCTRL1       , 0x00000000,} ,// TVFE_MISC_WSS1_MUXCTRL1
	{TVFE_MISC_WSS1_MUXCTRL2       , 0x00000000,} ,// TVFE_MISC_WSS1_MUXCTRL2
	{TVFE_MISC_WSS2_MUXCTRL1       , 0x00000000,} ,// TVFE_MISC_WSS2_MUXCTRL1
	{TVFE_MISC_WSS2_MUXCTRL2       , 0x00000000,} ,// TVFE_MISC_WSS2_MUXCTRL2
	{TVFE_MISC_WSS1_INDICATOR1     , 0x00000000,} ,// TVFE_MISC_WSS1_INDICATOR1
	{TVFE_MISC_WSS1_INDICATOR2     , 0x00000000,} ,// TVFE_MISC_WSS1_INDICATOR2
	{TVFE_MISC_WSS1_INDICATOR3     , 0x00000000,} ,// TVFE_MISC_WSS1_INDICATOR3
	{TVFE_MISC_WSS1_INDICATOR4     , 0x00000000,} ,// TVFE_MISC_WSS1_INDICATOR4
	{TVFE_MISC_WSS1_INDICATOR5     , 0x00000000,} ,// TVFE_MISC_WSS1_INDICATOR5
	{TVFE_MISC_WSS2_INDICATOR1     , 0x00000000,} ,// TVFE_MISC_WSS2_INDICATOR1
	{TVFE_MISC_WSS2_INDICATOR2     , 0x00000000,} ,// TVFE_MISC_WSS2_INDICATOR2
	{TVFE_MISC_WSS2_INDICATOR3     , 0x00000000,} ,// TVFE_MISC_WSS2_INDICATOR3
	{TVFE_MISC_WSS2_INDICATOR4     , 0x00000000,} ,// TVFE_MISC_WSS2_INDICATOR4
	{TVFE_MISC_WSS2_INDICATOR5     , 0x00000000,} ,// TVFE_MISC_WSS2_INDICATOR5
	{TVFE_AP_MUXCTRL1              , 0x11310010,} ,// TVFE_AP_MUXCTRL1
	{TVFE_AP_MUXCTRL2              , 0x00200010,} ,// TVFE_AP_MUXCTRL2
	{TVFE_AP_MUXCTRL3              , 0x10000030,} ,// TVFE_AP_MUXCTRL3
	{TVFE_AP_MUXCTRL4              , 0x00000000,} ,// TVFE_AP_MUXCTRL4
	{TVFE_AP_MUXCTRL5              , 0x10040000,} ,// TVFE_AP_MUXCTRL5
	{TVFE_AP_INDICATOR1            , 0x00000000,} ,// TVFE_AP_INDICATOR1
	{TVFE_AP_INDICATOR2            , 0x00000000,} ,// TVFE_AP_INDICATOR2
	{TVFE_AP_INDICATOR3            , 0x00000000,} ,// TVFE_AP_INDICATOR3
	{TVFE_AP_INDICATOR4            , 0x00000000,} ,// TVFE_AP_INDICATOR4
	{TVFE_AP_INDICATOR5            , 0x00000000,} ,// TVFE_AP_INDICATOR5
	{TVFE_AP_INDICATOR6            , 0x00000000,} ,// TVFE_AP_INDICATOR6
	{TVFE_AP_INDICATOR7            , 0x00000000,} ,// TVFE_AP_INDICATOR7
	{TVFE_AP_INDICATOR8            , 0x00000000,} ,// TVFE_AP_INDICATOR8
	{TVFE_AP_INDICATOR9            , 0x00000000,} ,// TVFE_AP_INDICATOR9
	{TVFE_AP_INDICATOR10           , 0x00000000,} ,// TVFE_AP_INDICATOR10
	{TVFE_AP_INDICATOR11           , 0x00000000,} ,// TVFE_AP_INDICATOR11
	{TVFE_AP_INDICATOR12           , 0x00000000,} ,// TVFE_AP_INDICATOR12
	{TVFE_AP_INDICATOR13           , 0x00000000,} ,// TVFE_AP_INDICATOR13
	{TVFE_AP_INDICATOR14           , 0x00000000,} ,// TVFE_AP_INDICATOR14
	{TVFE_AP_INDICATOR15           , 0x00000000,} ,// TVFE_AP_INDICATOR15
	{TVFE_AP_INDICATOR16           , 0x00000000,} ,// TVFE_AP_INDICATOR16
	{TVFE_AP_INDICATOR17           , 0x00000000,} ,// TVFE_AP_INDICATOR17
	{TVFE_AP_INDICATOR18           , 0x00000000,} ,// TVFE_AP_INDICATOR18
	{TVFE_AP_INDICATOR19           , 0x00000000,} ,// TVFE_AP_INDICATOR19
	{TVFE_BD_MUXCTRL1              , 0x01320000,} ,// TVFE_BD_MUXCTRL1
	{TVFE_BD_MUXCTRL2              , 0x0020d000,} ,// TVFE_BD_MUXCTRL2
	{TVFE_BD_MUXCTRL3              , 0x00000000,} ,// TVFE_BD_MUXCTRL3
	{TVFE_BD_MUXCTRL4              , 0x00000000,} ,// TVFE_BD_MUXCTRL4
	{TVFE_CLP_MUXCTRL1             , 0x00000000,} ,// TVFE_CLP_MUXCTRL1
	{TVFE_CLP_MUXCTRL2             , 0x00000000,} ,// TVFE_CLP_MUXCTRL2
	{TVFE_CLP_MUXCTRL3             , 0x00000000,} ,// TVFE_CLP_MUXCTRL3
	{TVFE_CLP_MUXCTRL4             , 0x00000000,} ,// TVFE_CLP_MUXCTRL4
	{TVFE_CLP_INDICATOR1           , 0x00000000,} ,// TVFE_CLP_INDICATOR1
	{TVFE_BPG_BACKP_H              , 0x00000000,} ,// TVFE_BPG_BACKP_H
	{TVFE_BPG_BACKP_V              , 0x00000000,} ,// TVFE_BPG_BACKP_V
	{TVFE_DEG_H                    , 0x003f80d8,} ,// TVFE_DEG_H
	{TVFE_DEG_VODD                 , 0x0027301b,} ,// TVFE_DEG_VODD
	{TVFE_DEG_VEVEN                , 0x0027301b,} ,// TVFE_DEG_VEVEN
	{TVFE_OGO_OFFSET1              , 0x00000000,} ,// TVFE_OGO_OFFSET1
	{TVFE_OGO_GAIN1                , 0x00000000,} ,// TVFE_OGO_GAIN1
	{TVFE_OGO_GAIN2                , 0x00000000,} ,// TVFE_OGO_GAIN2
	{TVFE_OGO_OFFSET2              , 0x00000000,} ,// TVFE_OGO_OFFSET2
	{TVFE_OGO_OFFSET3              , 0x00000000,} ,// TVFE_OGO_OFFSET3
	{TVFE_VAFE_CTRL                , 0x00000001,} ,// TVFE_VAFE_CTRL
	{TVFE_VAFE_STATUS              , 0x00000000,} ,// TVFE_VAFE_STATUS
	{TVFE_TOP_CTRL                 , 0x00008740,} ,// TVFE_TOP_CTRL
	{TVFE_CLAMP_INTF               , 0x00000000,} ,// TVFE_CLAMP_INTF
	{TVFE_RST_CTRL                 , 0x00000000,} ,// TVFE_RST_CTRL
	{TVFE_EXT_VIDEO_AFE_CTRL_MUX1  , 0x00000000,} ,// TVFE_EXT_VIDEO_AFE_CTRL_MUX1
	//{TVFE_AAFILTER_CTRL1           , 0x00082222,} ,// TVFE_AAFILTER_CTRL1
	//{TVFE_AAFILTER_CTRL2           , 0x252b39c6,} ,// TVFE_AAFILTER_CTRL2
	{TVFE_EDID_CONFIG              , TVAFE_EDID_CONFIG,} ,// TVFE_EDID_CONFIG
	{TVFE_EDID_RAM_ADDR            , 0x00000100,} ,// TVFE_EDID_RAM_ADDR
	{TVFE_EDID_RAM_WDATA           , 0x00000000,} ,// TVFE_EDID_RAM_WDATA
	{TVFE_EDID_RAM_RDATA           , 0x00000000,} ,// TVFE_EDID_RAM_RDATA
	{TVFE_APB_ERR_CTRL_MUX1        , 0x8fff8fff,} ,// TVFE_APB_ERR_CTRL_MUX1
	{TVFE_APB_ERR_CTRL_MUX2        , 0x00008fff,} ,// TVFE_APB_ERR_CTRL_MUX2
	{TVFE_APB_INDICATOR1           , 0x00000000,} ,// TVFE_APB_INDICATOR1
	{TVFE_APB_INDICATOR2           , 0x00000000,} ,// TVFE_APB_INDICATOR2
	{TVFE_ADC_READBACK_CTRL        , 0xa0142003,} ,// TVFE_ADC_READBACK_CTRL
	{TVFE_ADC_READBACK_INDICATOR   , 0x00000000,} ,// TVFE_ADC_READBACK_INDICATOR
	{TVFE_INT_CLR                  , 0x00000000,} ,// TVFE_INT_CLR
	{TVFE_INT_MSKN                 , 0x00000000,} ,// TVFE_INT_MASKN
	{TVFE_INT_INDICATOR1           , 0x00000000,} ,// TVFE_INT_INDICATOR1
	{TVFE_INT_SET                  , 0x00000000,} ,// TVFE_INT_SET
	{TVFE_CHIP_VERSION             , 0x00000000,} ,// TVFE_CHIP_VERSION
	{TVFE_FREERUN_GEN_WIDTH        , 0x00080008,},//TVFE_FREERUN_GEN_WIDTH
	{TVFE_FREERUN_GEN_PRD          , 0x020d0359,},//TVFE_FREERUN_GEN_PRD
	{TVFE_FREERUN_GEN_COAST        , 0x01cc001c,},//TVFE_FREERUN_GEN_COAST
	{TVFE_FREERUN_GEN_CTRL         , 0x00000001,},//TVFE_FREERUN_GEN_CTRL
	{TVFE_AAFILTER_CTRL1            ,   0x00100000,},//TVFE_AAFILTER_CTRL1 bypass all
	{TVFE_AAFILTER_CTRL2            ,   0x00000000,},//TVFE_AAFILTER_CTRL2
	{TVFE_AAFILTER_CTRL3            ,   0x00000000,},//TVFE_AAFILTER_CTRL3
	{TVFE_AAFILTER_CTRL4            ,   0x00000000,},//TVFE_AAFILTER_CTRL4
	{TVFE_AAFILTER_CTRL5            ,   0x00000000,},//TVFE_AAFILTER_CTRL5
	{TVFE_SOG_MON_CTRL1            ,   0x00000000,},//TVFE_SOG_MON_CTRL1
	{TVFE_ADC_READBACK_CTRL1,    0x00000000,},//TVFE_ADC_READBACK_CTRL1
	{TVFE_ADC_READBACK_CTRL2,    0x00000000,},//TVFE_ADC_READBACK_CTRL2
	{TVFE_AFC_CTRL1                      ,    0x00000000,},//TVFE_AFC_CTRL1
	{TVFE_AFC_CTRL2                      ,    0x00000000,},//TVFE_AFC_CTRL2
	{TVFE_AFC_CTRL3                      ,    0x00000000,},//TVFE_AFC_CTRL3
	{TVFE_AFC_CTRL4                      ,    0x00000000,},//TVFE_AFC_CTRL4
	{TVFE_AFC_CTRL5                      ,    0x00000000,},//TVFE_AFC_CTRL5
	{0xFFFFFFFF                    , 0x00000000,}, // TVFE_CHIP_VERSION
}; //TVIN_SIG_FMT_VGA_800X600P_60D317
/* TOP */

///zhuang wei
const static  int comp_top_reg_default[][2] = {
	{TVFE_DVSS_MUXCTRL              , 0x172a4083/*0x072a1480*/,} ,// TVFE_DVSS_MUXCTRL //zhuang
	{TVFE_DVSS_MUXVS_REF           , 0x00000000,} ,// TVFE_DVSS_MUXVS_REF
	{TVFE_DVSS_MUXCOAST_V          , 0x00000000,} ,// TVFE_DVSS_MUXCOAST_V
	{TVFE_DVSS_SEP_HVWIDTH         , 0x00000000,} ,// TVFE_DVSS_SEP_HVWIDTH
	{TVFE_DVSS_SEP_HPARA           , 0x00000000,} ,// TVFE_DVSS_SEP_HPARA
	{TVFE_DVSS_SEP_VINTEG          , 0x00000000,} ,// TVFE_DVSS_SEP_VINTEG
	{TVFE_DVSS_SEP_H_THR           , 0x00000000,} ,// TVFE_DVSS_SEP_H_THR
	{TVFE_DVSS_SEP_CTRL            , 0x00000000,} ,// TVFE_DVSS_SEP_CTRL
	{TVFE_DVSS_GEN_WIDTH           , 0x00080008,} ,// TVFE_DVSS_GEN_WIDTH
	{TVFE_DVSS_GEN_PRD             , 0x020d035a,} ,// TVFE_DVSS_GEN_PRD
	{TVFE_DVSS_GEN_COAST           , 0x00000000,} ,// TVFE_DVSS_GEN_COAST
	{TVFE_DVSS_NOSIG_PARA          , 0x0000000d/*0x00000000*/,} ,// TVFE_DVSS_NOSIG_PARA
	{TVFE_DVSS_NOSIG_PLS_TH        , 0x00FF0010/*0x00000000*/,} ,// TVFE_DVSS_NOSIG_PLS_TH
	{TVFE_DVSS_GATE_H              , 0x00260010,} ,// TVFE_DVSS_GATE_H
	{TVFE_DVSS_GATE_V              , 0x00000000,} ,// TVFE_DVSS_GATE_V
	{TVFE_DVSS_INDICATOR1          , 0x00000000,} ,// TVFE_DVSS_INDICATOR1
	{TVFE_DVSS_INDICATOR2          , 0x00000000,} ,// TVFE_DVSS_INDICATOR2
	{TVFE_DVSS_MVDET_CTRL1         , 0x00000000,} ,// TVFE_DVSS_MVDET_CTRL1
	{TVFE_DVSS_MVDET_CTRL2         , 0x00000000,} ,// TVFE_DVSS_MVDET_CTRL2
	{TVFE_DVSS_MVDET_CTRL3         , 0x00000000,} ,// TVFE_DVSS_MVDET_CTRL3
	{TVFE_DVSS_MVDET_CTRL4         , 0x00000000,} ,// TVFE_DVSS_MVDET_CTRL4
	{TVFE_DVSS_MVDET_CTRL5         , 0x00000000,} ,// TVFE_DVSS_MVDET_CTRL5
	{TVFE_DVSS_MVDET_CTRL6         , 0x00000000,} ,// TVFE_DVSS_MVDET_CTRL6
	{TVFE_DVSS_MVDET_CTRL7         , 0x00000000,} ,// TVFE_DVSS_MVDET_CTRL7
	{TVFE_SYNCTOP_SPOL_MUXCTRL     , 0x00000009,} ,// TVFE_SYNCTOP_SPOL_MUXCTRL
	{TVFE_SYNCTOP_INDICATOR1_HCNT  , 0x00000000,} ,// TVFE_SYNCTOP_INDICATOR1_HCNT
	{TVFE_SYNCTOP_INDICATOR2_VCNT  , 0x00000000,} ,// TVFE_SYNCTOP_INDICATOR2_VCNT
	{TVFE_SYNCTOP_INDICATOR3       , 0x00000000,} ,// TVFE_SYNCTOP_INDICATOR3
	{TVFE_SYNCTOP_SFG_MUXCTRL1     , 0x812880d8,} ,// TVFE_SYNCTOP_SFG_MUXCTRL1
	{TVFE_SYNCTOP_SFG_MUXCTRL2     , 0x00334400,} ,// TVFE_SYNCTOP_SFG_MUXCTRL2
	{TVFE_SYNCTOP_INDICATOR4       , 0x00000000,} ,// TVFE_SYNCTOP_INDICATOR4
	{TVFE_SYNCTOP_SAM_MUXCTRL      , 0x00088000,} ,// TVFE_SYNCTOP_SAM_MUXCTRL
	{TVFE_MISC_WSS1_MUXCTRL1       , 0x00000000,} ,// TVFE_MISC_WSS1_MUXCTRL1
	{TVFE_MISC_WSS1_MUXCTRL2       , 0x00000000,} ,// TVFE_MISC_WSS1_MUXCTRL2
	{TVFE_MISC_WSS2_MUXCTRL1       , 0x00000000,} ,// TVFE_MISC_WSS2_MUXCTRL1
	{TVFE_MISC_WSS2_MUXCTRL2       , 0x00000000,} ,// TVFE_MISC_WSS2_MUXCTRL2
	{TVFE_MISC_WSS1_INDICATOR1     , 0x00000000,} ,// TVFE_MISC_WSS1_INDICATOR1
	{TVFE_MISC_WSS1_INDICATOR2     , 0x00000000,} ,// TVFE_MISC_WSS1_INDICATOR2
	{TVFE_MISC_WSS1_INDICATOR3     , 0x00000000,} ,// TVFE_MISC_WSS1_INDICATOR3
	{TVFE_MISC_WSS1_INDICATOR4     , 0x00000000,} ,// TVFE_MISC_WSS1_INDICATOR4
	{TVFE_MISC_WSS1_INDICATOR5     , 0x00000000,} ,// TVFE_MISC_WSS1_INDICATOR5
	{TVFE_MISC_WSS2_INDICATOR1     , 0x00000000,} ,// TVFE_MISC_WSS2_INDICATOR1
	{TVFE_MISC_WSS2_INDICATOR2     , 0x00000000,} ,// TVFE_MISC_WSS2_INDICATOR2
	{TVFE_MISC_WSS2_INDICATOR3     , 0x00000000,} ,// TVFE_MISC_WSS2_INDICATOR3
	{TVFE_MISC_WSS2_INDICATOR4     , 0x00000000,} ,// TVFE_MISC_WSS2_INDICATOR4
	{TVFE_MISC_WSS2_INDICATOR5     , 0x00000000,} ,// TVFE_MISC_WSS2_INDICATOR5
	{TVFE_AP_MUXCTRL1              , 0x00000000,} ,// TVFE_AP_MUXCTRL1
	{TVFE_AP_MUXCTRL2              , 0x00000000,} ,// TVFE_AP_MUXCTRL2
	{TVFE_AP_MUXCTRL3              , 0x00000000,} ,// TVFE_AP_MUXCTRL3
	{TVFE_AP_MUXCTRL4              , 0x00000000,} ,// TVFE_AP_MUXCTRL4
	{TVFE_AP_MUXCTRL5              , 0x00000000,} ,// TVFE_AP_MUXCTRL5
	{TVFE_AP_INDICATOR1            , 0x00000000,} ,// TVFE_AP_INDICATOR1
	{TVFE_AP_INDICATOR2            , 0x00000000,} ,// TVFE_AP_INDICATOR2
	{TVFE_AP_INDICATOR3            , 0x00000000,} ,// TVFE_AP_INDICATOR3
	{TVFE_AP_INDICATOR4            , 0x00000000,} ,// TVFE_AP_INDICATOR4
	{TVFE_AP_INDICATOR5            , 0x00000000,} ,// TVFE_AP_INDICATOR5
	{TVFE_AP_INDICATOR6            , 0x00000000,} ,// TVFE_AP_INDICATOR6
	{TVFE_AP_INDICATOR7            , 0x00000000,} ,// TVFE_AP_INDICATOR7
	{TVFE_AP_INDICATOR8            , 0x00000000,} ,// TVFE_AP_INDICATOR8
	{TVFE_AP_INDICATOR9            , 0x00000000,} ,// TVFE_AP_INDICATOR9
	{TVFE_AP_INDICATOR10           , 0x00000000,} ,// TVFE_AP_INDICATOR10
	{TVFE_AP_INDICATOR11           , 0x00000000,} ,// TVFE_AP_INDICATOR11
	{TVFE_AP_INDICATOR12           , 0x00000000,} ,// TVFE_AP_INDICATOR12
	{TVFE_AP_INDICATOR13           , 0x00000000,} ,// TVFE_AP_INDICATOR13
	{TVFE_AP_INDICATOR14           , 0x00000000,} ,// TVFE_AP_INDICATOR14
	{TVFE_AP_INDICATOR15           , 0x00000000,} ,// TVFE_AP_INDICATOR15
	{TVFE_AP_INDICATOR16           , 0x00000000,} ,// TVFE_AP_INDICATOR16
	{TVFE_AP_INDICATOR17           , 0x00000000,} ,// TVFE_AP_INDICATOR17
	{TVFE_AP_INDICATOR18           , 0x00000000,} ,// TVFE_AP_INDICATOR18
	{TVFE_AP_INDICATOR19           , 0x00000000,} ,// TVFE_AP_INDICATOR19
	{TVFE_BD_MUXCTRL1              , 0x00000000,} ,// TVFE_BD_MUXCTRL1
	{TVFE_BD_MUXCTRL2              , 0x00000000,} ,// TVFE_BD_MUXCTRL2
	{TVFE_BD_MUXCTRL3              , 0x00000000,} ,// TVFE_BD_MUXCTRL3
	{TVFE_BD_MUXCTRL4              , 0x00000000,} ,// TVFE_BD_MUXCTRL4
	{TVFE_CLP_MUXCTRL1             , 0x00000000,} ,// TVFE_CLP_MUXCTRL1
	{TVFE_CLP_MUXCTRL2             , 0x00000000,} ,// TVFE_CLP_MUXCTRL2
	{TVFE_CLP_MUXCTRL3             , 0x00000000,} ,// TVFE_CLP_MUXCTRL3
	{TVFE_CLP_MUXCTRL4             , 0x00000000,} ,// TVFE_CLP_MUXCTRL4
	{TVFE_CLP_INDICATOR1           , 0x00000000,} ,// TVFE_CLP_INDICATOR1
	{TVFE_BPG_BACKP_H              , 0x00000000,} ,// TVFE_BPG_BACKP_H
	{TVFE_BPG_BACKP_V              , 0x00000000,} ,// TVFE_BPG_BACKP_V
	{TVFE_DEG_H                    , 0x00621121,} ,// TVFE_DEG_H
	{TVFE_DEG_VODD                 , 0x002e8018,} ,// TVFE_DEG_VODD //zhuang
	{TVFE_DEG_VEVEN                , 0x002e8018,} ,// TVFE_DEG_VEVEN //zhuang
	{TVFE_OGO_OFFSET1              , 0x00000000,} ,// TVFE_OGO_OFFSET1
	{TVFE_OGO_GAIN1                , 0x00000000,} ,// TVFE_OGO_GAIN1
	{TVFE_OGO_GAIN2                , 0x00000000,} ,// TVFE_OGO_GAIN2
	{TVFE_OGO_OFFSET2              , 0x00000000,} ,// TVFE_OGO_OFFSET2
	{TVFE_OGO_OFFSET3              , 0x00000000,} ,// TVFE_OGO_OFFSET3
	{TVFE_VAFE_CTRL                , 0x00000201,} ,// TVFE_VAFE_CTRL //zhuang
	{TVFE_VAFE_STATUS              , 0x00000000,} ,// TVFE_VAFE_STATUS
	{TVFE_TOP_CTRL                 , 0x00008340,} ,// TVFE_TOP_CTRL
	{TVFE_CLAMP_INTF               , 0x00000000,} ,// TVFE_CLAMP_INTF
	{TVFE_RST_CTRL                 , 0x00000000,} ,// TVFE_RST_CTRL
	{TVFE_EXT_VIDEO_AFE_CTRL_MUX1  , 0x00000000,} ,// TVFE_EXT_VIDEO_AFE_CTRL_MUX1
	//{TVFE_AAFILTER_CTRL1           , 0x00082222,} ,// TVFE_AAFILTER_CTRL1
	//{TVFE_AAFILTER_CTRL2           , 0x252b39c6,} ,// TVFE_AAFILTER_CTRL2
	{TVFE_EDID_CONFIG              , TVAFE_EDID_CONFIG,} ,// TVFE_EDID_CONFIG
	{TVFE_EDID_RAM_ADDR            , 0x00000100,} ,// TVFE_EDID_RAM_ADDR
	{TVFE_EDID_RAM_WDATA           , 0x00000000,} ,// TVFE_EDID_RAM_WDATA
	{TVFE_EDID_RAM_RDATA           , 0x00000000,} ,// TVFE_EDID_RAM_RDATA
	{TVFE_APB_ERR_CTRL_MUX1        , 0x8fff8fff,} ,// TVFE_APB_ERR_CTRL_MUX1
	{TVFE_APB_ERR_CTRL_MUX2        , 0x00008fff,} ,// TVFE_APB_ERR_CTRL_MUX2
	{TVFE_APB_INDICATOR1           , 0x00000000,} ,// TVFE_APB_INDICATOR1
	{TVFE_APB_INDICATOR2           , 0x00000000,} ,// TVFE_APB_INDICATOR2
	{TVFE_ADC_READBACK_CTRL        , 0x00000000,} ,// TVFE_ADC_READBACK_CTRL
	{TVFE_ADC_READBACK_INDICATOR   , 0x00000000,} ,// TVFE_ADC_READBACK_INDICATOR
	{TVFE_INT_CLR                  , 0x00000000,} ,// TVFE_INT_CLR
	{TVFE_INT_MSKN                 , 0x00000000,} ,// TVFE_INT_MASKN
	{TVFE_INT_INDICATOR1           , 0x00000000,} ,// TVFE_INT_INDICATOR1
	{TVFE_INT_SET                  , 0x00000000,} ,// TVFE_INT_SET
	{TVFE_CHIP_VERSION             , 0x00000000,} ,// TVFE_CHIP_VERSION
	{TVFE_FREERUN_GEN_WIDTH        , 0x00080008,},//TVFE_FREERUN_GEN_WIDTH
	{TVFE_FREERUN_GEN_PRD          , 0x020d035a,},//TVFE_FREERUN_GEN_PRD
	{TVFE_FREERUN_GEN_COAST        , 0x00000000,},//TVFE_FREERUN_GEN_COAST
	{TVFE_FREERUN_GEN_CTRL         , 0x00000001,},//TVFE_FREERUN_GEN_CTRL
	{TVFE_AAFILTER_CTRL1            ,   0x00100000,},//TVFE_AAFILTER_CTRL1 bypass all
	{TVFE_AAFILTER_CTRL2            ,   0x00000000,},//TVFE_AAFILTER_CTRL2
	{TVFE_AAFILTER_CTRL3            ,   0x00000000,},//TVFE_AAFILTER_CTRL3
	{TVFE_AAFILTER_CTRL4            ,   0x00000000,},//TVFE_AAFILTER_CTRL4
	{TVFE_AAFILTER_CTRL5            ,   0x00000000,},//TVFE_AAFILTER_CTRL5
	{TVFE_SOG_MON_CTRL1            ,   0x8000c804,},//TVFE_SOG_MON_CTRL1
	{TVFE_ADC_READBACK_CTRL1,    0x00000000,},//TVFE_ADC_READBACK_CTRL1
	{TVFE_ADC_READBACK_CTRL2,    0x00000000,},//TVFE_ADC_READBACK_CTRL2
	{TVFE_AFC_CTRL1                      ,    0x00000000,},//TVFE_AFC_CTRL1
	{TVFE_AFC_CTRL2                      ,    0x00000000,},//TVFE_AFC_CTRL2
	{TVFE_AFC_CTRL3                      ,    0x00000000,},//TVFE_AFC_CTRL3
	{TVFE_AFC_CTRL4                      ,    0x00000000,},//TVFE_AFC_CTRL4
	{TVFE_AFC_CTRL5                      ,    0x00000000,},//TVFE_AFC_CTRL5
	{0xFFFFFFFF                    , 0x00000000,}
};


const static unsigned int cvbs_atv_top_reg_default[][2] ={
	{TVFE_DVSS_MUXCTRL                      ,0x07000008/*0x00000000*/,}, // TVFE_DVSS_MUXCTRL
	{TVFE_DVSS_MUXVS_REF                    ,0x00000000,}, // TVFE_DVSS_MUXVS_REF
	{TVFE_DVSS_MUXCOAST_V                   ,0x00000000,}, // TVFE_DVSS_MUXCOAST_V
	{TVFE_DVSS_SEP_HVWIDTH                  ,0x00000000,}, // TVFE_DVSS_SEP_HVWIDTH
	{TVFE_DVSS_SEP_HPARA                    ,0x00000000,}, // TVFE_DVSS_SEP_HPARA
	{TVFE_DVSS_SEP_VINTEG                   ,0x00000000,}, // TVFE_DVSS_SEP_VINTEG
	{TVFE_DVSS_SEP_H_THR                    ,0x00000000,}, // TVFE_DVSS_SEP_H_THR
	{TVFE_DVSS_SEP_CTRL                     ,0x00000000,}, // TVFE_DVSS_SEP_CTRL
	{TVFE_DVSS_GEN_WIDTH                    ,0x00000000,}, // TVFE_DVSS_GEN_WIDTH
	{TVFE_DVSS_GEN_PRD                      ,0x00000000,}, // TVFE_DVSS_GEN_PRD
	{TVFE_DVSS_GEN_COAST                    ,0x00000000,}, // TVFE_DVSS_GEN_COAST
	{TVFE_DVSS_NOSIG_PARA                   ,0x00000000,}, // TVFE_DVSS_NOSIG_PARA
	{TVFE_DVSS_NOSIG_PLS_TH                 ,0x00000000,}, // TVFE_DVSS_NOSIG_PLS_TH
	{TVFE_DVSS_GATE_H                       ,0x00000000,}, // TVFE_DVSS_GATE_H
	{TVFE_DVSS_GATE_V                       ,0x00000000,}, // TVFE_DVSS_GATE_V
	{TVFE_DVSS_INDICATOR1                   ,0x00000000,}, // TVFE_DVSS_INDICATOR1
	{TVFE_DVSS_INDICATOR2                   ,0x00000000,}, // TVFE_DVSS_INDICATOR2
	{TVFE_DVSS_MVDET_CTRL1                  ,0x00000000,}, // TVFE_DVSS_MVDET_CTRL1
	{TVFE_DVSS_MVDET_CTRL2                  ,0x00000000,}, // TVFE_DVSS_MVDET_CTRL2
	{TVFE_DVSS_MVDET_CTRL3                  ,0x00000000,}, // TVFE_DVSS_MVDET_CTRL3
	{TVFE_DVSS_MVDET_CTRL4                  ,0x00000000,}, // TVFE_DVSS_MVDET_CTRL4
	{TVFE_DVSS_MVDET_CTRL5                  ,0x00000000,}, // TVFE_DVSS_MVDET_CTRL5
	{TVFE_DVSS_MVDET_CTRL6                  ,0x00000000,}, // TVFE_DVSS_MVDET_CTRL6
	{TVFE_DVSS_MVDET_CTRL7                  ,0x00000000,}, // TVFE_DVSS_MVDET_CTRL7
	{TVFE_SYNCTOP_SPOL_MUXCTRL              ,0x00000009,}, // TVFE_SYNCTOP_SPOL_MUXCTRL
	{TVFE_SYNCTOP_INDICATOR1_HCNT           ,0x00000000,}, // TVFE_SYNCTOP_INDICATOR1_HCNT
	{TVFE_SYNCTOP_INDICATOR2_VCNT           ,0x00000000,}, // TVFE_SYNCTOP_INDICATOR2_VCNT
	{TVFE_SYNCTOP_INDICATOR3                ,0x00000000,}, // TVFE_SYNCTOP_INDICATOR3
	{TVFE_SYNCTOP_SFG_MUXCTRL1              ,0x00000000,}, // TVFE_SYNCTOP_SFG_MUXCTRL1
	{TVFE_SYNCTOP_SFG_MUXCTRL2              ,0x00330000,}, // TVFE_SYNCTOP_SFG_MUXCTRL2
	{TVFE_SYNCTOP_INDICATOR4                ,0x00000000,}, // TVFE_SYNCTOP_INDICATOR4
	{TVFE_SYNCTOP_SAM_MUXCTRL               ,0x00082001,}, // TVFE_SYNCTOP_SAM_MUXCTRL
	{TVFE_MISC_WSS1_MUXCTRL1                ,0x00000000,}, // TVFE_MISC_WSS1_MUXCTRL1
	{TVFE_MISC_WSS1_MUXCTRL2                ,0x00000000,}, // TVFE_MISC_WSS1_MUXCTRL2
	{TVFE_MISC_WSS2_MUXCTRL1                ,0x00000000,}, // TVFE_MISC_WSS2_MUXCTRL1
	{TVFE_MISC_WSS2_MUXCTRL2                ,0x00000000,}, // TVFE_MISC_WSS2_MUXCTRL2
	{TVFE_MISC_WSS1_INDICATOR1              ,0x00000000,}, // TVFE_MISC_WSS1_INDICATOR1
	{TVFE_MISC_WSS1_INDICATOR2              ,0x00000000,}, // TVFE_MISC_WSS1_INDICATOR2
	{TVFE_MISC_WSS1_INDICATOR3              ,0x00000000,}, // TVFE_MISC_WSS1_INDICATOR3
	{TVFE_MISC_WSS1_INDICATOR4              ,0x00000000,}, // TVFE_MISC_WSS1_INDICATOR4
	{TVFE_MISC_WSS1_INDICATOR5              ,0x00000000,}, // TVFE_MISC_WSS1_INDICATOR5
	{TVFE_MISC_WSS2_INDICATOR1              ,0x00000000,}, // TVFE_MISC_WSS2_INDICATOR1
	{TVFE_MISC_WSS2_INDICATOR2              ,0x00000000,}, // TVFE_MISC_WSS2_INDICATOR2
	{TVFE_MISC_WSS2_INDICATOR3              ,0x00000000,}, // TVFE_MISC_WSS2_INDICATOR3
	{TVFE_MISC_WSS2_INDICATOR4              ,0x00000000,}, // TVFE_MISC_WSS2_INDICATOR4
	{TVFE_MISC_WSS2_INDICATOR5              ,0x00000000,}, // TVFE_MISC_WSS2_INDICATOR5
	{TVFE_AP_MUXCTRL1                       ,0x00000000,}, // TVFE_AP_MUXCTRL1
	{TVFE_AP_MUXCTRL2                       ,0x00000000,}, // TVFE_AP_MUXCTRL2
	{TVFE_AP_MUXCTRL3                       ,0x00000000,}, // TVFE_AP_MUXCTRL3
	{TVFE_AP_MUXCTRL4                       ,0x00000000,}, // TVFE_AP_MUXCTRL4
	{TVFE_AP_MUXCTRL5                       ,0x00000000,}, // TVFE_AP_MUXCTRL5
	{TVFE_AP_INDICATOR1                     ,0x00000000,}, // TVFE_AP_INDICATOR1
	{TVFE_AP_INDICATOR2                     ,0x00000000,}, // TVFE_AP_INDICATOR2
	{TVFE_AP_INDICATOR3                     ,0x00000000,}, // TVFE_AP_INDICATOR3
	{TVFE_AP_INDICATOR4                     ,0x00000000,}, // TVFE_AP_INDICATOR4
	{TVFE_AP_INDICATOR5                     ,0x00000000,}, // TVFE_AP_INDICATOR5
	{TVFE_AP_INDICATOR6                     ,0x00000000,}, // TVFE_AP_INDICATOR6
	{TVFE_AP_INDICATOR7                     ,0x00000000,}, // TVFE_AP_INDICATOR7
	{TVFE_AP_INDICATOR8                     ,0x00000000,}, // TVFE_AP_INDICATOR8
	{TVFE_AP_INDICATOR9                     ,0x00000000,}, // TVFE_AP_INDICATOR9
	{TVFE_AP_INDICATOR10                    ,0x00000000,}, // TVFE_AP_INDICATOR10
	{TVFE_AP_INDICATOR11                    ,0x00000000,}, // TVFE_AP_INDICATOR11
	{TVFE_AP_INDICATOR12                    ,0x00000000,}, // TVFE_AP_INDICATOR12
	{TVFE_AP_INDICATOR13                    ,0x00000000,}, // TVFE_AP_INDICATOR13
	{TVFE_AP_INDICATOR14                    ,0x00000000,}, // TVFE_AP_INDICATOR14
	{TVFE_AP_INDICATOR15                    ,0x00000000,}, // TVFE_AP_INDICATOR15
	{TVFE_AP_INDICATOR16                    ,0x00000000,}, // TVFE_AP_INDICATOR16
	{TVFE_AP_INDICATOR17                    ,0x00000000,}, // TVFE_AP_INDICATOR17
	{TVFE_AP_INDICATOR18                    ,0x00000000,}, // TVFE_AP_INDICATOR18
	{TVFE_AP_INDICATOR19                    ,0x00000000,}, // TVFE_AP_INDICATOR19
	{TVFE_BD_MUXCTRL1                       ,0x00000000,}, // TVFE_BD_MUXCTRL1
	{TVFE_BD_MUXCTRL2                       ,0x00000000,}, // TVFE_BD_MUXCTRL2
	{TVFE_BD_MUXCTRL3                       ,0x00000000,}, // TVFE_BD_MUXCTRL3
	{TVFE_BD_MUXCTRL4                       ,0x00000000,}, // TVFE_BD_MUXCTRL4
	{TVFE_CLP_MUXCTRL1                      ,0x00000000,}, // TVFE_CLP_MUXCTRL1
	{TVFE_CLP_MUXCTRL2                      ,0x00000000,}, // TVFE_CLP_MUXCTRL2
	{TVFE_CLP_MUXCTRL3                      ,0x00000000,}, // TVFE_CLP_MUXCTRL3
	{TVFE_CLP_MUXCTRL4                      ,0x00000000,}, // TVFE_CLP_MUXCTRL4
	{TVFE_CLP_INDICATOR1                    ,0x00000000,}, // TVFE_CLP_INDICATOR1
	{TVFE_BPG_BACKP_H                       ,0x00000000,}, // TVFE_BPG_BACKP_H
	{TVFE_BPG_BACKP_V                       ,0x00000000,}, // TVFE_BPG_BACKP_V
	{TVFE_DEG_H                             ,0x00000000,}, // TVFE_DEG_H
	{TVFE_DEG_VODD                          ,0x00000000,}, // TVFE_DEG_VODD
	{TVFE_DEG_VEVEN                         ,0x00000000,}, // TVFE_DEG_VEVEN
	{TVFE_OGO_OFFSET1                       ,0x00000000,}, // TVFE_OGO_OFFSET1
	{TVFE_OGO_GAIN1                         ,0x00000000,}, // TVFE_OGO_GAIN1
	{TVFE_OGO_GAIN2                         ,0x00000000,}, // TVFE_OGO_GAIN2
	{TVFE_OGO_OFFSET2                       ,0x00000000,}, // TVFE_OGO_OFFSET2
	{TVFE_OGO_OFFSET3                       ,0x00000000,}, // TVFE_OGO_OFFSET3
	{TVFE_VAFE_CTRL                         ,0x00000000,}, // TVFE_VAFE_CTRL
	{TVFE_VAFE_STATUS                       ,0x00000000,}, // TVFE_VAFE_STATUS
#ifdef CRYSTAL_25M
	{TVFE_TOP_CTRL                          ,0x30c4e6c/*0xc4f64 0x00004B60*/,}, // TVFE_TOP_CTRL
#else//24M
#if 0   //(defined(CONFIG_ADC_DOUBLE_SAMPLING_FOR_CVBS) && defined(CRYSTAL_24M) && (MESON_CPU_TYPE >= MESON_CPU_TYPE_MESONG9TV))
	{TVFE_TOP_CTRL				    ,0x010c4d6c/*0xc4f64 0x00004B60*/,}, // TVFE_TOP_CTRL
#else
	{TVFE_TOP_CTRL                          ,0x30c4f64/*0xc4f64 0x00004B60*/,}, // TVFE_TOP_CTRL
#endif
#endif
	{TVFE_CLAMP_INTF                        ,0x00008666,}, // TVFE_CLAMP_INTF
	{TVFE_RST_CTRL                          ,0x00000000,}, // TVFE_RST_CTRL
	{TVFE_EXT_VIDEO_AFE_CTRL_MUX1           ,0x00000000,}, // TVFE_EXT_VIDEO_AFE_CTRL_MUX1
	{TVFE_EDID_CONFIG                       ,TVAFE_EDID_CONFIG,}, // TVFE_EDID_CONFIG
	{TVFE_EDID_RAM_ADDR                     ,0x00000000,}, // TVFE_EDID_RAM_ADDR
	{TVFE_EDID_RAM_WDATA                    ,0x00000000,}, // TVFE_EDID_RAM_WDATA
	{TVFE_EDID_RAM_RDATA                    ,0x00000000,}, // TVFE_EDID_RAM_RDATA
	{TVFE_APB_ERR_CTRL_MUX1                 ,0x8fff8fff,}, // TVFE_APB_ERR_CTRL_MUX1
	{TVFE_APB_ERR_CTRL_MUX2                 ,0x00008fff,}, // TVFE_APB_ERR_CTRL_MUX2
	{TVFE_APB_INDICATOR1                    ,0x00000000,}, // TVFE_APB_INDICATOR1
	{TVFE_APB_INDICATOR2                    ,0x00000000,}, // TVFE_APB_INDICATOR2
	{TVFE_ADC_READBACK_CTRL                 ,0x80140003,}, // TVFE_ADC_READBACK_CTRL
	{TVFE_ADC_READBACK_INDICATOR            ,0x00000000,}, // TVFE_ADC_READBACK_INDICATOR
#if (MESON_CPU_TYPE >= MESON_CPU_TYPE_MESONG9TV)
	{TVFE_VAFE_CTRL0			,0x00090b00,}, // TVFE_VAFE_CTRL0
	{TVFE_VAFE_CTRL1			,0x00000110,}, // TVFE_VAFE_CTRL1
	{TVFE_VAFE_CTRL2			,0x0010ef93,}, // TVFE_VAFE_CTRL2
#endif
	{TVFE_INT_CLR                           ,0x00000000,}, // TVFE_INT_CLR
	{TVFE_INT_MSKN                          ,0x00000000,}, // TVFE_INT_MASKN
	{TVFE_INT_INDICATOR1                    ,0x00000000,}, // TVFE_INT_INDICATOR1
	{TVFE_INT_SET                           ,0x00000000,}, // TVFE_INT_SET
	//{TVFE_CHIP_VERSION                      ,0x00000000,}, // TVFE_CHIP_VERSION
	{TVFE_FREERUN_GEN_WIDTH   , 0x00000000,},//TVFE_FREERUN_GEN_WIDTH
	{TVFE_FREERUN_GEN_PRD       ,  0x00000000,},//TVFE_FREERUN_GEN_PRD
	{TVFE_FREERUN_GEN_COAST   , 0x00000000,},//TVFE_FREERUN_GEN_COAST
	{TVFE_FREERUN_GEN_CTRL      , 0x00000000,},//TVFE_FREERUN_GEN_CTRL
#if (MESON_CPU_TYPE >= MESON_CPU_TYPE_MESONG9TV)
#if 0//(defined(CONFIG_ADC_DOUBLE_SAMPLING_FOR_CVBS) && defined(CRYSTAL_24M) && (MESON_CPU_TYPE >= MESON_CPU_TYPE_MESONG9TV))
	{TVFE_AAFILTER_CTRL1		,	0x00012721,},//0x00082222 TVFE_AAFILTER_CTRL1 bypass all
	{TVFE_AAFILTER_CTRL2		,	0x1304fcfa,},//TVFE_AAFILTER_CTRL2
#else
	{TVFE_AAFILTER_CTRL1		,   0x00182222,},//0x00082222 TVFE_AAFILTER_CTRL1 bypass all
	{TVFE_AAFILTER_CTRL2		,   0x252b39c6,},//TVFE_AAFILTER_CTRL2
#endif
#else
	{TVFE_AAFILTER_CTRL1            ,   0x00100000,},//TVFE_AAFILTER_CTRL1 bypass all
	{TVFE_AAFILTER_CTRL2            ,   0x00000000,},//TVFE_AAFILTER_CTRL2
	{TVFE_AAFILTER_CTRL3            ,   0x00000000,},//TVFE_AAFILTER_CTRL3
	{TVFE_AAFILTER_CTRL4            ,   0x00000000,},//TVFE_AAFILTER_CTRL4
	{TVFE_AAFILTER_CTRL5            ,   0x00000000,},//TVFE_AAFILTER_CTRL5
#endif
	{TVFE_SOG_MON_CTRL1            ,   0x00000000,},//TVFE_SOG_MON_CTRL1
	{TVFE_ADC_READBACK_CTRL1,   0x00000000,},//TVFE_ADC_READBACK_CTRL1
	{TVFE_ADC_READBACK_CTRL2,   0x00000000,},//TVFE_ADC_READBACK_CTRL2
#ifdef CRYSTAL_25M
	{TVFE_AFC_CTRL1 		     ,	 0x85730459,},//TVFE_AFC_CTRL1
	{TVFE_AFC_CTRL2 		     ,	 0x342fa9ed,},//TVFE_AFC_CTRL2
	{TVFE_AFC_CTRL3 		     ,	 0x2a02396,},//TVFE_AFC_CTRL3
	{TVFE_AFC_CTRL4 		     ,	 0xfefbff14,},//TVFE_AFC_CTRL4
	{TVFE_AFC_CTRL5 		     ,		0x0,},//TVFE_AFC_CTRL5
#else//for 24M
#if 0//(defined(CONFIG_ADC_DOUBLE_SAMPLING_FOR_CVBS) && defined(CRYSTAL_24M) && (MESON_CPU_TYPE >= MESON_CPU_TYPE_MESONG9TV))
	{TVFE_AFC_CTRL1 			 ,	 0x893904d2,},//TVFE_AFC_CTRL1
	{TVFE_AFC_CTRL2 			 ,	 0x0f4b9ac9,},//TVFE_AFC_CTRL2
	{TVFE_AFC_CTRL3 			 ,	 0x01fd8c36,},//TVFE_AFC_CTRL3
	{TVFE_AFC_CTRL4 			 ,	 0x2de6d04f,},//TVFE_AFC_CTRL4
	{TVFE_AFC_CTRL5 			 ,	 0x00000004,},//TVFE_AFC_CTRL5
#else
#if (MESON_CPU_TYPE >= MESON_CPU_TYPE_MESONG9TV)
	{TVFE_AFC_CTRL1                      ,   0x05730459,},//TVFE_AFC_CTRL1
#else
    {TVFE_AFC_CTRL1                      ,   0x893904d2,},//TVFE_AFC_CTRL1
#endif
	{TVFE_AFC_CTRL2                      ,   0xf4b9ac9,},//TVFE_AFC_CTRL2
	{TVFE_AFC_CTRL3                      ,   0x1fd8c36,},//TVFE_AFC_CTRL3
	{TVFE_AFC_CTRL4                      ,   0x2de6d04f,},//TVFE_AFC_CTRL4
	{TVFE_AFC_CTRL5                      ,          0x4,},//TVFE_AFC_CTRL5
#endif
#endif
	{0xFFFFFFFF                             ,0x00000000,}
};

	

/* TOP */
const static unsigned int cvbs_top_reg_default[][2] = {
	{TVFE_DVSS_MUXCTRL                      ,0x07000008/*0x00000000*/,}, // TVFE_DVSS_MUXCTRL
	{TVFE_DVSS_MUXVS_REF                    ,0x00000000,}, // TVFE_DVSS_MUXVS_REF
	{TVFE_DVSS_MUXCOAST_V                   ,0x00000000,}, // TVFE_DVSS_MUXCOAST_V
	{TVFE_DVSS_SEP_HVWIDTH                  ,0x00000000,}, // TVFE_DVSS_SEP_HVWIDTH
	{TVFE_DVSS_SEP_HPARA                    ,0x00000000,}, // TVFE_DVSS_SEP_HPARA
	{TVFE_DVSS_SEP_VINTEG                   ,0x00000000,}, // TVFE_DVSS_SEP_VINTEG
	{TVFE_DVSS_SEP_H_THR                    ,0x00000000,}, // TVFE_DVSS_SEP_H_THR
	{TVFE_DVSS_SEP_CTRL                     ,0x00000000,}, // TVFE_DVSS_SEP_CTRL
	{TVFE_DVSS_GEN_WIDTH                    ,0x00000000,}, // TVFE_DVSS_GEN_WIDTH
	{TVFE_DVSS_GEN_PRD                      ,0x00000000,}, // TVFE_DVSS_GEN_PRD
	{TVFE_DVSS_GEN_COAST                    ,0x00000000,}, // TVFE_DVSS_GEN_COAST
	{TVFE_DVSS_NOSIG_PARA                   ,0x00000000,}, // TVFE_DVSS_NOSIG_PARA
	{TVFE_DVSS_NOSIG_PLS_TH                 ,0x00000000,}, // TVFE_DVSS_NOSIG_PLS_TH
	{TVFE_DVSS_GATE_H                       ,0x00000000,}, // TVFE_DVSS_GATE_H
	{TVFE_DVSS_GATE_V                       ,0x00000000,}, // TVFE_DVSS_GATE_V
	{TVFE_DVSS_INDICATOR1                   ,0x00000000,}, // TVFE_DVSS_INDICATOR1
	{TVFE_DVSS_INDICATOR2                   ,0x00000000,}, // TVFE_DVSS_INDICATOR2
	{TVFE_DVSS_MVDET_CTRL1                  ,0x00000000,}, // TVFE_DVSS_MVDET_CTRL1
	{TVFE_DVSS_MVDET_CTRL2                  ,0x00000000,}, // TVFE_DVSS_MVDET_CTRL2
	{TVFE_DVSS_MVDET_CTRL3                  ,0x00000000,}, // TVFE_DVSS_MVDET_CTRL3
	{TVFE_DVSS_MVDET_CTRL4                  ,0x00000000,}, // TVFE_DVSS_MVDET_CTRL4
	{TVFE_DVSS_MVDET_CTRL5                  ,0x00000000,}, // TVFE_DVSS_MVDET_CTRL5
	{TVFE_DVSS_MVDET_CTRL6                  ,0x00000000,}, // TVFE_DVSS_MVDET_CTRL6
	{TVFE_DVSS_MVDET_CTRL7                  ,0x00000000,}, // TVFE_DVSS_MVDET_CTRL7
	{TVFE_SYNCTOP_SPOL_MUXCTRL              ,0x00000009,}, // TVFE_SYNCTOP_SPOL_MUXCTRL
	{TVFE_SYNCTOP_INDICATOR1_HCNT           ,0x00000000,}, // TVFE_SYNCTOP_INDICATOR1_HCNT
	{TVFE_SYNCTOP_INDICATOR2_VCNT           ,0x00000000,}, // TVFE_SYNCTOP_INDICATOR2_VCNT
	{TVFE_SYNCTOP_INDICATOR3                ,0x00000000,}, // TVFE_SYNCTOP_INDICATOR3
	{TVFE_SYNCTOP_SFG_MUXCTRL1              ,0x00000000,}, // TVFE_SYNCTOP_SFG_MUXCTRL1
	{TVFE_SYNCTOP_SFG_MUXCTRL2              ,0x00330000,}, // TVFE_SYNCTOP_SFG_MUXCTRL2
	{TVFE_SYNCTOP_INDICATOR4                ,0x00000000,}, // TVFE_SYNCTOP_INDICATOR4
	{TVFE_SYNCTOP_SAM_MUXCTRL               ,0x00082001,}, // TVFE_SYNCTOP_SAM_MUXCTRL
	{TVFE_MISC_WSS1_MUXCTRL1                ,0x00000000,}, // TVFE_MISC_WSS1_MUXCTRL1
	{TVFE_MISC_WSS1_MUXCTRL2                ,0x00000000,}, // TVFE_MISC_WSS1_MUXCTRL2
	{TVFE_MISC_WSS2_MUXCTRL1                ,0x00000000,}, // TVFE_MISC_WSS2_MUXCTRL1
	{TVFE_MISC_WSS2_MUXCTRL2                ,0x00000000,}, // TVFE_MISC_WSS2_MUXCTRL2
	{TVFE_MISC_WSS1_INDICATOR1              ,0x00000000,}, // TVFE_MISC_WSS1_INDICATOR1
	{TVFE_MISC_WSS1_INDICATOR2              ,0x00000000,}, // TVFE_MISC_WSS1_INDICATOR2
	{TVFE_MISC_WSS1_INDICATOR3              ,0x00000000,}, // TVFE_MISC_WSS1_INDICATOR3
	{TVFE_MISC_WSS1_INDICATOR4              ,0x00000000,}, // TVFE_MISC_WSS1_INDICATOR4
	{TVFE_MISC_WSS1_INDICATOR5              ,0x00000000,}, // TVFE_MISC_WSS1_INDICATOR5
	{TVFE_MISC_WSS2_INDICATOR1              ,0x00000000,}, // TVFE_MISC_WSS2_INDICATOR1
	{TVFE_MISC_WSS2_INDICATOR2              ,0x00000000,}, // TVFE_MISC_WSS2_INDICATOR2
	{TVFE_MISC_WSS2_INDICATOR3              ,0x00000000,}, // TVFE_MISC_WSS2_INDICATOR3
	{TVFE_MISC_WSS2_INDICATOR4              ,0x00000000,}, // TVFE_MISC_WSS2_INDICATOR4
	{TVFE_MISC_WSS2_INDICATOR5              ,0x00000000,}, // TVFE_MISC_WSS2_INDICATOR5
	{TVFE_AP_MUXCTRL1                       ,0x00000000,}, // TVFE_AP_MUXCTRL1
	{TVFE_AP_MUXCTRL2                       ,0x00000000,}, // TVFE_AP_MUXCTRL2
	{TVFE_AP_MUXCTRL3                       ,0x00000000,}, // TVFE_AP_MUXCTRL3
	{TVFE_AP_MUXCTRL4                       ,0x00000000,}, // TVFE_AP_MUXCTRL4
	{TVFE_AP_MUXCTRL5                       ,0x00000000,}, // TVFE_AP_MUXCTRL5
	{TVFE_AP_INDICATOR1                     ,0x00000000,}, // TVFE_AP_INDICATOR1
	{TVFE_AP_INDICATOR2                     ,0x00000000,}, // TVFE_AP_INDICATOR2
	{TVFE_AP_INDICATOR3                     ,0x00000000,}, // TVFE_AP_INDICATOR3
	{TVFE_AP_INDICATOR4                     ,0x00000000,}, // TVFE_AP_INDICATOR4
	{TVFE_AP_INDICATOR5                     ,0x00000000,}, // TVFE_AP_INDICATOR5
	{TVFE_AP_INDICATOR6                     ,0x00000000,}, // TVFE_AP_INDICATOR6
	{TVFE_AP_INDICATOR7                     ,0x00000000,}, // TVFE_AP_INDICATOR7
	{TVFE_AP_INDICATOR8                     ,0x00000000,}, // TVFE_AP_INDICATOR8
	{TVFE_AP_INDICATOR9                     ,0x00000000,}, // TVFE_AP_INDICATOR9
	{TVFE_AP_INDICATOR10                    ,0x00000000,}, // TVFE_AP_INDICATOR10
	{TVFE_AP_INDICATOR11                    ,0x00000000,}, // TVFE_AP_INDICATOR11
	{TVFE_AP_INDICATOR12                    ,0x00000000,}, // TVFE_AP_INDICATOR12
	{TVFE_AP_INDICATOR13                    ,0x00000000,}, // TVFE_AP_INDICATOR13
	{TVFE_AP_INDICATOR14                    ,0x00000000,}, // TVFE_AP_INDICATOR14
	{TVFE_AP_INDICATOR15                    ,0x00000000,}, // TVFE_AP_INDICATOR15
	{TVFE_AP_INDICATOR16                    ,0x00000000,}, // TVFE_AP_INDICATOR16
	{TVFE_AP_INDICATOR17                    ,0x00000000,}, // TVFE_AP_INDICATOR17
	{TVFE_AP_INDICATOR18                    ,0x00000000,}, // TVFE_AP_INDICATOR18
	{TVFE_AP_INDICATOR19                    ,0x00000000,}, // TVFE_AP_INDICATOR19
	{TVFE_BD_MUXCTRL1                       ,0x00000000,}, // TVFE_BD_MUXCTRL1
	{TVFE_BD_MUXCTRL2                       ,0x00000000,}, // TVFE_BD_MUXCTRL2
	{TVFE_BD_MUXCTRL3                       ,0x00000000,}, // TVFE_BD_MUXCTRL3
	{TVFE_BD_MUXCTRL4                       ,0x00000000,}, // TVFE_BD_MUXCTRL4
	{TVFE_CLP_MUXCTRL1                      ,0x00000000,}, // TVFE_CLP_MUXCTRL1
	{TVFE_CLP_MUXCTRL2                      ,0x00000000,}, // TVFE_CLP_MUXCTRL2
	{TVFE_CLP_MUXCTRL3                      ,0x00000000,}, // TVFE_CLP_MUXCTRL3
	{TVFE_CLP_MUXCTRL4                      ,0x00000000,}, // TVFE_CLP_MUXCTRL4
	{TVFE_CLP_INDICATOR1                    ,0x00000000,}, // TVFE_CLP_INDICATOR1
	{TVFE_BPG_BACKP_H                       ,0x00000000,}, // TVFE_BPG_BACKP_H
	{TVFE_BPG_BACKP_V                       ,0x00000000,}, // TVFE_BPG_BACKP_V
	{TVFE_DEG_H                             ,0x00000000,}, // TVFE_DEG_H
	{TVFE_DEG_VODD                          ,0x00000000,}, // TVFE_DEG_VODD
	{TVFE_DEG_VEVEN                         ,0x00000000,}, // TVFE_DEG_VEVEN
	{TVFE_OGO_OFFSET1                       ,0x00000000,}, // TVFE_OGO_OFFSET1
	{TVFE_OGO_GAIN1                         ,0x00000000,}, // TVFE_OGO_GAIN1
	{TVFE_OGO_GAIN2                         ,0x00000000,}, // TVFE_OGO_GAIN2
	{TVFE_OGO_OFFSET2                       ,0x00000000,}, // TVFE_OGO_OFFSET2
	{TVFE_OGO_OFFSET3                       ,0x00000000,}, // TVFE_OGO_OFFSET3
	{TVFE_VAFE_CTRL                         ,0x00000000,}, // TVFE_VAFE_CTRL
	{TVFE_VAFE_STATUS                       ,0x00000000,}, // TVFE_VAFE_STATUS
#ifdef CRYSTAL_25M
	{TVFE_TOP_CTRL                          ,0x30c4e6c/*0xc4f64 0x00004B60*/,}, // TVFE_TOP_CTRL
#else//24M
#if (defined(CONFIG_ADC_DOUBLE_SAMPLING_FOR_CVBS) && defined(CRYSTAL_24M) && (MESON_CPU_TYPE >= MESON_CPU_TYPE_MESONG9TV))
	{TVFE_TOP_CTRL				    ,0x010c4d6c/*0xc4f64 0x00004B60*/,}, // TVFE_TOP_CTRL
#else
	{TVFE_TOP_CTRL                          ,0x30c4f64/*0xc4f64 0x00004B60*/,}, // TVFE_TOP_CTRL
#endif
#endif
	{TVFE_CLAMP_INTF                        ,0x00008666,}, // TVFE_CLAMP_INTF
	{TVFE_RST_CTRL                          ,0x00000000,}, // TVFE_RST_CTRL
	{TVFE_EXT_VIDEO_AFE_CTRL_MUX1           ,0x00000000,}, // TVFE_EXT_VIDEO_AFE_CTRL_MUX1
	{TVFE_EDID_CONFIG                       ,TVAFE_EDID_CONFIG,}, // TVFE_EDID_CONFIG
	{TVFE_EDID_RAM_ADDR                     ,0x00000000,}, // TVFE_EDID_RAM_ADDR
	{TVFE_EDID_RAM_WDATA                    ,0x00000000,}, // TVFE_EDID_RAM_WDATA
	{TVFE_EDID_RAM_RDATA                    ,0x00000000,}, // TVFE_EDID_RAM_RDATA
	{TVFE_APB_ERR_CTRL_MUX1                 ,0x8fff8fff,}, // TVFE_APB_ERR_CTRL_MUX1
	{TVFE_APB_ERR_CTRL_MUX2                 ,0x00008fff,}, // TVFE_APB_ERR_CTRL_MUX2
	{TVFE_APB_INDICATOR1                    ,0x00000000,}, // TVFE_APB_INDICATOR1
	{TVFE_APB_INDICATOR2                    ,0x00000000,}, // TVFE_APB_INDICATOR2
	{TVFE_ADC_READBACK_CTRL                 ,0x80140003,}, // TVFE_ADC_READBACK_CTRL
	{TVFE_ADC_READBACK_INDICATOR            ,0x00000000,}, // TVFE_ADC_READBACK_INDICATOR
#if (MESON_CPU_TYPE >= MESON_CPU_TYPE_MESONG9TV)
	{TVFE_VAFE_CTRL0			,0x00090b00,}, // TVFE_VAFE_CTRL0
	{TVFE_VAFE_CTRL1			,0x00000110,}, // TVFE_VAFE_CTRL1
	{TVFE_VAFE_CTRL2			,0x0010ef93,}, // TVFE_VAFE_CTRL2
#endif
	{TVFE_INT_CLR                           ,0x00000000,}, // TVFE_INT_CLR
	{TVFE_INT_MSKN                          ,0x00000000,}, // TVFE_INT_MASKN
	{TVFE_INT_INDICATOR1                    ,0x00000000,}, // TVFE_INT_INDICATOR1
	{TVFE_INT_SET                           ,0x00000000,}, // TVFE_INT_SET
	//{TVFE_CHIP_VERSION                      ,0x00000000,}, // TVFE_CHIP_VERSION
	{TVFE_FREERUN_GEN_WIDTH   , 0x00000000,},//TVFE_FREERUN_GEN_WIDTH
	{TVFE_FREERUN_GEN_PRD       ,  0x00000000,},//TVFE_FREERUN_GEN_PRD
	{TVFE_FREERUN_GEN_COAST   , 0x00000000,},//TVFE_FREERUN_GEN_COAST
	{TVFE_FREERUN_GEN_CTRL      , 0x00000000,},//TVFE_FREERUN_GEN_CTRL
#if (MESON_CPU_TYPE >= MESON_CPU_TYPE_MESONG9TV)
#if (defined(CONFIG_ADC_DOUBLE_SAMPLING_FOR_CVBS) && defined(CRYSTAL_24M) && (MESON_CPU_TYPE >= MESON_CPU_TYPE_MESONG9TV))
	{TVFE_AAFILTER_CTRL1		,	0x00012721,},//0x00082222 TVFE_AAFILTER_CTRL1 bypass all
	{TVFE_AAFILTER_CTRL2		,	0x1304fcfa,},//TVFE_AAFILTER_CTRL2
#else
	{TVFE_AAFILTER_CTRL1		,   0x00182222,},//0x00082222 TVFE_AAFILTER_CTRL1 bypass all
	{TVFE_AAFILTER_CTRL2		,   0x252b39c6,},//TVFE_AAFILTER_CTRL2
#endif
#else
	{TVFE_AAFILTER_CTRL1            ,   0x00100000,},//TVFE_AAFILTER_CTRL1 bypass all
	{TVFE_AAFILTER_CTRL2            ,   0x00000000,},//TVFE_AAFILTER_CTRL2
	{TVFE_AAFILTER_CTRL3            ,   0x00000000,},//TVFE_AAFILTER_CTRL3
	{TVFE_AAFILTER_CTRL4            ,   0x00000000,},//TVFE_AAFILTER_CTRL4
	{TVFE_AAFILTER_CTRL5            ,   0x00000000,},//TVFE_AAFILTER_CTRL5
#endif
	{TVFE_SOG_MON_CTRL1            ,   0x00000000,},//TVFE_SOG_MON_CTRL1
	{TVFE_ADC_READBACK_CTRL1,   0x00000000,},//TVFE_ADC_READBACK_CTRL1
	{TVFE_ADC_READBACK_CTRL2,   0x00000000,},//TVFE_ADC_READBACK_CTRL2
#ifdef CRYSTAL_25M
	{TVFE_AFC_CTRL1 		     ,	 0x85730459,},//TVFE_AFC_CTRL1
	{TVFE_AFC_CTRL2 		     ,	 0x342fa9ed,},//TVFE_AFC_CTRL2
	{TVFE_AFC_CTRL3 		     ,	 0x2a02396,},//TVFE_AFC_CTRL3
	{TVFE_AFC_CTRL4 		     ,	 0xfefbff14,},//TVFE_AFC_CTRL4
	{TVFE_AFC_CTRL5 		     ,		0x0,},//TVFE_AFC_CTRL5
#else//for 24M
#if (defined(CONFIG_ADC_DOUBLE_SAMPLING_FOR_CVBS) && defined(CRYSTAL_24M) && (MESON_CPU_TYPE >= MESON_CPU_TYPE_MESONG9TV))
	{TVFE_AFC_CTRL1 			 ,	 0x893904d2,},//TVFE_AFC_CTRL1
	{TVFE_AFC_CTRL2 			 ,	 0x0f4b9ac9,},//TVFE_AFC_CTRL2
	{TVFE_AFC_CTRL3 			 ,	 0x01fd8c36,},//TVFE_AFC_CTRL3
	{TVFE_AFC_CTRL4 			 ,	 0x2de6d04f,},//TVFE_AFC_CTRL4
	{TVFE_AFC_CTRL5 			 ,	 0x00000004,},//TVFE_AFC_CTRL5
#else
#if (MESON_CPU_TYPE >= MESON_CPU_TYPE_MESONG9TV)
	{TVFE_AFC_CTRL1                      ,   0x05730459,},//TVFE_AFC_CTRL1
#else
    {TVFE_AFC_CTRL1                      ,   0x893904d2,},//TVFE_AFC_CTRL1
#endif
	{TVFE_AFC_CTRL2                      ,   0xf4b9ac9,},//TVFE_AFC_CTRL2
	{TVFE_AFC_CTRL3                      ,   0x1fd8c36,},//TVFE_AFC_CTRL3
	{TVFE_AFC_CTRL4                      ,   0x2de6d04f,},//TVFE_AFC_CTRL4
	{TVFE_AFC_CTRL5                      ,          0x4,},//TVFE_AFC_CTRL5
#endif
#endif
	{0xFFFFFFFF                             ,0x00000000,}
};


/*
 * tvafe cvd2 video poaition reg setting
 */
static enum tvafe_adc_ch_e tvafe_adc_pin_muxing(enum tvafe_adc_pin_e pin)
{
	enum tvafe_adc_ch_e ret = TVAFE_ADC_CH_NULL;
#if (MESON_CPU_TYPE >= MESON_CPU_TYPE_MESONG9TV)
	if (pin == TVAFE_CVBS_IN0)
	{
		W_APB_BIT(TVFE_VAFE_CTRL1,1,VAFE_IN_SEL_BIT,VAFE_IN_SEL_WID);
		W_APB_BIT(TVFE_VAFE_CTRL2,3,4,3);//AFE_PGA_RESV<4:6>:Selected Pin
		ret = TVAFE_ADC_CH_0;
	}
	else if (pin == TVAFE_CVBS_IN1)
	{
		W_APB_BIT(TVFE_VAFE_CTRL1,2,VAFE_IN_SEL_BIT,VAFE_IN_SEL_WID);
		W_APB_BIT(TVFE_VAFE_CTRL2,5,4,3);
		ret = TVAFE_ADC_CH_1;
	}
	else if (pin == TVAFE_CVBS_IN2)
	{
		W_APB_BIT(TVFE_VAFE_CTRL1,3,VAFE_IN_SEL_BIT,VAFE_IN_SEL_WID);
		W_APB_BIT(TVFE_VAFE_CTRL2,6,4,3);
		ret = TVAFE_ADC_CH_2;
	}
	else if (pin == TVAFE_CVBS_IN3)
	{
		/* atv demod data for cvd2 */
		W_APB_REG(TVFE_ATV_DMD_CLP_CTRL, 0x1300010);
		W_APB_BIT(TVFE_VAFE_CTRL2,6,4,3);
		ret = TVAFE_ADC_CH_3;
	}
#else
	if ((pin >= TVAFE_ADC_PIN_A_PGA_0) && (pin <= TVAFE_ADC_PIN_A_PGA_3))
	{
		W_APB_BIT(ADC_REG_06, 1, ENPGA_BIT, ENPGA_WID);
		W_APB_BIT(ADC_REG_17, pin-TVAFE_ADC_PIN_A_PGA_0, INMUXA_BIT, INMUXA_WID);
		ret = TVAFE_ADC_CH_PGA;
	}
	else if ((pin >= TVAFE_ADC_PIN_A_0) && (pin <= TVAFE_ADC_PIN_A_3))
	{
		W_APB_BIT(ADC_REG_06, 0, ENPGA_BIT, ENPGA_WID);
		W_APB_BIT(ADC_REG_17, pin-TVAFE_ADC_PIN_A_0, INMUXA_BIT, INMUXA_WID);
		ret = TVAFE_ADC_CH_A;
	}
	else if ((pin >= TVAFE_ADC_PIN_B_0) && (pin <= TVAFE_ADC_PIN_B_4))
	{
		W_APB_BIT(ADC_REG_17, pin-TVAFE_ADC_PIN_B_0, INMUXB_BIT, INMUXB_WID);
		ret = TVAFE_ADC_CH_B;
	}
	else if ((pin >= TVAFE_ADC_PIN_C_0) && (pin <= TVAFE_ADC_PIN_C_4))
	{
		W_APB_BIT(ADC_REG_18, pin-TVAFE_ADC_PIN_C_0, INMUXC_BIT, INMUXC_WID);
		ret = TVAFE_ADC_CH_C;
	}
#endif
	return ret;
}

/*
000: abc
001: acb
010: bac
011: bca
100: cab
101: cba
 * tvafe top mux setting for yuv channel
 */

#if (MESON_CPU_TYPE < MESON_CPU_TYPE_MESONG9TV)
static int tvafe_adc_top_muxing(enum tvafe_adc_ch_e gy,
		enum tvafe_adc_ch_e bpb,
		enum tvafe_adc_ch_e rpr,
		unsigned int s_video_flag)
{
	int ret = 0;
	switch (gy)
	{
		case TVAFE_ADC_CH_PGA:
		case TVAFE_ADC_CH_A:
			switch (bpb)
			{
				case TVAFE_ADC_CH_B:
					// abc => abc
					if (s_video_flag || (rpr == TVAFE_ADC_CH_C))
					{
						W_APB_BIT(TVFE_TOP_CTRL, 0, SWT_GY_BCB_RCR_IN_BIT,
								SWT_GY_BCB_RCR_IN_WID);
					}
					else
					{
						ret = -EFAULT;
					}
					break;
				case TVAFE_ADC_CH_C:
					// acb => abc
					if (s_video_flag || (rpr == TVAFE_ADC_CH_B))
					{
						W_APB_BIT(TVFE_TOP_CTRL, 1, SWT_GY_BCB_RCR_IN_BIT,
								SWT_GY_BCB_RCR_IN_WID);
					}
					else
					{
						ret = -EFAULT;
					}
					break;
				default:
					ret = -EFAULT;
					break;
			}
			break;
		case TVAFE_ADC_CH_B:
			switch (bpb)
			{
				case TVAFE_ADC_CH_PGA:
				case TVAFE_ADC_CH_A:
					// bac => abc
					if (s_video_flag || (rpr == TVAFE_ADC_CH_C))
					{
						W_APB_BIT(TVFE_TOP_CTRL, 2, SWT_GY_BCB_RCR_IN_BIT,
								SWT_GY_BCB_RCR_IN_WID);
					}
					else
					{
						ret = -EFAULT;
					}
					break;
				case TVAFE_ADC_CH_C:
					// bca => abc
					if (s_video_flag || (rpr == TVAFE_ADC_CH_PGA)
							|| (rpr == TVAFE_ADC_CH_A))
					{
						W_APB_BIT(TVFE_TOP_CTRL, 3, SWT_GY_BCB_RCR_IN_BIT,
								SWT_GY_BCB_RCR_IN_WID);
					}
					else
					{
						ret = -EFAULT;
					}
					break;
				default:
					ret = -EFAULT;
					break;
			}
			break;
		case TVAFE_ADC_CH_C:
			switch (bpb)
			{
				case TVAFE_ADC_CH_PGA:
				case TVAFE_ADC_CH_A:
					// cab => abc
					if (s_video_flag || (rpr == TVAFE_ADC_CH_B))
					{
						W_APB_BIT(TVFE_TOP_CTRL, 4, SWT_GY_BCB_RCR_IN_BIT,
								SWT_GY_BCB_RCR_IN_WID);
					}
					else
					{
						ret = -EFAULT;
					}
					break;
				case TVAFE_ADC_CH_B:
					// cba => abc
					if (s_video_flag || (rpr == TVAFE_ADC_CH_PGA)
							|| (rpr == TVAFE_ADC_CH_A))
					{
						W_APB_BIT(TVFE_TOP_CTRL, 5, SWT_GY_BCB_RCR_IN_BIT,
								SWT_GY_BCB_RCR_IN_WID);
					}
					else
					{
						ret = -EFAULT;
					}
					break;
				default:
					ret = -EFAULT;
					break;
			}
			break;
		default:
			ret = -EFAULT;
			break;
	}
	return ret;
}
#endif

/*
 * tvafe pin mux setting for input source
 */
int tvafe_set_source_muxing(enum tvin_port_e port, struct tvafe_pin_mux_s *pinmux)
{
	int ret = 0;

	switch (port)
	{
		case TVIN_PORT_CVBS0:
#if (MESON_CPU_TYPE >= MESON_CPU_TYPE_MESONG9TV)
			tvafe_adc_pin_muxing(pinmux->pin[CVBS_IN0]);
#else
			if (tvafe_adc_pin_muxing(pinmux->pin[CVBS0_Y]) == TVAFE_ADC_CH_PGA)
			{
				W_APB_BIT(ADC_REG_20, pinmux->pin[CVBS0_Y]-TVAFE_ADC_PIN_A_PGA_0, INMUXBUF_BIT, INMUXBUF_WID);
				if (pinmux->pin[CVBS0_SOG] >= TVAFE_ADC_PIN_SOG_0)
					W_APB_BIT(ADC_REG_24, (pinmux->pin[CVBS0_SOG] - TVAFE_ADC_PIN_SOG_0), INMUXSOG_BIT, INMUXSOG_WID);
			}
			else
			{
				ret = -EFAULT;
			}
#endif
			break;
		case TVIN_PORT_CVBS1:
#if (MESON_CPU_TYPE >= MESON_CPU_TYPE_MESONG9TV)
			tvafe_adc_pin_muxing(pinmux->pin[CVBS_IN1]);
#else
			if (tvafe_adc_pin_muxing(pinmux->pin[CVBS1_Y]) == TVAFE_ADC_CH_PGA)
			{
				W_APB_BIT(ADC_REG_20, pinmux->pin[CVBS1_Y]-TVAFE_ADC_PIN_A_PGA_0, INMUXBUF_BIT, INMUXBUF_WID);
				if (pinmux->pin[CVBS1_SOG] >= TVAFE_ADC_PIN_SOG_0)
					W_APB_BIT(ADC_REG_24, (pinmux->pin[CVBS1_SOG] - TVAFE_ADC_PIN_SOG_0), INMUXSOG_BIT, INMUXSOG_WID);
			}
			else
			{
				ret = -EFAULT;
			}
#endif
			break;
		case TVIN_PORT_CVBS2:
#if (MESON_CPU_TYPE >= MESON_CPU_TYPE_MESONG9TV)
			tvafe_adc_pin_muxing(pinmux->pin[CVBS_IN2]);
#else
			if (tvafe_adc_pin_muxing(pinmux->pin[CVBS2_Y]) == TVAFE_ADC_CH_PGA)
			{
				W_APB_BIT(ADC_REG_20, pinmux->pin[CVBS2_Y]-TVAFE_ADC_PIN_A_PGA_0, INMUXBUF_BIT, INMUXBUF_WID);
				if (pinmux->pin[CVBS2_SOG] >= TVAFE_ADC_PIN_SOG_0)
					W_APB_BIT(ADC_REG_24, (pinmux->pin[CVBS2_SOG] - TVAFE_ADC_PIN_SOG_0), INMUXSOG_BIT, INMUXSOG_WID);
			}
			else
			{
				ret = -EFAULT;
			}
#endif
			break;
		case TVIN_PORT_CVBS3:
#if (MESON_CPU_TYPE >= MESON_CPU_TYPE_MESONG9TV)
			tvafe_adc_pin_muxing(pinmux->pin[CVBS_IN3]);
#else
			if (tvafe_adc_pin_muxing(pinmux->pin[CVBS3_Y]) == TVAFE_ADC_CH_PGA)
			{
				W_APB_BIT(ADC_REG_20, pinmux->pin[CVBS3_Y]-TVAFE_ADC_PIN_A_PGA_0, INMUXBUF_BIT, INMUXBUF_WID);
				if (pinmux->pin[CVBS3_SOG] >= TVAFE_ADC_PIN_SOG_0)
					W_APB_BIT(ADC_REG_24, (pinmux->pin[CVBS3_SOG] - TVAFE_ADC_PIN_SOG_0), INMUXSOG_BIT, INMUXSOG_WID);
			}
			else
			{
				ret = -EFAULT;
			}
#endif
			break;
#if (MESON_CPU_TYPE < MESON_CPU_TYPE_MESONG9TV)
		case TVIN_PORT_CVBS4:
			if (tvafe_adc_pin_muxing(pinmux->pin[CVBS4_Y]) == TVAFE_ADC_CH_PGA)
			{
				W_APB_BIT(ADC_REG_20, pinmux->pin[CVBS4_Y]-TVAFE_ADC_PIN_A_PGA_0, INMUXBUF_BIT, INMUXBUF_WID);
				if (pinmux->pin[CVBS4_SOG] >= TVAFE_ADC_PIN_SOG_0)
					W_APB_BIT(ADC_REG_24, (pinmux->pin[CVBS5_SOG] - TVAFE_ADC_PIN_SOG_0), INMUXSOG_BIT, INMUXSOG_WID);
			}
			else
			{
				ret = -EFAULT;
			}
			break;
		case TVIN_PORT_CVBS5:
			if (tvafe_adc_pin_muxing(pinmux->pin[CVBS5_Y]) == TVAFE_ADC_CH_PGA)
			{
				W_APB_BIT(ADC_REG_20, pinmux->pin[CVBS5_Y]-TVAFE_ADC_PIN_A_PGA_0, INMUXBUF_BIT, INMUXBUF_WID);
				if (pinmux->pin[CVBS5_SOG] >= TVAFE_ADC_PIN_SOG_0)
					W_APB_BIT(ADC_REG_24, (pinmux->pin[CVBS5_SOG] - TVAFE_ADC_PIN_SOG_0), INMUXSOG_BIT, INMUXSOG_WID);
			}
			else
			{
				ret = -EFAULT;
			}
			break;
		case TVIN_PORT_CVBS6:
			if (tvafe_adc_pin_muxing(pinmux->pin[CVBS6_Y]) == TVAFE_ADC_CH_PGA)
			{
				W_APB_BIT(ADC_REG_20, pinmux->pin[CVBS6_Y]-TVAFE_ADC_PIN_A_PGA_0, INMUXBUF_BIT, INMUXBUF_WID);
				if (pinmux->pin[CVBS6_SOG] >= TVAFE_ADC_PIN_SOG_0)
					W_APB_BIT(ADC_REG_24, (pinmux->pin[CVBS6_SOG] - TVAFE_ADC_PIN_SOG_0), INMUXSOG_BIT, INMUXSOG_WID);
			}
			else
			{
				ret = -EFAULT;
			}
			break;
		case TVIN_PORT_CVBS7:
			if (tvafe_adc_pin_muxing(pinmux->pin[CVBS7_Y]) == TVAFE_ADC_CH_PGA)
			{
				W_APB_BIT(ADC_REG_20, pinmux->pin[CVBS7_Y]-TVAFE_ADC_PIN_A_PGA_0, INMUXBUF_BIT, INMUXBUF_WID);
				if (pinmux->pin[CVBS7_SOG] >= TVAFE_ADC_PIN_SOG_0)
					W_APB_BIT(ADC_REG_24, (pinmux->pin[CVBS7_SOG] - TVAFE_ADC_PIN_SOG_0), INMUXSOG_BIT, INMUXSOG_WID);
			}
			else
			{
				ret = -EFAULT;
			}
			break;
		case TVIN_PORT_SVIDEO0:
			if (tvafe_adc_top_muxing(tvafe_adc_pin_muxing(pinmux->pin[S_VIDEO0_Y]), tvafe_adc_pin_muxing(pinmux->pin[S_VIDEO0_C]), TVAFE_ADC_CH_NULL, 1) == 0)
			{
				W_APB_BIT(ADC_REG_20, tvafe_default_cvbs_out-TVAFE_ADC_PIN_A_PGA_0, INMUXBUF_BIT, INMUXBUF_WID);
				if (pinmux->pin[S_VIDEO0_SOG] >= TVAFE_ADC_PIN_SOG_0)
					W_APB_BIT(ADC_REG_24, (pinmux->pin[S_VIDEO0_SOG] - TVAFE_ADC_PIN_SOG_0), INMUXSOG_BIT, INMUXSOG_WID);
			}
			else
			{
				ret = -EFAULT;
			}
			break;
		case TVIN_PORT_SVIDEO1:
			if (tvafe_adc_top_muxing(tvafe_adc_pin_muxing(pinmux->pin[S_VIDEO1_Y]), tvafe_adc_pin_muxing(pinmux->pin[S_VIDEO1_C]), TVAFE_ADC_CH_NULL, 1) == 0)
			{
				W_APB_BIT(ADC_REG_20, tvafe_default_cvbs_out-TVAFE_ADC_PIN_A_PGA_0, INMUXBUF_BIT, INMUXBUF_WID);
				if (pinmux->pin[S_VIDEO1_SOG] >= TVAFE_ADC_PIN_SOG_0)
					W_APB_BIT(ADC_REG_24, (pinmux->pin[S_VIDEO1_SOG] - TVAFE_ADC_PIN_SOG_0), INMUXSOG_BIT, INMUXSOG_WID);
			}
			else
			{
				ret = -EFAULT;
			}
			break;
		case TVIN_PORT_SVIDEO2:
			if (tvafe_adc_top_muxing(tvafe_adc_pin_muxing(pinmux->pin[S_VIDEO2_Y]), tvafe_adc_pin_muxing(pinmux->pin[S_VIDEO2_C]), TVAFE_ADC_CH_NULL, 1) == 0)
			{
				W_APB_BIT(ADC_REG_20, tvafe_default_cvbs_out-TVAFE_ADC_PIN_A_PGA_0, INMUXBUF_BIT, INMUXBUF_WID);
				if (pinmux->pin[S_VIDEO2_SOG] >= TVAFE_ADC_PIN_SOG_0)
					W_APB_BIT(ADC_REG_24, (pinmux->pin[S_VIDEO2_SOG] - TVAFE_ADC_PIN_SOG_0), INMUXSOG_BIT, INMUXSOG_WID);
			}
			else
			{
				ret = -EFAULT;
			}
			break;
		case TVIN_PORT_SVIDEO3:
			if (tvafe_adc_top_muxing(tvafe_adc_pin_muxing(pinmux->pin[S_VIDEO3_Y]), tvafe_adc_pin_muxing(pinmux->pin[S_VIDEO3_C]), TVAFE_ADC_CH_NULL, 1) == 0)
			{
				W_APB_BIT(ADC_REG_20 , tvafe_default_cvbs_out-TVAFE_ADC_PIN_A_PGA_0, INMUXBUF_BIT, INMUXBUF_WID);
				if (pinmux->pin[S_VIDEO3_SOG] >= TVAFE_ADC_PIN_SOG_0)
					W_APB_BIT(ADC_REG_24, (pinmux->pin[S_VIDEO3_SOG] - TVAFE_ADC_PIN_SOG_0), INMUXSOG_BIT, INMUXSOG_WID);
			}
			else
			{
				ret = -EFAULT;
			}
			break;
		case TVIN_PORT_SVIDEO4:
			if (tvafe_adc_top_muxing(tvafe_adc_pin_muxing(pinmux->pin[S_VIDEO4_Y]), tvafe_adc_pin_muxing(pinmux->pin[S_VIDEO4_C]), TVAFE_ADC_CH_NULL, 1) == 0)
			{
				W_APB_BIT(ADC_REG_20, tvafe_default_cvbs_out-TVAFE_ADC_PIN_A_PGA_0, INMUXBUF_BIT, INMUXBUF_WID);
				if (pinmux->pin[S_VIDEO4_SOG] >= TVAFE_ADC_PIN_SOG_0)
					W_APB_BIT(ADC_REG_24, (pinmux->pin[S_VIDEO4_SOG] - TVAFE_ADC_PIN_SOG_0), INMUXSOG_BIT, INMUXSOG_WID);
			}
			else
			{
				ret = -EFAULT;
			}
			break;
		case TVIN_PORT_SVIDEO5:
			if (tvafe_adc_top_muxing(tvafe_adc_pin_muxing(pinmux->pin[S_VIDEO5_Y]), tvafe_adc_pin_muxing(pinmux->pin[S_VIDEO5_C]), TVAFE_ADC_CH_NULL, 1) == 0)
			{
				W_APB_BIT(ADC_REG_20, tvafe_default_cvbs_out-TVAFE_ADC_PIN_A_PGA_0, INMUXBUF_BIT, INMUXBUF_WID);
				if (pinmux->pin[S_VIDEO5_SOG] >= TVAFE_ADC_PIN_SOG_0)
					W_APB_BIT(ADC_REG_24, (pinmux->pin[S_VIDEO5_SOG] - TVAFE_ADC_PIN_SOG_0), INMUXSOG_BIT, INMUXSOG_WID);
			}
			else
			{
				ret = -EFAULT;
			}
			break;
		case TVIN_PORT_SVIDEO6:
			if (tvafe_adc_top_muxing(tvafe_adc_pin_muxing(pinmux->pin[S_VIDEO6_Y]), tvafe_adc_pin_muxing(pinmux->pin[S_VIDEO6_C]), TVAFE_ADC_CH_NULL, 1) == 0)
			{
				W_APB_BIT(ADC_REG_20, tvafe_default_cvbs_out-TVAFE_ADC_PIN_A_PGA_0, INMUXBUF_BIT, INMUXBUF_WID);
				if (pinmux->pin[S_VIDEO6_SOG] >= TVAFE_ADC_PIN_SOG_0)
					W_APB_BIT(ADC_REG_24, (pinmux->pin[S_VIDEO6_SOG] - TVAFE_ADC_PIN_SOG_0), INMUXSOG_BIT, INMUXSOG_WID);
			}
			else
			{
				ret = -EFAULT;
			}
			break;
		case TVIN_PORT_SVIDEO7:
			if (tvafe_adc_top_muxing(tvafe_adc_pin_muxing(pinmux->pin[S_VIDEO7_Y]), tvafe_adc_pin_muxing(pinmux->pin[S_VIDEO7_C]), TVAFE_ADC_CH_NULL, 1) == 0)
			{
				W_APB_BIT(ADC_REG_20, tvafe_default_cvbs_out-TVAFE_ADC_PIN_A_PGA_0, INMUXBUF_BIT, INMUXBUF_WID);
				if (pinmux->pin[S_VIDEO7_SOG] >= TVAFE_ADC_PIN_SOG_0)
					W_APB_BIT(ADC_REG_24, (pinmux->pin[S_VIDEO7_SOG] - TVAFE_ADC_PIN_SOG_0), INMUXSOG_BIT, INMUXSOG_WID);
			}
			else
			{
				ret = -EFAULT;
			}
			break;
		case TVIN_PORT_VGA0:
			if (tvafe_adc_top_muxing(tvafe_adc_pin_muxing(pinmux->pin[VGA0_G]), tvafe_adc_pin_muxing(pinmux->pin[VGA0_B]), tvafe_adc_pin_muxing(pinmux->pin[VGA0_R]), 0) == 0)
			{
				W_APB_BIT(ADC_REG_20, tvafe_default_cvbs_out-TVAFE_ADC_PIN_A_PGA_0, INMUXBUF_BIT, INMUXBUF_WID);
				if (pinmux->pin[VGA0_SOG] >= TVAFE_ADC_PIN_SOG_0)
					W_APB_BIT(ADC_REG_24, (pinmux->pin[VGA0_SOG] - TVAFE_ADC_PIN_SOG_0), INMUXSOG_BIT, INMUXSOG_WID);

				W_APB_BIT(ADC_REG_39, 0, INSYNCMUXCTRL_BIT, INSYNCMUXCTRL_WID);
			}
			else
			{
				ret = -EFAULT;
			}
			break;
		case TVIN_PORT_VGA1:
			if (tvafe_adc_top_muxing(tvafe_adc_pin_muxing(pinmux->pin[VGA1_G]), tvafe_adc_pin_muxing(pinmux->pin[VGA1_B]), tvafe_adc_pin_muxing(pinmux->pin[VGA1_R]), 0) == 0)
			{
				W_APB_BIT(ADC_REG_20, tvafe_default_cvbs_out-TVAFE_ADC_PIN_A_PGA_0, INMUXBUF_BIT, INMUXBUF_WID);
				if (pinmux->pin[VGA1_SOG] >= TVAFE_ADC_PIN_SOG_0)
					W_APB_BIT(ADC_REG_24, (pinmux->pin[VGA1_SOG] - TVAFE_ADC_PIN_SOG_0), INMUXSOG_BIT, INMUXSOG_WID);
				W_APB_BIT(ADC_REG_39, 1, INSYNCMUXCTRL_BIT, INSYNCMUXCTRL_WID);
			}
			else
			{
				ret = -EFAULT;
			}
			break;
		case TVIN_PORT_VGA2:
			if (tvafe_adc_top_muxing(tvafe_adc_pin_muxing(pinmux->pin[VGA2_G]), tvafe_adc_pin_muxing(pinmux->pin[VGA2_B]), tvafe_adc_pin_muxing(pinmux->pin[VGA2_R]), 0) == 0)
			{
				W_APB_BIT(ADC_REG_20, tvafe_default_cvbs_out-TVAFE_ADC_PIN_A_PGA_0, INMUXBUF_BIT, INMUXBUF_WID);
				if (pinmux->pin[VGA2_SOG] >= TVAFE_ADC_PIN_SOG_0)
					W_APB_BIT(ADC_REG_24, (pinmux->pin[VGA2_SOG] - TVAFE_ADC_PIN_SOG_0), INMUXSOG_BIT, INMUXSOG_WID);
			}
			else
			{
				ret = -EFAULT;
			}
			break;
		case TVIN_PORT_VGA3:
			if (tvafe_adc_top_muxing(tvafe_adc_pin_muxing(pinmux->pin[VGA3_G]), tvafe_adc_pin_muxing(pinmux->pin[VGA3_B]), tvafe_adc_pin_muxing(pinmux->pin[VGA3_R]), 0) == 0)
			{
				W_APB_BIT(ADC_REG_20, tvafe_default_cvbs_out-TVAFE_ADC_PIN_A_PGA_0, INMUXBUF_BIT, INMUXBUF_WID);
				if (pinmux->pin[VGA3_SOG] >= TVAFE_ADC_PIN_SOG_0)
					W_APB_BIT(ADC_REG_24, (pinmux->pin[VGA3_SOG] - TVAFE_ADC_PIN_SOG_0), INMUXSOG_BIT, INMUXSOG_WID);
			}
			else
			{
				ret = -EFAULT;
			}
			break;
		case TVIN_PORT_VGA4:
			if (tvafe_adc_top_muxing(tvafe_adc_pin_muxing(pinmux->pin[VGA4_G]), tvafe_adc_pin_muxing(pinmux->pin[VGA4_B]), tvafe_adc_pin_muxing(pinmux->pin[VGA4_R]), 0) == 0)
			{
				W_APB_BIT(ADC_REG_20, tvafe_default_cvbs_out-TVAFE_ADC_PIN_A_PGA_0, INMUXBUF_BIT, INMUXBUF_WID);
				if (pinmux->pin[VGA4_SOG] >= TVAFE_ADC_PIN_SOG_0)
					W_APB_BIT(ADC_REG_24, (pinmux->pin[VGA4_SOG] - TVAFE_ADC_PIN_SOG_0), INMUXSOG_BIT, INMUXSOG_WID);
			}
			else
			{
				ret = -EFAULT;
			}
			break;
		case TVIN_PORT_VGA5:
			if (tvafe_adc_top_muxing(tvafe_adc_pin_muxing(pinmux->pin[VGA5_G]), tvafe_adc_pin_muxing(pinmux->pin[VGA5_B]), tvafe_adc_pin_muxing(pinmux->pin[VGA5_R]), 0) == 0)
			{
				W_APB_BIT(ADC_REG_20, tvafe_default_cvbs_out-TVAFE_ADC_PIN_A_PGA_0, INMUXBUF_BIT, INMUXBUF_WID);
				if (pinmux->pin[VGA5_SOG] >= TVAFE_ADC_PIN_SOG_0)
					W_APB_BIT(ADC_REG_24, (pinmux->pin[VGA5_SOG] - TVAFE_ADC_PIN_SOG_0), INMUXSOG_BIT, INMUXSOG_WID);
			}
			else
			{
				ret = -EFAULT;
			}
			break;
		case TVIN_PORT_VGA6:
			if (tvafe_adc_top_muxing(tvafe_adc_pin_muxing(pinmux->pin[VGA6_G]), tvafe_adc_pin_muxing(pinmux->pin[VGA6_B]), tvafe_adc_pin_muxing(pinmux->pin[VGA6_R]), 0) == 0)
			{
				W_APB_BIT(ADC_REG_20, tvafe_default_cvbs_out-TVAFE_ADC_PIN_A_PGA_0, INMUXBUF_BIT, INMUXBUF_WID);
				if (pinmux->pin[VGA6_SOG] >= TVAFE_ADC_PIN_SOG_0)
					W_APB_BIT(ADC_REG_24, (pinmux->pin[VGA6_SOG] - TVAFE_ADC_PIN_SOG_0), INMUXSOG_BIT, INMUXSOG_WID);
			}
			else
			{
				ret = -EFAULT;
			}
			break;
		case TVIN_PORT_VGA7:
			if (tvafe_adc_top_muxing(tvafe_adc_pin_muxing(pinmux->pin[VGA7_G]), tvafe_adc_pin_muxing(pinmux->pin[VGA7_B]), tvafe_adc_pin_muxing(pinmux->pin[VGA7_R]), 0) == 0)
			{
				W_APB_BIT(ADC_REG_20, tvafe_default_cvbs_out-TVAFE_ADC_PIN_A_PGA_0, INMUXBUF_BIT, INMUXBUF_WID);
				if (pinmux->pin[VGA7_SOG] >= TVAFE_ADC_PIN_SOG_0)
					W_APB_BIT(ADC_REG_24, (pinmux->pin[VGA7_SOG] - TVAFE_ADC_PIN_SOG_0), INMUXSOG_BIT, INMUXSOG_WID);
			}
			else
			{
				ret = -EFAULT;
			}
			break;
		case TVIN_PORT_COMP0:
			if (tvafe_adc_top_muxing(tvafe_adc_pin_muxing(pinmux->pin[COMP0_Y]), tvafe_adc_pin_muxing(pinmux->pin[COMP0_PB]), tvafe_adc_pin_muxing(pinmux->pin[COMP0_PR]), 0) == 0)
			{
				W_APB_BIT(ADC_REG_20, tvafe_default_cvbs_out-TVAFE_ADC_PIN_A_PGA_0, INMUXBUF_BIT, INMUXBUF_WID);
				if (pinmux->pin[COMP0_SOG] >= TVAFE_ADC_PIN_SOG_0)
					W_APB_BIT(ADC_REG_24, (pinmux->pin[COMP0_SOG] - TVAFE_ADC_PIN_SOG_0), INMUXSOG_BIT, INMUXSOG_WID);
			}
			else
			{
				ret = -EFAULT;
			}
			break;
		case TVIN_PORT_COMP1:
			if (tvafe_adc_top_muxing(tvafe_adc_pin_muxing(pinmux->pin[COMP1_Y]), tvafe_adc_pin_muxing(pinmux->pin[COMP1_PB]), tvafe_adc_pin_muxing(pinmux->pin[COMP1_PR]), 0) == 0)
			{
				W_APB_BIT(ADC_REG_20, tvafe_default_cvbs_out-TVAFE_ADC_PIN_A_PGA_0, INMUXBUF_BIT, INMUXBUF_WID);
				if (pinmux->pin[COMP1_SOG] >= TVAFE_ADC_PIN_SOG_0)
					W_APB_BIT(ADC_REG_24, (pinmux->pin[COMP1_SOG] - TVAFE_ADC_PIN_SOG_0), INMUXSOG_BIT, INMUXSOG_WID);
			}
			else
			{
				ret = -EFAULT;
			}
			break;
		case TVIN_PORT_COMP2:
			if (tvafe_adc_top_muxing(tvafe_adc_pin_muxing(pinmux->pin[COMP2_Y]), tvafe_adc_pin_muxing(pinmux->pin[COMP2_PB]), tvafe_adc_pin_muxing(pinmux->pin[COMP2_PR]), 0) == 0)
			{
				W_APB_BIT(ADC_REG_20, tvafe_default_cvbs_out-TVAFE_ADC_PIN_A_PGA_0, INMUXBUF_BIT, INMUXBUF_WID);
				if (pinmux->pin[COMP2_SOG] >= TVAFE_ADC_PIN_SOG_0)
					W_APB_BIT(ADC_REG_24, (pinmux->pin[COMP2_SOG] - TVAFE_ADC_PIN_SOG_0), INMUXSOG_BIT, INMUXSOG_WID);
			}
			else
			{
				ret = -EFAULT;
			}
			break;
		case TVIN_PORT_COMP3:
			if (tvafe_adc_top_muxing(tvafe_adc_pin_muxing( pinmux->pin[COMP3_Y]), tvafe_adc_pin_muxing(pinmux->pin[COMP3_PB]), tvafe_adc_pin_muxing(pinmux->pin[COMP3_PR]), 0) == 0)
			{
				W_APB_BIT(ADC_REG_20, tvafe_default_cvbs_out-TVAFE_ADC_PIN_A_PGA_0, INMUXBUF_BIT, INMUXBUF_WID);
				if (pinmux->pin[COMP3_SOG] >= TVAFE_ADC_PIN_SOG_0)
					W_APB_BIT(ADC_REG_24, (pinmux->pin[COMP3_SOG] - TVAFE_ADC_PIN_SOG_0), INMUXSOG_BIT, INMUXSOG_WID);
			}
			else
			{
				ret = -EFAULT;
			}
			break;
		case TVIN_PORT_COMP4:
			if (tvafe_adc_top_muxing(tvafe_adc_pin_muxing(pinmux->pin[COMP4_Y]), tvafe_adc_pin_muxing(pinmux->pin[COMP4_PB]), tvafe_adc_pin_muxing(pinmux->pin[COMP4_PR]), 0) == 0)
			{
				W_APB_BIT(ADC_REG_20, tvafe_default_cvbs_out-TVAFE_ADC_PIN_A_PGA_0, INMUXBUF_BIT, INMUXBUF_WID);
				if (pinmux->pin[COMP4_SOG] >= TVAFE_ADC_PIN_SOG_0)
					W_APB_BIT(ADC_REG_24, (pinmux->pin[COMP4_SOG] - TVAFE_ADC_PIN_SOG_0), INMUXSOG_BIT, INMUXSOG_WID);
			}
			else
			{
				ret = -EFAULT;
			}
			break;
		case TVIN_PORT_COMP5:
			if (tvafe_adc_top_muxing(tvafe_adc_pin_muxing(pinmux->pin[COMP5_Y]), tvafe_adc_pin_muxing(pinmux->pin[COMP5_PB]), tvafe_adc_pin_muxing(pinmux->pin[COMP5_PR]), 0) == 0)
			{
				W_APB_BIT(ADC_REG_20, tvafe_default_cvbs_out-TVAFE_ADC_PIN_A_PGA_0, INMUXBUF_BIT, INMUXBUF_WID);
				if (pinmux->pin[COMP5_SOG] >= TVAFE_ADC_PIN_SOG_0)
					W_APB_BIT(ADC_REG_24, (pinmux->pin[COMP5_SOG] - TVAFE_ADC_PIN_SOG_0), INMUXSOG_BIT, INMUXSOG_WID);
			}
			else
			{
				ret = -EFAULT;
			}
			break;
		case TVIN_PORT_COMP6:
			if (tvafe_adc_top_muxing(tvafe_adc_pin_muxing(pinmux->pin[COMP6_Y]), tvafe_adc_pin_muxing(pinmux->pin[COMP6_PB]), tvafe_adc_pin_muxing(pinmux->pin[COMP6_PR]), 0) == 0)
			{
				W_APB_BIT(ADC_REG_20, tvafe_default_cvbs_out-TVAFE_ADC_PIN_A_PGA_0, INMUXBUF_BIT, INMUXBUF_WID);
				if (pinmux->pin[COMP6_SOG] >= TVAFE_ADC_PIN_SOG_0)
					W_APB_BIT(ADC_REG_24, (pinmux->pin[COMP6_SOG] - TVAFE_ADC_PIN_SOG_0), INMUXSOG_BIT, INMUXSOG_WID);
			}
			else
			{
				ret = -EFAULT;
			}
			break;
		case TVIN_PORT_COMP7:
			if (tvafe_adc_top_muxing(tvafe_adc_pin_muxing(pinmux->pin[COMP7_Y]), tvafe_adc_pin_muxing(pinmux->pin[COMP7_PB]), tvafe_adc_pin_muxing(pinmux->pin[COMP7_PR]), 0) == 0)
			{
				W_APB_BIT(ADC_REG_20, tvafe_default_cvbs_out-TVAFE_ADC_PIN_A_PGA_0, INMUXBUF_BIT, INMUXBUF_WID);
				if (pinmux->pin[COMP7_SOG] >= TVAFE_ADC_PIN_SOG_0)
					W_APB_BIT(ADC_REG_24, (pinmux->pin[COMP7_SOG] - TVAFE_ADC_PIN_SOG_0), INMUXSOG_BIT, INMUXSOG_WID);
			}
			else
			{
				ret = -EFAULT;
			}
			break;
#endif
		default:
			ret = -EFAULT;
			break;
	}
	if (ret == 0)
		pr_info("[tvafe..] %s set pin mux to port:0x%x ok.\n", __func__, port);
	else
		pr_info("[tvafe..] %s set pin mux error!!!!!.\n", __func__);

	return ret;
}

/*
 * tvafe load edid table
 */
void tvafe_vga_set_edid(struct tvafe_vga_edid_s *edid)
{
	unsigned int i = 0;

	W_APB_BIT(TVFE_TOP_CTRL, 1, EDID_CLK_EN_BIT,EDID_CLK_EN_WID); // VGA_CLK_EN
	// APB Bus accessing mode
	W_APB_REG(TVFE_EDID_CONFIG , 0x00000000);
	W_APB_REG(TVFE_EDID_RAM_ADDR, 0x00000000);
	for (i=0; i<256; i++)
		W_APB_REG(TVFE_EDID_RAM_WDATA, (unsigned int)edid->value[i]);
	// Slave IIC acessing mode, 8-bit standard IIC protocol
	W_APB_REG(TVFE_EDID_CONFIG, TVAFE_EDID_CONFIG);
}

/*
 * tvafe get edid table from Reg
 */
void tvafe_vga_get_edid(struct tvafe_vga_edid_s *edid)
{
	unsigned int i = 0;

	W_APB_BIT(TVFE_TOP_CTRL , 1, EDID_CLK_EN_BIT,EDID_CLK_EN_WID); // VGA_CLK_EN
	// APB Bus accessing mode
	W_APB_REG(TVFE_EDID_CONFIG, 0x00000000);
	W_APB_REG(TVFE_EDID_RAM_ADDR, 0x00000100);
	for (i=0; i<256; i++)
		edid->value[i] = (unsigned char)(R_APB_BIT(TVFE_EDID_RAM_RDATA,
					EDID_RAM_RDATA_BIT, EDID_RAM_RDATA_WID));
	// Slave IIC acessing mode, 8-bit standard IIC protocol
	W_APB_REG(TVFE_EDID_CONFIG, TVAFE_EDID_CONFIG);

	return;
}

#if 0
///////////////////TVFE top control////////////////////
const static unsigned int aafilter_ctl[][2] = {

	//TVIN_SIG_FMT_NULL = 0,
	{0,0},
	//VDIN_SIG_FORMAT_VGA_512X384P_60D147,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_VGA_560X384P_60D147,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_VGA_640X200P_59D924,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_VGA_640X350P_85D080,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_VGA_640X400P_59D940,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_VGA_640X400P_85D080,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_VGA_640X400P_59D638,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_VGA_640X400P_56D416,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_VGA_640X480I_29D970,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_VGA_640X480P_66D619,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_VGA_640X480P_66D667,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_VGA_640X480P_59D940,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_VGA_640X480P_60D000,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_VGA_640X480P_72D809,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_VGA_640X480P_75D000_A,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_VGA_640X480P_85D008,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_VGA_640X480P_59D638,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_VGA_640X480P_75D000_B,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_VGA_640X870P_75D000,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_VGA_720X350P_70D086,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_VGA_720X400P_85D039,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_VGA_720X400P_70D086,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_VGA_720X400P_87D849,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_VGA_720X400P_59D940,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_VGA_720X480P_59D940,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_VGA_752X484I_29D970,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_VGA_768X574I_25D000,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_VGA_800X600P_56D250,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_VGA_800X600P_60D317,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_VGA_800X600P_72D188,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_VGA_800X600P_75D000,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_VGA_800X600P_85D061,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_VGA_832X624P_75D087,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_VGA_848X480P_84D751,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_VGA_1024X768P_59D278,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_VGA_1024X768P_74D927,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_VGA_1024X768I_43D479,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_VGA_1024X768P_60D004,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_VGA_1024X768P_70D069,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_VGA_1024X768P_75D029,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_VGA_1024X768P_84D997,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_VGA_1024X768P_60D000,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_VGA_1024X768P_74D925,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_VGA_1024X768P_75D020,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_VGA_1024X768P_70D008,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_VGA_1024X768P_75D782,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_VGA_1024X768P_77D069,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_VGA_1024X768P_71D799,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_VGA_1024X1024P_60D000,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_VGA_1053X754I_43D453,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_VGA_1056X768I_43D470,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_VGA_1120X750I_40D021,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_VGA_1152X864P_70D012,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_VGA_1152X864P_75D000,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_VGA_1152X864P_84D999,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_VGA_1152X870P_75D062,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_VGA_1152X900P_65D950,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_VGA_1152X900P_66D004,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_VGA_1152X900P_76D047,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_VGA_1152X900P_76D149,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_VGA_1244X842I_30D000,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_VGA_1280X768P_59D995,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_VGA_1280X768P_74D893,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_VGA_1280X768P_84D837,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_VGA_1280X960P_60D000,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_VGA_1280X960P_75D000,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_VGA_1280X960P_85D002,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_VGA_1280X1024I_43D436,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_VGA_1280X1024P_60D020,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_VGA_1280X1024P_75D025,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_VGA_1280X1024P_85D024,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_VGA_1280X1024P_59D979,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_VGA_1280X1024P_72D005,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_VGA_1280X1024P_60D002,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_VGA_1280X1024P_67D003,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_VGA_1280X1024P_74D112,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_VGA_1280X1024P_76D179,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_VGA_1280X1024P_66D718,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_VGA_1280X1024P_66D677,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_VGA_1280X1024P_76D107,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_VGA_1280X1024P_59D996,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_VGA_1360X768P_59D799,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_VGA_1360X1024I_51D476,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_VGA_1440X1080P_60D000,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_VGA_1600X1200I_48D040,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_VGA_1600X1200P_60D000,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_VGA_1600X1200P_65D000,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_VGA_1600X1200P_70D000,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_VGA_1600X1200P_75D000,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_VGA_1600X1200P_80D000,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_VGA_1600X1200P_85D000,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_VGA_1600X1280P_66D931,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_VGA_1680X1080P_60D000,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_VGA_1792X1344P_60D000,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_VGA_1792X1344P_74D997,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_VGA_1856X1392P_59D995,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_VGA_1856X1392P_75D000,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_VGA_1868X1200P_75D000,
	//VDIN_SIG_FORMAT_VGA_1920X1080P_60D000,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_VGA_1920X1080P_75D000,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_VGA_1920X1080P_85D000,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_VGA_1920X1200P_84D932,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_VGA_1920X1200P_75D000,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_VGA_1920X1200P_85D000,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_VGA_1920X1234P_75D000,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_VGA_1920X1234P_85D000,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_VGA_1920X1440P_60D000,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_VGA_1920X1440P_75D000,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_VGA_2048X1536P_60D000_A,
	{
		0x00082222, //
		0x252b39c6, //
	},
	//VDIN_SIG_FORMAT_VGA_2048X1536P_75D000,
	{0,0},
	//VDIN_SIG_FORMAT_VGA_2048X1536P_60D000_B,
	{0,0},
	//VDIN_SIG_FORMAT_VGA_2048X2048P_60D008,
	{0,0},
	//TVIN_SIG_FMT_VGA_MAX,
	{0,0},

	///////////////////////////////////////////////////////////////
	//VDIN_SIG_FORMAT_COMPONENT_480P_60D000,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_COMPONENT_480I_59D940,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_COMPONENT_576P_50D000,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_COMPONENT_576I_50D000,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_COMPONENT_720P_59D940,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_COMPONENT_720P_50D000,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_COMPONENT_1080P_23D976,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_COMPONENT_1080P_24D000,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_COMPONENT_1080P_25D000,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_COMPONENT_1080P_30D000,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_COMPONENT_1080P_50D000,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_COMPONENT_1080P_60D000,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_COMPONENT_1080I_29D970,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_COMPONENT_1080I_47D952,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_COMPONENT_1080I_48D000,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_COMPONENT_1080I_50D000_A,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_COMPONENT_1080I_50D000_B,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_COMPONENT_1080I_50D000_C,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_COMPONENT_1080I_60D000,
	{
		0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//TVIN_SIG_FMT_COMP_MAX,
	{0,0},

	//VDIN_SIG_FORMAT_CVBS_NTSC_M,
	{       0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_CVBS_NTSC_443,
	{       0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_CVBS_PAL_I,
	{       0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_CVBS_PAL_M,
	{       0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_CVBS_PAL_60,
	{       0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_CVBS_PAL_CN,
	{       0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},
	//VDIN_SIG_FORMAT_CVBS_SECAM,
	{       0x00082222, // TVFE_AAFILTER_CTRL1
		0x252b39c6, // TVFE_AAFILTER_CTRL2
	},

	//VDIN_SIG_FORMAT_MAX,
	{0,0 }
};

// *****************************************************************************
// Function:set aafilter control
//
//   Params: format index
//
//   Return: none
//
// *****************************************************************************
void tvafe_top_set_aafilter_control(num tvin_sig_fmt_e fmt)
{

	W_APB_REG(TVFE_AAFILTER_CTRL1, aafilter_ctl[fmt][0]);
	W_APB_REG(TVFE_AAFILTER_CTRL2, aafilter_ctl[fmt][1]);


	return;
}

// *****************************************************************************
// Function:set bp gate of tvfe top module
//
//   Params: format index
//
//   Return: none
//
// *****************************************************************************
void tvafe_top_set_bp_gate(enum tvin_sig_fmt_e fmt)
{
	unsigned int h_bp_end,h_bp_start;
	unsigned int v_bp_end,v_bp_start;
	const struct tvin_format_s *fmt_info = tvin_get_fmt_info(fmt);

	h_bp_start = fmt_info->hs_width + 1;
	W_APB_BIT(TVFE_BPG_BACKP_H, h_bp_start, BACKP_H_ST_BIT, BACKP_H_ST_WID);

	h_bp_end = fmt_info->h_total - fmt_info->hs_front + 1;
	W_APB_BIT(TVFE_BPG_BACKP_H, h_bp_end, BACKP_H_ED_BIT, BACKP_H_ED_WID);

	v_bp_start = fmt_info->vs_width + 1;
	W_APB_BIT(TVFE_BPG_BACKP_V, v_bp_start, BACKP_V_ST_BIT, BACKP_V_ST_WID);

	v_bp_end = fmt_info->v_total - fmt_info->vs_front + 1;
	W_APB_BIT(TVFE_BPG_BACKP_V, v_bp_end, BACKP_V_ED_BIT, BACKP_V_ED_WID);

	return;
}

// *****************************************************************************
// Function:set mvdet control of tvfe module
//
//   Params: format index
//
//   Return: none
//
// *****************************************************************************
void tvafe_top_set_mvdet_control(enum tvin_sig_fmt_e fmt)
{
	unsigned int sd_mvd_reg_15_1b[] = {0, 0, 0, 0, 0, 0, 0,};

	if ((fmt > TVIN_SIG_FMT_COMP_480P_60D000)
			&& (fmt < TVIN_SIG_FMT_COMP_576I_50D000)) {
		W_APB_REG(TVFE_DVSS_MVDET_CTRL1, sd_mvd_reg_15_1b[0]);
		W_APB_REG(TVFE_DVSS_MVDET_CTRL2, sd_mvd_reg_15_1b[1]);
		W_APB_REG(TVFE_DVSS_MVDET_CTRL3, sd_mvd_reg_15_1b[2]);
		W_APB_REG(TVFE_DVSS_MVDET_CTRL4, sd_mvd_reg_15_1b[3]);
		W_APB_REG(TVFE_DVSS_MVDET_CTRL5, sd_mvd_reg_15_1b[4]);
		W_APB_REG(TVFE_DVSS_MVDET_CTRL6, sd_mvd_reg_15_1b[5]);
		W_APB_REG(TVFE_DVSS_MVDET_CTRL7, sd_mvd_reg_15_1b[6]);

	}

	return;
}
#endif

/*
 * tvafe wss setting
 */
void tvafe_top_set_wss_control(enum tvin_sig_fmt_e fmt)
{
	unsigned int hd_mvd_reg_2a_2d[] = {0, 0, 0, 0};

	if (fmt > TVIN_SIG_FMT_COMP_720P_59HZ_D940
			&& fmt < TVIN_SIG_FMT_COMP_1080I_60HZ_D000) {
		W_APB_REG(TVFE_MISC_WSS1_MUXCTRL1, hd_mvd_reg_2a_2d[0]);
		W_APB_REG(TVFE_MISC_WSS1_MUXCTRL2, hd_mvd_reg_2a_2d[1]);
		W_APB_REG(TVFE_MISC_WSS2_MUXCTRL1, hd_mvd_reg_2a_2d[2]);
		W_APB_REG(TVFE_MISC_WSS2_MUXCTRL2, hd_mvd_reg_2a_2d[3]);
	}

	return;
}

/*
 * tvafe top field generation setting
 */
void tvafe_top_set_sfg_mux_control(enum tvin_sig_fmt_e fmt)
{
	unsigned short h_total = 0,pixel_clk = 0,tmp = 0;

	if (fmt < TVIN_SIG_FMT_VGA_MAX)
	{
		h_total = tvin_vga_fmt_tbl[fmt-TVIN_SIG_FMT_VGA_512X384P_60HZ_D147].h_total;
		pixel_clk = tvin_vga_fmt_tbl[fmt-TVIN_SIG_FMT_VGA_512X384P_60HZ_D147].pixel_clk;
	}
	else
	{
		h_total = tvin_comp_fmt_tbl[fmt-TVIN_SIG_FMT_COMP_480P_60HZ_D000].h_total;
		pixel_clk = tvin_comp_fmt_tbl[fmt-TVIN_SIG_FMT_COMP_480P_60HZ_D000].pixel_clk;
	}

	if (pixel_clk)
	{
		//tmp = (h_total*219)/pixel_clk; // ((h_total*5M/(pixel_clk*10K))/2)*7/8 = h_total*219/pixel_clk
		//if (tmp > 255)
		//    tmp = 255;
		//W_APB_REG(ADC_REG_31, tmp);
		tmp = (h_total+2)>>2;
		W_APB_BIT(TVFE_SYNCTOP_SFG_MUXCTRL1, tmp, SFG_DET_HSTART_BIT, SFG_DET_HSTART_WID);
		tmp = (h_total+h_total+h_total+2)>>2;
		W_APB_BIT(TVFE_SYNCTOP_SFG_MUXCTRL1, tmp, SFG_DET_HEND_BIT, SFG_DET_HEND_WID);
	}

	return;
}

/*
 * tvafe top field generation setting
 * the parameter stdvalue  replace the parameter original
 */
void tvafe_reset_cal_value(struct tvafe_adc_cal_s *original,struct tvafe_adc_cal_s *stdvalue)
{
#ifdef LOG_ADC_CAL
	printk("++++++reset cal value\n");
#endif
	//a
	original->a_analog_clamp    = stdvalue->a_analog_clamp;
	original->a_analog_gain     = stdvalue->a_analog_gain;
	original->a_digital_offset1 = stdvalue->a_digital_offset1;
	//b
	original->b_analog_clamp    = stdvalue->b_analog_clamp;
	original->b_analog_gain     = stdvalue->b_analog_gain;
	original->b_digital_offset1 = stdvalue->b_digital_offset1;
	//c
	original->c_analog_clamp    = stdvalue->c_analog_clamp;
	original->c_analog_gain     = stdvalue->c_analog_gain;
	original->c_digital_offset1 = stdvalue->c_digital_offset1;
}

/*
 * compare calibration value (calvalue) with standard value (stdvalue)
 */
unsigned char tvafe_compare_cal_value(unsigned short calvalue,unsigned short stdvalue)
{
	unsigned char ret=0;
	if(calvalue > stdvalue)
	{
		if((calvalue - stdvalue) > threshold_value)
		{
			ret =1;
			return ret;
		}
	}
	else if((stdvalue - calvalue) > threshold_value)
	{
		ret=1;
		return ret;
	}
	return ret;

}

/*
 * if adc_cal value is not valid,then adc_cal value will been replaced
 */
void tvafe_adjust_cal_value(struct tvafe_adc_cal_s *para,bool iscomponent)
{
	unsigned char flag = 0;
	struct tvafe_adc_cal_s *stdvaluep = iscomponent ? (&cal_std_value_component) : (&cal_std_value_vga);

	flag |= tvafe_compare_cal_value(para->a_analog_clamp,   stdvaluep->a_analog_clamp   );
	flag |= tvafe_compare_cal_value(para->a_analog_gain,    stdvaluep->a_analog_gain    );

	flag |= tvafe_compare_cal_value(para->b_analog_clamp,   stdvaluep->b_analog_clamp   );
	flag |= tvafe_compare_cal_value(para->b_analog_gain,    stdvaluep->b_analog_gain    );

	flag |= tvafe_compare_cal_value(para->c_analog_clamp,   stdvaluep->c_analog_clamp   );
	flag |= tvafe_compare_cal_value(para->c_analog_gain,    stdvaluep->c_analog_gain    );

	if(flag)
		tvafe_reset_cal_value(para,stdvaluep);

}

void tvafe_set_regmap(struct am_regs_s *p)
{
    unsigned short i;

    for (i=0; i<p->length; i++) {
        switch (p->am_reg[i].type)
        {
            case REG_TYPE_PHY:
                #ifdef PQ_DEBUG_EN
                    pr_info("%s: bus type: phy..............\n", __func__);
                #endif
            break;
            case REG_TYPE_CBUS:
                if (p->am_reg[i].mask == 0xffffffff)
                    WRITE_CBUS_REG(p->am_reg[i].addr, p->am_reg[i].val);
                else
                    WRITE_CBUS_REG(p->am_reg[i].addr, (READ_CBUS_REG(p->am_reg[i].addr) & (~(p->am_reg[i].mask))) | (p->am_reg[i].val & p->am_reg[i].mask));
                #ifdef PQ_DEBUG_EN
					pr_info("%s: cbus: Reg0x%x(%u)=0x%x(%u)val=%x(%u)mask=%x(%u)\n", __func__, p->am_reg[i].addr,p->am_reg[i].addr,
					(p->am_reg[i].val & p->am_reg[i].mask),(p->am_reg[i].val & p->am_reg[i].mask),
					p->am_reg[i].val,p->am_reg[i].val,p->am_reg[i].mask,p->am_reg[i].mask);
                #endif
            break;
            case REG_TYPE_APB:
                if (p->am_reg[i].mask == 0xffffffff)
                    W_APB_REG(p->am_reg[i].addr<<2, p->am_reg[i].val);
                else
                    W_APB_REG(p->am_reg[i].addr<<2, (R_APB_REG(p->am_reg[i].addr<<2) & (~(p->am_reg[i].mask))) | (p->am_reg[i].val & p->am_reg[i].mask));
                #ifdef PQ_DEBUG_EN
					pr_info("%s: apb: Reg0x%x(%u)=0x%x(%u)val=%x(%u)mask=%x(%u)\n", __func__, p->am_reg[i].addr,p->am_reg[i].addr,
					(p->am_reg[i].val & p->am_reg[i].mask),(p->am_reg[i].val & p->am_reg[i].mask),
					p->am_reg[i].val,p->am_reg[i].val,p->am_reg[i].mask,p->am_reg[i].mask);
                #endif
            break;
            case REG_TYPE_MPEG:
                if (p->am_reg[i].mask == 0xffffffff)
                    WRITE_MPEG_REG(p->am_reg[i].addr, p->am_reg[i].val);
                else
                    WRITE_MPEG_REG(p->am_reg[i].addr, (READ_MPEG_REG(p->am_reg[i].addr) & (~(p->am_reg[i].mask))) | (p->am_reg[i].val & p->am_reg[i].mask));
                #ifdef PQ_DEBUG_EN
					pr_info("%s: mpeg: Reg0x%x(%u)=0x%x(%u)val=%x(%u)mask=%x(%u)\n", __func__, p->am_reg[i].addr,p->am_reg[i].addr,
					(p->am_reg[i].val & p->am_reg[i].mask),(p->am_reg[i].val & p->am_reg[i].mask),
					p->am_reg[i].val,p->am_reg[i].val,p->am_reg[i].mask,p->am_reg[i].mask);
                #endif
            break;
            case REG_TYPE_AXI:
                if (p->am_reg[i].mask == 0xffffffff)
                    WRITE_AXI_REG(p->am_reg[i].addr, p->am_reg[i].val);
                else
                    WRITE_AXI_REG(p->am_reg[i].addr, (READ_AXI_REG(p->am_reg[i].addr) & (~(p->am_reg[i].mask))) | (p->am_reg[i].val & p->am_reg[i].mask));
                #ifdef PQ_DEBUG_EN
					pr_info("%s: axi: Reg0x%x(%u)=0x%x(%u)val=%x(%u)mask=%x(%u)\n", __func__, p->am_reg[i].addr,p->am_reg[i].addr,
					(p->am_reg[i].val & p->am_reg[i].mask),(p->am_reg[i].val & p->am_reg[i].mask),
					p->am_reg[i].val,p->am_reg[i].val,p->am_reg[i].mask,p->am_reg[i].mask);
                #endif
            break;
            default:
            #ifdef PQ_DEBUG_EN
                pr_info("%s: bus type error!!!bustype = 0x%x................\n", __func__, p->am_reg[i].type);
            #endif
            break;
        }
    }

    return;
}
/*
 * tvafe set the calibartion value
 */
void tvafe_set_cal_value(struct tvafe_cal_s *cal)
{
	struct tvafe_adc_cal_s *para = &cal->cal_val;
	unsigned int clamp_h = 0, clamp_l = 0;

	if (!(para->reserved & TVAFE_ADC_CAL_VALID))
		return;

	W_APB_BIT(TVFE_OGO_OFFSET1, 1, OGO_EN_BIT, OGO_EN_WID);

	clamp_h = (para->a_analog_clamp)>>2;
	clamp_l = (para->a_analog_clamp)&3;
	W_APB_BIT(ADC_REG_0B, clamp_h, CTRCLREFA_0B_BIT, CTRCLREFA_0B_WID);
	W_APB_BIT(ADC_REG_0F, clamp_l, CTRCLREFA_0F_BIT, CTRCLREFA_0F_WID);
	clamp_h = (para->b_analog_clamp)>>2;
	clamp_l = (para->b_analog_clamp)&3;
	W_APB_BIT(ADC_REG_0C, clamp_h, CTRCLREFB_0C_BIT, CTRCLREFB_0C_WID);
	W_APB_BIT(ADC_REG_10, clamp_l, CTRCLREFB_10_BIT, CTRCLREFB_10_WID);
	clamp_h = (para->c_analog_clamp)>>2;
	clamp_l = (para->c_analog_clamp)&3;
	W_APB_BIT(ADC_REG_0D, clamp_h, CTRCLREFC_0D_BIT, CTRCLREFC_0D_WID);
	W_APB_BIT(ADC_REG_11, clamp_l, CTRCLREFC_11_BIT, CTRCLREFC_11_WID);

	W_APB_BIT(ADC_REG_07, para->a_analog_gain, ADCGAINA_BIT, ADCGAINA_WID);
	W_APB_BIT(ADC_REG_08, para->b_analog_gain, ADCGAINB_BIT, ADCGAINB_WID);
	W_APB_BIT(ADC_REG_09, para->c_analog_gain, ADCGAINC_BIT, ADCGAINC_WID);

	W_APB_BIT(TVFE_OGO_OFFSET1, para->a_digital_offset1,
			OGO_YG_OFFSET1_BIT, OGO_YG_OFFSET1_WID);
	W_APB_BIT(TVFE_OGO_OFFSET1, para->b_digital_offset1,
			OGO_UB_OFFSET1_BIT, OGO_UB_OFFSET1_WID);
	W_APB_BIT(TVFE_OGO_OFFSET3, para->c_digital_offset1,
			OGO_VR_OFFSET1_BIT, OGO_VR_OFFSET1_WID);

	W_APB_BIT(TVFE_OGO_GAIN1, para->a_digital_gain,
			OGO_YG_GAIN_BIT, OGO_YG_GAIN_WID);
	W_APB_BIT(TVFE_OGO_GAIN1, para->b_digital_gain,
			OGO_UB_GAIN_BIT, OGO_UB_GAIN_WID);
	W_APB_BIT(TVFE_OGO_GAIN2, para->c_digital_gain,
			OGO_VR_GAIN_BIT, OGO_VR_GAIN_WID);

	W_APB_BIT(TVFE_OGO_OFFSET2, para->a_digital_offset2,
			OGO_YG_OFFSET2_BIT, OGO_YG_OFFSET2_WID);
	W_APB_BIT(TVFE_OGO_OFFSET2, para->b_digital_offset2,
			OGO_UB_OFFSET2_BIT, OGO_UB_OFFSET2_WID);
	W_APB_BIT(TVFE_OGO_OFFSET3, para->c_digital_offset2,
			OGO_VR_OFFSET2_BIT, OGO_VR_OFFSET2_WID);

}

void tvafe_set_cal_value2(struct tvafe_adc_cal_s *cal)
{
    struct tvafe_adc_cal_s *para = cal;
    unsigned int clamp_h = 0, clamp_l = 0;

    if (!(para->reserved & TVAFE_ADC_CAL_VALID))
        return;

    W_APB_BIT(TVFE_OGO_OFFSET1, 1, OGO_EN_BIT, OGO_EN_WID);

    clamp_h = (para->a_analog_clamp)>>2;
    clamp_l = (para->a_analog_clamp)&3;
    W_APB_BIT(ADC_REG_0B, clamp_h, CTRCLREFA_0B_BIT, CTRCLREFA_0B_WID);
    W_APB_BIT(ADC_REG_0F, clamp_l, CTRCLREFA_0F_BIT, CTRCLREFA_0F_WID);
    clamp_h = (para->b_analog_clamp)>>2;
    clamp_l = (para->b_analog_clamp)&3;
    W_APB_BIT(ADC_REG_0C, clamp_h, CTRCLREFB_0C_BIT, CTRCLREFB_0C_WID);
    W_APB_BIT(ADC_REG_10, clamp_l, CTRCLREFB_10_BIT, CTRCLREFB_10_WID);
    clamp_h = (para->c_analog_clamp)>>2;
    clamp_l = (para->c_analog_clamp)&3;
    W_APB_BIT(ADC_REG_0D, clamp_h, CTRCLREFC_0D_BIT, CTRCLREFC_0D_WID);
    W_APB_BIT(ADC_REG_11, clamp_l, CTRCLREFC_11_BIT, CTRCLREFC_11_WID);

    W_APB_BIT(ADC_REG_07, para->a_analog_gain, ADCGAINA_BIT, ADCGAINA_WID);
    W_APB_BIT(ADC_REG_08, para->b_analog_gain, ADCGAINB_BIT, ADCGAINB_WID);
    W_APB_BIT(ADC_REG_09, para->c_analog_gain, ADCGAINC_BIT, ADCGAINC_WID);

    W_APB_BIT(TVFE_OGO_OFFSET1, para->a_digital_offset1,
        OGO_YG_OFFSET1_BIT, OGO_YG_OFFSET1_WID);
    W_APB_BIT(TVFE_OGO_OFFSET1, para->b_digital_offset1,
        OGO_UB_OFFSET1_BIT, OGO_UB_OFFSET1_WID);
    W_APB_BIT(TVFE_OGO_OFFSET3, para->c_digital_offset1,
        OGO_VR_OFFSET1_BIT, OGO_VR_OFFSET1_WID);

    W_APB_BIT(TVFE_OGO_GAIN1, para->a_digital_gain,
        OGO_YG_GAIN_BIT, OGO_YG_GAIN_WID);
    W_APB_BIT(TVFE_OGO_GAIN1, para->b_digital_gain,
        OGO_UB_GAIN_BIT, OGO_UB_GAIN_WID);
    W_APB_BIT(TVFE_OGO_GAIN2, para->c_digital_gain,
        OGO_VR_GAIN_BIT, OGO_VR_GAIN_WID);

    W_APB_BIT(TVFE_OGO_OFFSET2, para->a_digital_offset2,
        OGO_YG_OFFSET2_BIT, OGO_YG_OFFSET2_WID);
    W_APB_BIT(TVFE_OGO_OFFSET2, para->b_digital_offset2,
        OGO_UB_OFFSET2_BIT, OGO_UB_OFFSET2_WID);
    W_APB_BIT(TVFE_OGO_OFFSET3, para->c_digital_offset2,
        OGO_VR_OFFSET2_BIT, OGO_VR_OFFSET2_WID);

}


/*
 * tvafe get the calibration value from Reg
 */
int tvafe_get_cal_value(struct tvafe_cal_s *cal)
{
	struct tvafe_adc_cal_s *para = &cal->cal_val;
	unsigned int clamp_h = 0, clamp_l = 0;

	clamp_h = R_APB_BIT(ADC_REG_0B, CTRCLREFA_0B_BIT, CTRCLREFA_0B_WID);
	clamp_l = R_APB_BIT(ADC_REG_0F, CTRCLREFA_0F_BIT, CTRCLREFA_0F_WID);
	para->a_analog_clamp = (clamp_h<<2)|clamp_l;
	clamp_h = R_APB_BIT(ADC_REG_0C, CTRCLREFB_0C_BIT, CTRCLREFB_0C_WID);
	clamp_l = R_APB_BIT(ADC_REG_10, CTRCLREFB_10_BIT, CTRCLREFB_10_WID);
	para->b_analog_clamp = (clamp_h<<2)|clamp_l;
	clamp_h = R_APB_BIT(ADC_REG_0D, CTRCLREFC_0D_BIT, CTRCLREFC_0D_WID);
	clamp_l = R_APB_BIT(ADC_REG_11, CTRCLREFC_11_BIT, CTRCLREFC_11_WID);
	para->c_analog_clamp = (clamp_h<<2)|clamp_l;

	para->a_analog_gain = R_APB_BIT(ADC_REG_07, ADCGAINA_BIT, ADCGAINA_WID);
	para->b_analog_gain = R_APB_BIT(ADC_REG_08, ADCGAINB_BIT, ADCGAINB_WID);
	para->c_analog_gain = R_APB_BIT(ADC_REG_09, ADCGAINC_BIT, ADCGAINC_WID);

	para->a_digital_offset1 = R_APB_BIT(TVFE_OGO_OFFSET1,
			OGO_YG_OFFSET1_BIT, OGO_YG_OFFSET1_WID);
	para->b_digital_offset1 = R_APB_BIT(TVFE_OGO_OFFSET1,
			OGO_UB_OFFSET1_BIT, OGO_UB_OFFSET1_WID);
	para->c_digital_offset1 = R_APB_BIT(TVFE_OGO_OFFSET3,
			OGO_VR_OFFSET1_BIT, OGO_VR_OFFSET1_WID);

	para->a_digital_gain = R_APB_BIT(TVFE_OGO_GAIN1,
			OGO_YG_GAIN_BIT, OGO_YG_GAIN_WID);
	para->b_digital_gain = R_APB_BIT(TVFE_OGO_GAIN1,
			OGO_UB_GAIN_BIT, OGO_UB_GAIN_WID);
	para->c_digital_gain = R_APB_BIT(TVFE_OGO_GAIN2,
			OGO_VR_GAIN_BIT, OGO_VR_GAIN_WID);

	para->a_digital_offset2 = R_APB_BIT(TVFE_OGO_OFFSET2,
			OGO_YG_OFFSET2_BIT, OGO_YG_OFFSET2_WID);
	para->b_digital_offset2 = R_APB_BIT(TVFE_OGO_OFFSET2,
			OGO_UB_OFFSET2_BIT, OGO_UB_OFFSET2_WID);
	para->c_digital_offset2 = R_APB_BIT(TVFE_OGO_OFFSET3,
			OGO_VR_OFFSET2_BIT, OGO_VR_OFFSET2_WID);
	/*protect for adc calibriation failed*/
	if( 10 > para->a_analog_clamp ||120 < para->a_analog_clamp ||
			10 > para->b_analog_clamp ||120 < para->b_analog_clamp ||
			10 > para->c_analog_clamp ||120 < para->c_analog_clamp
	  )
	{
		pr_info("[%s]:adc calibriation error.a_analog_clamp:%u.a_analog_clamp:%u. a_analog_clamp:%u.\n",
				__func__,para->a_analog_clamp,para->b_analog_clamp,para->c_analog_clamp);
		return -EFAULT;
	}
	else
		return 0;
}

int tvafe_get_cal_value2(struct tvafe_adc_cal_s *cal)
{
    struct tvafe_adc_cal_s *para = cal;
    unsigned int clamp_h = 0, clamp_l = 0;

    clamp_h = R_APB_BIT(ADC_REG_0B, CTRCLREFA_0B_BIT, CTRCLREFA_0B_WID);
    clamp_l = R_APB_BIT(ADC_REG_0F, CTRCLREFA_0F_BIT, CTRCLREFA_0F_WID);
    para->a_analog_clamp = (clamp_h<<2)|clamp_l;
    clamp_h = R_APB_BIT(ADC_REG_0C, CTRCLREFB_0C_BIT, CTRCLREFB_0C_WID);
    clamp_l = R_APB_BIT(ADC_REG_10, CTRCLREFB_10_BIT, CTRCLREFB_10_WID);
    para->b_analog_clamp = (clamp_h<<2)|clamp_l;
    clamp_h = R_APB_BIT(ADC_REG_0D, CTRCLREFC_0D_BIT, CTRCLREFC_0D_WID);
    clamp_l = R_APB_BIT(ADC_REG_11, CTRCLREFC_11_BIT, CTRCLREFC_11_WID);
    para->c_analog_clamp = (clamp_h<<2)|clamp_l;

    para->a_analog_gain = R_APB_BIT(ADC_REG_07, ADCGAINA_BIT, ADCGAINA_WID);
    para->b_analog_gain = R_APB_BIT(ADC_REG_08, ADCGAINB_BIT, ADCGAINB_WID);
    para->c_analog_gain = R_APB_BIT(ADC_REG_09, ADCGAINC_BIT, ADCGAINC_WID);

    para->a_digital_offset1 = R_APB_BIT(TVFE_OGO_OFFSET1,
        OGO_YG_OFFSET1_BIT, OGO_YG_OFFSET1_WID);
    para->b_digital_offset1 = R_APB_BIT(TVFE_OGO_OFFSET1,
        OGO_UB_OFFSET1_BIT, OGO_UB_OFFSET1_WID);
    para->c_digital_offset1 = R_APB_BIT(TVFE_OGO_OFFSET3,
        OGO_VR_OFFSET1_BIT, OGO_VR_OFFSET1_WID);

    para->a_digital_gain = R_APB_BIT(TVFE_OGO_GAIN1,
        OGO_YG_GAIN_BIT, OGO_YG_GAIN_WID);
    para->b_digital_gain = R_APB_BIT(TVFE_OGO_GAIN1,
        OGO_UB_GAIN_BIT, OGO_UB_GAIN_WID);
    para->c_digital_gain = R_APB_BIT(TVFE_OGO_GAIN2,
        OGO_VR_GAIN_BIT, OGO_VR_GAIN_WID);

    para->a_digital_offset2 = R_APB_BIT(TVFE_OGO_OFFSET2,
        OGO_YG_OFFSET2_BIT, OGO_YG_OFFSET2_WID);
    para->b_digital_offset2 = R_APB_BIT(TVFE_OGO_OFFSET2,
        OGO_UB_OFFSET2_BIT, OGO_UB_OFFSET2_WID);
    para->c_digital_offset2 = R_APB_BIT(TVFE_OGO_OFFSET3,
        OGO_VR_OFFSET2_BIT, OGO_VR_OFFSET2_WID);
    /*protect for adc calibriation failed*/
    if( 10 > para->a_analog_clamp ||120 < para->a_analog_clamp ||
        10 > para->b_analog_clamp ||120 < para->b_analog_clamp ||
        10 > para->c_analog_clamp ||120 < para->c_analog_clamp
      )
    {
        pr_info("[%s]:adc calibriation error.a_analog_clamp:%u.a_analog_clamp:%u. a_analog_clamp:%u.\n",
                    __func__,para->a_analog_clamp,para->b_analog_clamp,para->c_analog_clamp);
        return -EFAULT;
    }
    else
        return 0;
}


/*
 * tvafe read the calibration value from adc readback module
 */
void tvafe_adc_cal_read(unsigned char *ch, unsigned int *a, unsigned int *b, unsigned int *c)
{
	unsigned int i = 0, j = 0, data = 0, da[16], db[16], dc[16];

	for (i = 0; i < 16; i++)
	{
		data = R_APB_REG(TVFE_ADC_READBACK_INDICATOR);
		da[i] = (data >> ADC_READBACK_DA_BIT) & ((1 << ADC_READBACK_DA_WID) - 1);
		db[i] = (data >> ADC_READBACK_DB_BIT) & ((1 << ADC_READBACK_DB_WID) - 1);
		dc[i] = (data >> ADC_READBACK_DC_BIT) & ((1 << ADC_READBACK_DC_WID) - 1);
	}
	for (i = 0; i < 15; i++)
	{
		for (j = i + 1; j < 16; j++)
		{
			if (da[i] > da[j])
			{
				data  = da[i];
				da[i] = da[j];
				da[j] = data;
			}
			if (db[i] > db[j])
			{
				data  = db[i];
				db[i] = db[j];
				db[j] = data;
			}
			if (dc[i] > dc[j])
			{
				data  = dc[i];
				dc[i] = dc[j];
				dc[j] = data;
			}
		}
	}
	*a = *b = *c = 4;
	for (i = 4; i < 12; i++)
	{
		*a += da[i];
		*b += db[i];
		*c += dc[i];
	}
	*a >>= 3;
	*b >>= 3;
	*c >>= 3;
#ifdef LOG_ADC_CAL
	pr_info("%s = %4d %4d %4d\n", ch, (int)*a, (int)*b, (int)*c);
#endif
}

/*
 * if adc_cal value is not valid,then adc_cal value will been replaced
 *  for Y:
 *  diff*0.777V/(479/1023)V <= 1023, diff <=616
 *  standard is 0.7V, to support -4% ~ +107% supper black/white, increase the range to 111% of 0.7V
 *  for Cb/Cr/R/G/B:
 *  diff*0.7V/(479/1023)V <= 1023, diff <= 684
 *  standard is 0.7V
 */
bool tvafe_cal_gain_overflow(unsigned int a1, unsigned int a2, bool is_component)
{
	unsigned int win = is_component ? 578 : 630;//616 : 684;add a patch for adc gain overflow

	if ((a2 - a1) > win)
		return (true);
	else
		return (false);
}

/*
 * standard is 16~235, supper black/white is 7~250
 */
bool tvafe_adc_cal(struct tvin_parm_s *parm, struct tvafe_cal_s *cal)
{
	enum tvin_port_e port = parm->port;
	struct tvafe_cal_operand_s *operand = &cal->cal_operand;
	struct tvafe_adc_cal_s *adc_cal = &cal->cal_val;
	bool is_component = (port >= TVIN_PORT_COMP0) && (port <= TVIN_PORT_COMP7);
	unsigned int a = 0, b = 0, c = 0, mutex = 0;
	unsigned int z_lu = is_component ?  37 :    0; //  37 = 1023*4%/111%
	unsigned int z_ch = is_component ? 512 :    0;
	unsigned int l_dg = is_component ? 975 : 1024; // 976 = 219*111%*4+3
	unsigned int l_do = is_component ?  28 :    0; //  28 = 7*4
	unsigned int c_dg = is_component ? 899 : 1024; // 899 = 224*4+3
	unsigned int c_do = is_component ?  64 :    0; //  64 = 16*4
	unsigned short step_up = 0, step_dn = 0;

	if(is_component){
		adc_cal = &(cal->fmt_cal_val.comp_cal_val[0]);
		operand->cal_fmt_max = 3;
	}else{
		adc_cal = &(cal->cal_val);
		operand->cal_fmt_max = 1;
	}
	switch (operand->step++)
	{
		case TVAFE_ADC_CAL_STEP_GAIN*0:
			// init
            adc_cal[operand->cal_fmt_cnt].a_analog_clamp    =   64;
            adc_cal[operand->cal_fmt_cnt].a_analog_gain     =  128;
            adc_cal[operand->cal_fmt_cnt].a_digital_gain    = l_dg; // 0dB
            adc_cal[operand->cal_fmt_cnt].a_digital_offset1 =    0;
            adc_cal[operand->cal_fmt_cnt].a_digital_offset2 = l_do;
            adc_cal[operand->cal_fmt_cnt].b_analog_clamp    =   64;
            adc_cal[operand->cal_fmt_cnt].b_analog_gain     =  128;
            adc_cal[operand->cal_fmt_cnt].b_digital_gain    = c_dg; // 0dB
            adc_cal[operand->cal_fmt_cnt].b_digital_offset1 =    0;
            adc_cal[operand->cal_fmt_cnt].b_digital_offset2 = c_do;
            adc_cal[operand->cal_fmt_cnt].c_analog_clamp    =   64;
            adc_cal[operand->cal_fmt_cnt].c_analog_gain     =  128;
            adc_cal[operand->cal_fmt_cnt].c_digital_gain    = c_dg; // 0dB
            adc_cal[operand->cal_fmt_cnt].c_digital_offset1 =    0;
            adc_cal[operand->cal_fmt_cnt].c_digital_offset2 = c_do;
			// config readback
			W_APB_BIT(TVFE_ADC_READBACK_CTRL, 0,
					ADC_READBACK_MODE_BIT , ADC_READBACK_MODE_WID );
			// record bpg_h, bpg_v, bpg_m, clamp_inv, clamp_ext, clk_ext, clk_ctl, lpf_a, lpf_b, lpf_c
			operand->bpg_h     = R_APB_REG(TVFE_BPG_BACKP_H);
			operand->bpg_v     = R_APB_REG(TVFE_BPG_BACKP_V);
			operand->bpg_m     = R_APB_BIT(TVFE_TOP_CTRL,
					TVFE_BACKP_GATE_MUX_BIT, TVFE_BACKP_GATE_MUX_WID);
			operand->clamp_inv = R_APB_BIT(TVFE_DVSS_MUXCTRL,
					DVSS_CLAMP_INV_BIT,      DVSS_CLAMP_INV_WID     );
			operand->clamp_ext = R_APB_BIT(ADC_REG_2F,
					CLAMPEXT_BIT,            CLAMPEXT_WID           );
			//  operand->lpf_a     = R_APB_BIT(ADC_REG_19, ENLPFA_BIT,    ENLPFA_WID   );
			//  operand->lpf_b     = R_APB_BIT(ADC_REG_1A, ENLPFB_BIT,    ENLPFB_WID   );
			//  operand->lpf_c     = R_APB_BIT(ADC_REG_1B, ENLPFC_BIT,    ENLPFC_WID   );
#ifdef CONFIG_ADC_CAL_SIGNALED
			operand->pin_a_mux = R_APB_BIT(ADC_REG_17,
			        INMUXA_BIT, INMUXA_WID);
			operand->pin_b_mux = R_APB_BIT(ADC_REG_17,
			        INMUXB_BIT, INMUXB_WID);
			operand->pin_c_mux = R_APB_BIT(ADC_REG_18,
					INMUXC_BIT, INMUXC_WID);
			//operand->sog_mux   = R_APB_BIT(ADC_REG_24,
			//		INMUXSOG_BIT, INMUXSOG_WID);
#endif
			operand->clk_ext   = R_APB_BIT(ADC_REG_58,
					EXTCLKSEL_BIT, EXTCLKSEL_WID);
			operand->clk_ctl   = READ_CBUS_REG(HHI_VAFE_CLKIN_CNTL);
			operand->vafe_ctl  = R_APB_REG(TVFE_VAFE_CTRL);
			operand->sync_mux  = R_APB_BIT(ADC_REG_39,INSYNCMUXCTRL_BIT,INSYNCMUXCTRL_WID);
			// disable lpf_a, lpf_b, lpf_c
			//  W_APB_BIT(ADC_REG_19, 0, ENLPFA_BIT, ENLPFA_WID);
			//  W_APB_BIT(ADC_REG_1A, 0, ENLPFB_BIT, ENLPFB_WID);
			//  W_APB_BIT(ADC_REG_1B, 0, ENLPFC_BIT, ENLPFC_WID);
#ifdef CONFIG_ADC_CAL_SIGNALED
			// change the pinmux
			W_APB_BIT(ADC_REG_17, 1, INMUXA_BIT, INMUXA_WID);
			W_APB_BIT(ADC_REG_17, 1, INMUXB_BIT, INMUXB_WID);
			W_APB_BIT(ADC_REG_18, 1, INMUXC_BIT, INMUXC_WID);
			//W_APB_BIT(ADC_REG_24, 1, INMUXSOG_BIT, INMUXSOG_WID);
#endif
			// set clk_ext & clk_ctl
			W_APB_BIT(ADC_REG_58, 1, EXTCLKSEL_BIT, EXTCLKSEL_WID);
			if (!is_component){
				WRITE_CBUS_REG(HHI_VAFE_CLKIN_CNTL , 0x00000100); //vga
			}else{
				if(operand->cal_fmt_cnt == 0){
	            	WRITE_CBUS_REG(HHI_VAFE_CLKIN_CNTL , 0x00000b00); //1080p
	            }else if(operand->cal_fmt_cnt == 1){
					WRITE_CBUS_REG(HHI_VAFE_CLKIN_CNTL , 0x00000b01); //720p,1080i
				}else{
					WRITE_CBUS_REG(HHI_VAFE_CLKIN_CNTL , 0x00000100); //480,576
				}
			}
			// set vfe mux
			W_APB_BIT(ADC_REG_39, 1, INSYNCMUXCTRL_BIT, INSYNCMUXCTRL_WID);
			W_APB_BIT(TVFE_VAFE_CTRL, 1,
					VAFE_HS_VS_MUX_BIT, VAFE_HS_VS_MUX_WID);
			// load adc cal values
            adc_cal[operand->cal_fmt_cnt].reserved |= TVAFE_ADC_CAL_VALID;
            tvafe_set_cal_value2(&adc_cal[operand->cal_fmt_cnt]);
            adc_cal[operand->cal_fmt_cnt].reserved &= ~TVAFE_ADC_CAL_VALID;
			// disable offset calibration
			W_APB_BIT(TVFE_VAFE_CTRL, 0,
					VAFE_ENOFFSETCAL_BIT, VAFE_ENOFFSETCAL_WID);
			// enable gain calibration
			W_APB_BIT(TVFE_VAFE_CTRL, 1,
					VAFE_ENGAINCAL_BIT, VAFE_ENGAINCAL_WID);
			// select level 1
			W_APB_BIT(TVFE_VAFE_CTRL, 1,
					VAFE_SELGAINCALLVL_BIT, VAFE_SELGAINCALLVL_WID);
			break;
		case TVAFE_ADC_CAL_STEP_GAIN*1:
		case TVAFE_ADC_CAL_STEP_GAIN*3:
		case TVAFE_ADC_CAL_STEP_GAIN*5:
		case TVAFE_ADC_CAL_STEP_GAIN*7:
		case TVAFE_ADC_CAL_STEP_GAIN*9:
		case TVAFE_ADC_CAL_STEP_GAIN*11:
		case TVAFE_ADC_CAL_STEP_GAIN*13:
		case TVAFE_ADC_CAL_STEP_GAIN*15:
#ifdef LOG_ADC_CAL
                pr_info("\nA_gn = %4d %4d %4d\n", (int)adc_cal[operand->cal_fmt_cnt].a_analog_gain,
                    (int)adc_cal[operand->cal_fmt_cnt].b_analog_gain, (int)adc_cal[operand->cal_fmt_cnt].c_analog_gain);
            #endif
			// get data of level 1
			tvafe_adc_cal_read("Lvl1", &operand->a, &operand->b, &operand->c);
			// select level 2
			W_APB_BIT(TVFE_VAFE_CTRL, 2,
					VAFE_SELGAINCALLVL_BIT, VAFE_SELGAINCALLVL_WID);
			break;
		case TVAFE_ADC_CAL_STEP_GAIN*2:
			if (!mutex)
			{
				mutex   =  1;
				step_up = 64;
				step_dn = 64;
			}
		case TVAFE_ADC_CAL_STEP_GAIN*4:
			if (!mutex)
			{
				mutex   =  1;
				step_up = 32;
				step_dn = 32;
			}
		case TVAFE_ADC_CAL_STEP_GAIN*6:
			if (!mutex)
			{
				mutex   =  1;
				step_up = 16;
				step_dn = 16;
			}
		case TVAFE_ADC_CAL_STEP_GAIN*8:
			if (!mutex)
			{
				mutex   = 1;
				step_up = 8;
				step_dn = 8;
			}
		case TVAFE_ADC_CAL_STEP_GAIN*10:
			if (!mutex)
			{
				mutex   = 1;
				step_up = 4;
				step_dn = 4;
			}
		case TVAFE_ADC_CAL_STEP_GAIN*12:
			if (!mutex)
			{
				mutex   = 1;
				step_up = 2;
				step_dn = 2;
			}
		case TVAFE_ADC_CAL_STEP_GAIN*14:
			if (!mutex)
			{
				mutex   = 1;
				step_up = 1;
				step_dn = 1;
			}
		case TVAFE_ADC_CAL_STEP_GAIN*16:
			if (!mutex)
			{
				mutex   = 1;
				step_up = 0;
				step_dn = 1;
			}
			// get data of level 2
			tvafe_adc_cal_read("Lvl2", &a, &b, &c);
#ifdef LOG_ADC_CAL
			pr_info("Diff = %4d %4d %4d\n", (int)(a - operand->a),
					(int)(b - operand->b), (int)(c - operand->c));
#endif
			// tune analog gain
			if (tvafe_cal_gain_overflow(operand->a, a, is_component))
                adc_cal[operand->cal_fmt_cnt].a_analog_gain -= step_dn;
            else
                adc_cal[operand->cal_fmt_cnt].a_analog_gain += step_up;
            if (tvafe_cal_gain_overflow(operand->b, b, false))
                adc_cal[operand->cal_fmt_cnt].b_analog_gain -= step_dn;
            else
                adc_cal[operand->cal_fmt_cnt].b_analog_gain += step_up;
            if (tvafe_cal_gain_overflow(operand->c, c, false))
                adc_cal[operand->cal_fmt_cnt].c_analog_gain -= step_dn;
            else
                adc_cal[operand->cal_fmt_cnt].c_analog_gain += step_up;
            // load adc cal values
            adc_cal[operand->cal_fmt_cnt].reserved |= TVAFE_ADC_CAL_VALID;
            tvafe_set_cal_value2( &adc_cal[operand->cal_fmt_cnt]);
            adc_cal[operand->cal_fmt_cnt].reserved &= ~TVAFE_ADC_CAL_VALID;
			// select level 1
			W_APB_BIT(TVFE_VAFE_CTRL, 1,
					VAFE_SELGAINCALLVL_BIT, VAFE_SELGAINCALLVL_WID);
			if (operand->step == TVAFE_ADC_CAL_STEP_GAIN*16 + 1)
			{
#ifdef LOG_ADC_CAL
                    pr_info("\nfresh_adc_cal after gain\n");
                    pr_info("A_cl = %4d %4d %4d\n",  (int)(adc_cal[operand->cal_fmt_cnt].a_analog_clamp),
                                                     (int)(adc_cal[operand->cal_fmt_cnt].b_analog_clamp),
                                                     (int)(adc_cal[operand->cal_fmt_cnt].c_analog_clamp));
                    pr_info("A_gn = %4d %4d %4d\n",  (int)(adc_cal[operand->cal_fmt_cnt].a_analog_gain),
                                                     (int)(adc_cal[operand->cal_fmt_cnt].b_analog_gain),
                                                     (int)(adc_cal[operand->cal_fmt_cnt].c_analog_gain));
                    pr_info("D_gn = %4d %4d %4d\n",  (int)(adc_cal[operand->cal_fmt_cnt].a_digital_gain),
                                                     (int)(adc_cal[operand->cal_fmt_cnt].b_digital_gain),
                                                     (int)(adc_cal[operand->cal_fmt_cnt].c_digital_gain));
                    pr_info("D_o1 = %4d %4d %4d\n", ((int)(adc_cal[operand->cal_fmt_cnt].a_digital_offset1) << 21) >> 21,
                                                    ((int)(adc_cal[operand->cal_fmt_cnt].b_digital_offset1) << 21) >> 21,
                                                    ((int)(adc_cal[operand->cal_fmt_cnt].c_digital_offset1) << 21) >> 21);
                    pr_info("D_o2 = %4d %4d %4d\n", ((int)(adc_cal[operand->cal_fmt_cnt].a_digital_offset2) << 21) >> 21,
                                                    ((int)(adc_cal[operand->cal_fmt_cnt].b_digital_offset2) << 21) >> 21,
                                                    ((int)(adc_cal[operand->cal_fmt_cnt].c_digital_offset2) << 21) >> 21);
				pr_info("\n");
#endif

				// disable gain calibration
#ifdef CONFIG_ADC_CAL_SIGNALED
				W_APB_REG(TVFE_VAFE_CTRL, operand->vafe_ctl);
#endif
				W_APB_BIT(TVFE_VAFE_CTRL, 0, VAFE_ENGAINCAL_BIT, VAFE_ENGAINCAL_WID);
				// set bpg_h, bpg_v, bpg_m, clamp_inv, clamp_ext
				W_APB_REG(TVFE_BPG_BACKP_H, 1);
				W_APB_REG(TVFE_BPG_BACKP_V, 1);
				//W_APB_BIT(TVFE_TOP_CTRL,     1, TVFE_BACKP_GATE_MUX_BIT, TVFE_BACKP_GATE_MUX_WID);
				//W_APB_BIT(TVFE_DVSS_MUXCTRL, 1, DVSS_CLAMP_INV_BIT,      DVSS_CLAMP_INV_WID     );
				W_APB_BIT(ADC_REG_2F, 1, CLAMPEXT_BIT, CLAMPEXT_WID);
				W_APB_BIT(ADC_REG_58, 1, EXTCLKSEL_BIT, EXTCLKSEL_WID);
#ifdef CONFIG_ADC_CAL_SIGNALED
				// recover pinmux
				W_APB_BIT(ADC_REG_17, operand->pin_a_mux, INMUXA_BIT, INMUXA_WID);
				W_APB_BIT(ADC_REG_17, operand->pin_b_mux, INMUXB_BIT, INMUXB_WID);
				W_APB_BIT(ADC_REG_18, operand->pin_c_mux, INMUXC_BIT, INMUXC_WID);
				WRITE_CBUS_REG(HHI_VAFE_CLKIN_CNTL, operand->clk_ctl);
				W_APB_BIT(ADC_REG_39, operand->sync_mux, INSYNCMUXCTRL_BIT,INSYNCMUXCTRL_WID);
#endif
			}
			break;
		case TVAFE_ADC_CAL_STEP_STAGE + TVAFE_ADC_CAL_STEP_CLAMP*1:
			if (!mutex)
			{
				mutex   =  1;
				step_up = 32;
				step_dn = 32;
			}
		case TVAFE_ADC_CAL_STEP_STAGE + TVAFE_ADC_CAL_STEP_CLAMP*2:
			if (!mutex)
			{
				mutex   =  1;
				step_up = 16;
				step_dn = 16;
			}
		case TVAFE_ADC_CAL_STEP_STAGE + TVAFE_ADC_CAL_STEP_CLAMP*3:
			if (!mutex)
			{
				mutex   = 1;
				step_up = 8;
				step_dn = 8;
			}
		case TVAFE_ADC_CAL_STEP_STAGE + TVAFE_ADC_CAL_STEP_CLAMP*4:
			if (!mutex)
			{
				mutex   = 1;
				step_up = 4;
				step_dn = 4;
			}
		case TVAFE_ADC_CAL_STEP_STAGE + TVAFE_ADC_CAL_STEP_CLAMP*5:
			if (!mutex)
			{
				mutex   = 1;
				step_up = 2;
				step_dn = 2;
			}
		case TVAFE_ADC_CAL_STEP_STAGE + TVAFE_ADC_CAL_STEP_CLAMP*6:
			if (!mutex)
			{
				mutex   = 1;
				step_up = 1;
				step_dn = 1;
			}
		case TVAFE_ADC_CAL_STEP_STAGE + TVAFE_ADC_CAL_STEP_CLAMP*7:
			if (!mutex)
			{
				mutex   = 1;
				step_up = 0;
				step_dn = 1;
			}
#ifdef LOG_ADC_CAL
                pr_info("\nA_cl = %4d %4d %4d\n", (int)adc_cal[operand->cal_fmt_cnt].a_analog_clamp,
                    (int)adc_cal[operand->cal_fmt_cnt].b_analog_clamp, (int)adc_cal[operand->cal_fmt_cnt].c_analog_clamp);
#endif
			// get data of blank
			tvafe_adc_cal_read("Dark", &a, &b, &c);
			// tune analog clamp
			if (a > z_lu)
                adc_cal[operand->cal_fmt_cnt].a_analog_clamp -= step_dn;
			else
                adc_cal[operand->cal_fmt_cnt].a_analog_clamp += step_up;
			if (b > z_ch)
                adc_cal[operand->cal_fmt_cnt].b_analog_clamp -= step_dn;
            else
                adc_cal[operand->cal_fmt_cnt].b_analog_clamp += step_up;
            if (c > z_ch)
                adc_cal[operand->cal_fmt_cnt].c_analog_clamp -= step_dn;
            else
                adc_cal[operand->cal_fmt_cnt].c_analog_clamp += step_up;
			// load adc cal values
            adc_cal[operand->cal_fmt_cnt].reserved |= TVAFE_ADC_CAL_VALID;
            tvafe_set_cal_value2(&adc_cal[operand->cal_fmt_cnt]);
            adc_cal[operand->cal_fmt_cnt].reserved &= ~TVAFE_ADC_CAL_VALID;
			if (operand->step == TVAFE_ADC_CAL_STEP_STAGE + TVAFE_ADC_CAL_STEP_CLAMP*7 + 1)
			{
#ifdef LOG_ADC_CAL
                    pr_info("\nfresh_adc_cal after clamp\n");
                    pr_info("A_cl = %4d %4d %4d\n",  (int)(adc_cal[operand->cal_fmt_cnt].a_analog_clamp),
                                                     (int)(adc_cal[operand->cal_fmt_cnt].b_analog_clamp),
                                                     (int)(adc_cal[operand->cal_fmt_cnt].c_analog_clamp));
                    pr_info("A_gn = %4d %4d %4d\n",  (int)(adc_cal[operand->cal_fmt_cnt].a_analog_gain),
                                                     (int)(adc_cal[operand->cal_fmt_cnt].b_analog_gain),
                                                     (int)(adc_cal[operand->cal_fmt_cnt].c_analog_gain));
                    pr_info("D_gn = %4d %4d %4d\n",  (int)(adc_cal[operand->cal_fmt_cnt].a_digital_gain),
                                                     (int)(adc_cal[operand->cal_fmt_cnt].b_digital_gain),
                                                     (int)(adc_cal[operand->cal_fmt_cnt].c_digital_gain));
                    pr_info("D_o1 = %4d %4d %4d\n", ((int)(adc_cal[operand->cal_fmt_cnt].a_digital_offset1) << 21) >> 21,
                                                    ((int)(adc_cal[operand->cal_fmt_cnt].b_digital_offset1) << 21) >> 21,
                                                    ((int)(adc_cal[operand->cal_fmt_cnt].c_digital_offset1) << 21) >> 21);
                    pr_info("D_o2 = %4d %4d %4d\n", ((int)(adc_cal[operand->cal_fmt_cnt].a_digital_offset2) << 21) >> 21,
                                                    ((int)(adc_cal[operand->cal_fmt_cnt].b_digital_offset2) << 21) >> 21,
                                                    ((int)(adc_cal[operand->cal_fmt_cnt].c_digital_offset2) << 21) >> 21);
                    pr_info("\n");
#endif
#ifdef TVAFE_ADC_CAL_VALIDATION
				// adjust adc cal values
                    tvafe_adjust_cal_value(adc_cal[operand->cal_fmt_cnt],is_component);
				// load adc cal values
				adc_cal->reserved |= TVAFE_ADC_CAL_VALID;
                    tvafe_set_cal_value2(&adc_cal[operand->cal_fmt_cnt]);
                    adc_cal[operand->cal_fmt_cnt].reserved &= ~TVAFE_ADC_CAL_VALID;
                #endif
#ifdef LOG_ADC_CAL
                    pr_info("\nfinal_adc_cal after validation\n");
                    pr_info("A_cl = %4d %4d %4d\n",  (int)(adc_cal[operand->cal_fmt_cnt].a_analog_clamp),
                                                     (int)(adc_cal[operand->cal_fmt_cnt].b_analog_clamp),
                                                     (int)(adc_cal[operand->cal_fmt_cnt].c_analog_clamp));
                    pr_info("A_gn = %4d %4d %4d\n",  (int)(adc_cal[operand->cal_fmt_cnt].a_analog_gain),
                                                     (int)(adc_cal[operand->cal_fmt_cnt].b_analog_gain),
                                                     (int)(adc_cal[operand->cal_fmt_cnt].c_analog_gain));
                    pr_info("D_gn = %4d %4d %4d\n",  (int)(adc_cal[operand->cal_fmt_cnt].a_digital_gain),
                                                     (int)(adc_cal[operand->cal_fmt_cnt].b_digital_gain),
                                                     (int)(adc_cal[operand->cal_fmt_cnt].c_digital_gain));
                    pr_info("D_o1 = %4d %4d %4d\n", ((int)(adc_cal[operand->cal_fmt_cnt].a_digital_offset1) << 21) >> 21,
                                                    ((int)(adc_cal[operand->cal_fmt_cnt].b_digital_offset1) << 21) >> 21,
                                                    ((int)(adc_cal[operand->cal_fmt_cnt].c_digital_offset1) << 21) >> 21);
                    pr_info("D_o2 = %4d %4d %4d\n", ((int)(adc_cal[operand->cal_fmt_cnt].a_digital_offset2) << 21) >> 21,
                                                    ((int)(adc_cal[operand->cal_fmt_cnt].b_digital_offset2) << 21) >> 21,
                                                    ((int)(adc_cal[operand->cal_fmt_cnt].c_digital_offset2) << 21) >> 21);
                    //pr_info("\n");
#endif

				// restore bpg_h, bpg_v, bpg_m, clamp_inv, clamp_ext, clk_ext, clk_ctl, lpf_a, lpf_b, lpf_c
				W_APB_REG(TVFE_BPG_BACKP_H, operand->bpg_h);
				W_APB_REG(TVFE_BPG_BACKP_V, operand->bpg_v);
				W_APB_BIT(TVFE_TOP_CTRL, operand->bpg_m,
						TVFE_BACKP_GATE_MUX_BIT, TVFE_BACKP_GATE_MUX_WID);
				//W_APB_BIT(TVFE_DVSS_MUXCTRL, operand->clamp_inv, DVSS_CLAMP_INV_BIT,      DVSS_CLAMP_INV_WID     );
				W_APB_BIT(ADC_REG_2F, operand->clamp_ext,
						CLAMPEXT_BIT,            CLAMPEXT_WID           );
				// W_APB_BIT(ADC_REG_19, operand->lpf_a, ENLPFA_BIT,    ENLPFA_WID   );
				// W_APB_BIT(ADC_REG_1A, operand->lpf_b, ENLPFB_BIT,    ENLPFB_WID   );
				// W_APB_BIT(ADC_REG_1B, operand->lpf_c, ENLPFC_BIT,    ENLPFC_WID   );
				W_APB_BIT(ADC_REG_58, operand->clk_ext, EXTCLKSEL_BIT, EXTCLKSEL_WID);
#ifndef CONFIG_ADC_CAL_SIGNALED
				W_APB_BIT(ADC_REG_39, operand->sync_mux, INSYNCMUXCTRL_BIT,INSYNCMUXCTRL_WID);
				WRITE_CBUS_REG(HHI_VAFE_CLKIN_CNTL, operand->clk_ctl);
#endif
				W_APB_REG(TVFE_VAFE_CTRL, operand->vafe_ctl);
				// validate result
				// adc_cal->reserved |= TVAFE_ADC_CAL_VALID;
				// end of cal
				operand->step = 0;
				operand->cal_fmt_cnt++;
			}
			break;
		default:
			break;
	}

    if (operand->step || operand->cal_fmt_cnt < operand->cal_fmt_max){
			if(operand->step == 0 && operand->cal_fmt_cnt == 1)
				pr_info("COMP Calbration for 1080p End\n\n");
			else if(operand->step == 0 && operand->cal_fmt_cnt == 2)
				pr_info("COMP Calbration for 720p and 1080i End\n\n");
            return (true);
    }else{
		operand->cal_fmt_cnt = 0;

		if(operand->cal_fmt_max == 1){
			pr_info("VGA Calbration End\n\n");
		}else{
			pr_info("COMP Calbration for 480 and 576 End\n");
			pr_info("COMP Calbration End\n\n");
		}
        return (false);
    }
}

/*
 * tvafe clamp adjustmen by adc readback module
 */
void tvafe_adc_clamp_adjust(struct tvin_parm_s *parm, struct tvafe_cal_s *cal)
{
	enum tvin_port_e port = parm->port;
	struct tvafe_cal_operand_s *operand = &cal->cal_operand;
	struct tvafe_adc_cal_s *adc_cal = &cal->cal_val;
	unsigned int data = R_APB_REG(TVFE_ADC_READBACK_INDICATOR);
	unsigned int tgt_y = 0, tgt_c = 0;
	bool cond = true;

	// do adjust after TVIN_IOC_S_AFE_ADC_CAL is executed
	if (!(adc_cal->reserved & TVAFE_ADC_CAL_VALID))
		return;

	// skip over the 1st vsync after TVIN_IOC_S_AFE_ADC_CAL is executed
	operand->cnt++;
	if (operand->cnt <= 1)
		return;

	if ((!adc_cal->a_analog_clamp) ||
			(!adc_cal->b_analog_clamp) ||
			(!adc_cal->c_analog_clamp) ||
			(adc_cal->a_analog_clamp == 127) ||
			(adc_cal->b_analog_clamp == 127) ||
			(adc_cal->c_analog_clamp == 127)
	   )
		return;

	if ((port >= TVIN_PORT_COMP0) &&
			(port <= TVIN_PORT_COMP7)
	   )
	{
		tgt_y = 37;
		tgt_c = 512;
	}
	else if ((port >= TVIN_PORT_VGA0) &&
			(port <= TVIN_PORT_VGA7)
		)
	{
		tgt_y = 0;
		tgt_c = 0;
	}
	else
		return;

	operand->adc0 += (data >> 20) & 0x000003ff;
	operand->adc1 += (data >> 10) & 0x000003ff;
	operand->adc2 += (data >>  0) & 0x000003ff;
	if (operand->cnt == 5)
	{
		operand->adj  = 0;
		operand->adc0  = (operand->adc0 + 2) >> 2;
		operand->adc1  = (operand->adc1 + 2) >> 2;
		operand->adc2  = (operand->adc2 + 2) >> 2;
		if (operand->adc0 > tgt_y)
		{
			cond = (!operand->dir) || (operand->dir0);
			if ((adc_cal->a_analog_clamp) && cond)
			{
				adc_cal->a_analog_clamp--;
				operand->dir   = 1;
				operand->dir0  = 1;
				operand->adj  = 1;
			}
		}
		else
		{
			cond = (!operand->dir) || (!operand->dir0);
			if ((adc_cal->a_analog_clamp < 127) && cond)
			{
				adc_cal->a_analog_clamp++;
				operand->dir   = 1;
				operand->adj = 1;
			}
		}
		if (operand->adc1 > tgt_c)
		{
			cond = (!operand->dir) || (operand->dir1);
			if ((adc_cal->b_analog_clamp) && cond)
			{
				adc_cal->b_analog_clamp--;
				operand->dir   = 1;
				operand->dir1  = 1;
				operand->adj  = 1;
			}
		}
		else
		{
			cond = (!operand->dir) || (!operand->dir1);
			if ((adc_cal->b_analog_clamp < 127) && cond)
			{
				adc_cal->b_analog_clamp++;
				operand->dir   = 1;
				operand->adj = 1;
			}
		}
		if (operand->adc2 > tgt_c)
		{
			cond = (!operand->dir) || (operand->dir2);
			if ((adc_cal->c_analog_clamp) && cond)
			{
				adc_cal->c_analog_clamp--;
				operand->dir   = 1;
				operand->dir0  = 1;
				operand->adj  = 1;
			}
		}
		else
		{
			cond = (!operand->dir) || (!operand->dir2);
			if ((adc_cal->c_analog_clamp < 127) && cond)
			{
				adc_cal->c_analog_clamp++;
				operand->dir   = 1;
				operand->adj = 1;
			}
		}
		if (operand->adj)
		{
			tvafe_set_cal_value(cal);
			operand->data0 = operand->adc0;
			operand->data1 = operand->adc1;
			operand->data2 = operand->adc2;
		}
		operand->cnt  = 1;
		operand->adc0 = 0;
		operand->adc1 = 0;
		operand->adc2 = 0;
	}
}

/*
 * tvafe get the wss data from Reg
 */
void tvafe_get_wss_data(struct tvafe_comp_wss_s *wss)
{
	wss->wss1[0] = R_APB_REG(TVFE_MISC_WSS1_INDICATOR1);
	wss->wss1[1] = R_APB_REG(TVFE_MISC_WSS1_INDICATOR2);
	wss->wss1[2] = R_APB_REG(TVFE_MISC_WSS1_INDICATOR3);
	wss->wss1[3] = R_APB_REG(TVFE_MISC_WSS1_INDICATOR4);
	wss->wss1[4] = R_APB_BIT(TVFE_MISC_WSS1_INDICATOR5,
			WSS1_DATA_143_128_BIT, WSS1_DATA_143_128_WID);
	wss->wss2[0] = R_APB_REG(TVFE_MISC_WSS2_INDICATOR1);
	wss->wss2[1] = R_APB_REG(TVFE_MISC_WSS2_INDICATOR2);
	wss->wss2[2] = R_APB_REG(TVFE_MISC_WSS2_INDICATOR3);
	wss->wss2[3] = R_APB_REG(TVFE_MISC_WSS2_INDICATOR4);
	wss->wss2[4] = R_APB_BIT(TVFE_MISC_WSS2_INDICATOR5,
			WSS2_DATA_143_128_BIT, WSS2_DATA_143_128_WID);
}

/*
 * tvafe set the de by format
 */
void tvafe_top_set_de(enum tvin_sig_fmt_e fmt)
{
	unsigned int hs = 0, he = 0, vs = 0, ve = 0;
	const struct tvin_format_s *fmt_info = tvin_get_fmt_info(fmt);

    if(!fmt_info) {
        pr_err("[tvafe..] %s: error,fmt is null!!! \n",__func__);
        return;
    }

	hs = fmt_info->hs_width + fmt_info->hs_bp;

	/* fix Comp de bug*/
	if (fmt < TVIN_SIG_FMT_COMP_MAX && fmt >= TVIN_SIG_FMT_COMP_480P_60HZ_D000)
		hs = (unsigned int)((signed int)hs + (signed int)tvafe_comp_hs_patch[fmt- TVIN_SIG_FMT_COMP_480P_60HZ_D000]);

	vs = fmt_info->vs_width + fmt_info->vs_bp;
	he = hs + fmt_info->h_active - 1;
	ve = vs + fmt_info->v_active - 1;


	W_APB_BIT(TVFE_DEG_H, hs,
			DEG_HSTART_BIT, DEG_HSTART_WID);
	W_APB_BIT(TVFE_DEG_H, he,
			DEG_HEND_BIT, DEG_HEND_WID);
	W_APB_BIT(TVFE_DEG_VODD, vs,
			DEG_VSTART_ODD_BIT, DEG_VSTART_ODD_WID);
	W_APB_BIT(TVFE_DEG_VODD, ve,
			DEG_VEND_ODD_BIT, DEG_VEND_ODD_WID);
	W_APB_BIT(TVFE_DEG_VEVEN, vs,
			DEG_VSTART_EVEN_BIT, DEG_VSTART_EVEN_WID);
	W_APB_BIT(TVFE_DEG_VEVEN, ve,
			DEG_VEND_EVEN_BIT, DEG_VEND_EVEN_WID);

	//fix 1080i60 field gen bug
	if ((fmt == TVIN_SIG_FMT_COMP_1080I_60HZ_D000) |(fmt== TVIN_SIG_FMT_COMP_1080I_50HZ_D000_A))
		W_APB_BIT(TVFE_SYNCTOP_SFG_MUXCTRL1, 0,
				SFG_FLD_MANUAL_INV_BIT, SFG_FLD_MANUAL_INV_WID);
	else
		W_APB_BIT(TVFE_SYNCTOP_SFG_MUXCTRL1, 1,
				SFG_FLD_MANUAL_INV_BIT, SFG_FLD_MANUAL_INV_WID);
}

/*
 * tvafe congigure top setting by format
 */
static void tvafe_top_config(enum tvin_sig_fmt_e fmt)
{

	//tvafe_top_set_aafilter_control(fmt);
	//tvafe_top_set_bp_gate(fmt);
	//tvafe_top_set_mvdet_control(fmt);
	tvafe_top_set_sfg_mux_control(fmt);
	tvafe_top_set_wss_control(fmt);
	tvafe_top_set_de(fmt);
}

/*
 * tvafe configure vga setting by format
 */
void tvafe_set_vga_fmt(struct tvin_parm_s *parm, struct tvafe_cal_s *cal, struct tvafe_pin_mux_s *pinmux)
{
	enum tvin_sig_fmt_e fmt = parm->info.fmt;
	enum tvin_port_e port = parm->port;
	unsigned int i = 0;

	/**enable auto mode clock**/
	WRITE_CBUS_REG(HHI_TVFE_AUTOMODE_CLK_CNTL, 0x100);

	/** write top register **/
	while (vga_top_reg_default[i][0] != 0xFFFFFFFF) {
		W_APB_REG(vga_top_reg_default[i][0], vga_top_reg_default[i][1]);
		i++;
	}

	/** write 7740 register **/
	tvafe_adc_configure(fmt);

	//set top register
	tvafe_top_config(fmt);

	/* load calibration data */
	tvafe_set_cal_value(cal);

	/* must reload mux if you change adc reg table!!! */
	tvafe_set_source_muxing(port, pinmux);

}

/*
 * tvafe configure comp setting by format
 */
void tvafe_set_comp_fmt(struct tvin_parm_s *parm, struct tvafe_cal_s *cal, struct tvafe_pin_mux_s *pinmux)
{
	enum tvin_sig_fmt_e fmt = parm->info.fmt;
	enum tvin_port_e port = parm->port;
	unsigned int i = 0;

	/**enable auto mode clock**/
	WRITE_CBUS_REG(HHI_TVFE_AUTOMODE_CLK_CNTL, 0x100);
	WRITE_CBUS_REG(HHI_VAFE_CLKOSCIN_CNTL, 0x100);

	/** write top register **/
	while (comp_top_reg_default[i][0] != 0xFFFFFFFF) {
		W_APB_REG(comp_top_reg_default[i][0], comp_top_reg_default[i][1]);
		i++;
	}

	/** write 7740 register **/
	tvafe_adc_configure(fmt);

	//set top register
	tvafe_top_config(fmt);

	/* load calibration data */
	tvafe_set_cal_value(cal);

	/* must reload mux if you change adc reg table!!! */
	tvafe_set_source_muxing(port, pinmux);
}

/*
 * tvafe init cvbs setting with pal-i
 */
static void tvafe_set_cvbs_default(struct tvafe_cvd2_s *cvd2, struct tvafe_cvd2_mem_s *mem, enum tvin_port_e port, struct tvafe_pin_mux_s *mux)
{
	unsigned int i = 0;

	/**disable auto mode clock**/
	WRITE_CBUS_REG(HHI_TVFE_AUTOMODE_CLK_CNTL, 0);

#if (MESON_CPU_TYPE < MESON_CPU_TYPE_MESONG9TV)
	/** write 7740 register **/
	tvafe_adc_configure(TVIN_SIG_FMT_CVBS_PAL_I);
#else
	if(port == TVIN_PORT_CVBS3){
	/** DADC CNTL for LIF signal input **/
		WRITE_CBUS_REG(HHI_DADC_CNTL, 0x1411036);
		WRITE_CBUS_REG(HHI_DADC_CNTL2, 0x0);
		WRITE_CBUS_REG(HHI_DADC_CNTL3, 0x430036);
		WRITE_CBUS_REG(HHI_DADC_CNTL4, 0x80600240);
	}else{
	/** CADC CNTL for LIF signal input **/
		WRITE_CBUS_REG(HHI_CADC_CNTL, 0x8022436);
		WRITE_CBUS_REG(HHI_CADC_CNTL2, 0x848000);
	}
	/** enable tv_decoder mem clk **/
	WRITE_CBUS_REG_BITS(HHI_VPU_CLK_CNTL, 1, 28, 1);
#endif
	/** write top register **/
	i = 0;
	
	#if (MESON_CPU_TYPE >= MESON_CPU_TYPE_MESONG9TV)
	if ((port != TVIN_PORT_CVBS3) && (port != TVIN_PORT_CVBS0))
	#else
	if (port != TVIN_PORT_CVBS0)
	#endif
	{
		while (cvbs_top_reg_default[i][0] != 0xFFFFFFFF){
			
			W_APB_REG(cvbs_top_reg_default[i][0], cvbs_top_reg_default[i][1]);
			i++;		
			
			}
		
		pr_info("\n CVBS IN USE \n");
		pr_info("\n reg_TVFE_TOP_CTRL = %x \n",R_APB_REG(TVFE_TOP_CTRL));

	}else{
		
		while (cvbs_atv_top_reg_default[i][0] != 0xFFFFFFFF){

			W_APB_REG(cvbs_atv_top_reg_default[i][0], cvbs_atv_top_reg_default[i][1]);
			i++;
	
			}
		pr_info("\n  ATV IN USE \n");
		pr_info("\n reg_TVFE_TOP_CTRL = %x \n",R_APB_REG(TVFE_TOP_CTRL));
	}
	/* init some variables  */
	cvd2->vd_port = port;

	/* set cvd2 default format to pal-i */
	tvafe_cvd2_try_format(cvd2, mem, TVIN_SIG_FMT_CVBS_PAL_I);

}

void tvafe_enable_avout(enum tvin_port_e port,bool enable)
{
#if (MESON_CPU_TYPE >= MESON_CPU_TYPE_MESONG9TV)
	if(enable){
		if(port == TVIN_PORT_CVBS3){
			WRITE_CBUS_REG_BITS(HHI_VDAC_CNTL0, 0, 10, 1);//disable AFE output buffer
			WRITE_CBUS_REG_BITS(HHI_VDAC_CNTL1, 0, 3, 1);//open CVBS DAC
			WRITE_CBUS_REG_BITS(HHI_VDAC_CNTL0, 1, 0, 3);//clock delay control
			WRITE_CBUS_REG_BITS(HHI_VIID_CLK_DIV, 1, 19, 1);//vdac_clock_mux form atv demod
			WRITE_CBUS_REG_BITS(HHI_VID_CLK_CNTL2, 1, 8, 1);//
			WRITE_CBUS_REG_BITS(HHI_VID_CLK_CNTL2, 1, 4, 1);//vdac_clk gated clock control
			WRITE_VCBUS_REG_BITS(VENC_VDAC_DACSEL0, 1, 5, 1);
			//WRITE_CBUS_REG_BITS(HHI_GCLK_OTHER, 1, 0, 1);//if set not need set again
		}
		else{
			W_APB_REG(TVFE_ATV_DMD_CLP_CTRL, 0);//close atv demod to cvd
			WRITE_CBUS_REG_BITS(HHI_VDAC_CNTL1, 1, 3, 1);//close CVBS DAC(CDAC_PWD=1)
			WRITE_CBUS_REG_BITS(HHI_VDAC_CNTL0, 1, 10, 1);//Enable AFE output buffer
		}
	}
	else{
		WRITE_CBUS_REG_BITS(HHI_VDAC_CNTL1, 1, 3, 1);//close CVBS DAC
		WRITE_CBUS_REG_BITS(HHI_VDAC_CNTL0, 0, 10, 1);//Disable AFE output buffer
	}
#else
	if(enable)
		//close/open av out
		W_APB_BIT(ADC_REG_20, 1, ENCVBSBUF_BIT,ENCVBSBUF_WID);
	else
		//close av out
		W_APB_BIT(ADC_REG_20, 0, ENCVBSBUF_BIT,ENCVBSBUF_WID);
#endif
}
/*
 * tvafe init the whole module
 */
static bool enableavout = true;
module_param(enableavout, bool, 0644);
MODULE_PARM_DESC(enableavout, "disable av out when load adc reg");
void tvafe_init_reg(struct tvafe_cvd2_s *cvd2, struct tvafe_cvd2_mem_s *mem, enum tvin_port_e port, struct tvafe_pin_mux_s *pinmux)
{
	unsigned int i = 0;

	if ((port >= TVIN_PORT_VGA0) && (port <= TVIN_PORT_VGA7))
	{
		/**enable auto mode clock**/
		WRITE_CBUS_REG(HHI_TVFE_AUTOMODE_CLK_CNTL, 0x100);


		/** write top register don't after write 7740 register**/
		while (vga_top_reg_default[i][0] != 0xFFFFFFFF) {
			W_APB_REG(vga_top_reg_default[i][0], vga_top_reg_default[i][1]);
			i++;
		}
		/** write 7740 register **/
		tvafe_adc_configure(TVIN_SIG_FMT_VGA_1024X768P_60HZ_D004);

		tvafe_top_config(TVIN_SIG_FMT_VGA_1024X768P_60HZ_D004);
	}
	else if ((port >= TVIN_PORT_COMP0) && (port <= TVIN_PORT_COMP7))
	{
		/**enable auto mode clock**/
		WRITE_CBUS_REG(HHI_TVFE_AUTOMODE_CLK_CNTL, 0x100);
		WRITE_CBUS_REG(HHI_VAFE_CLKOSCIN_CNTL, 0x100);

		/** write top register don't after write 7740 register**/
		while (comp_top_reg_default[i][0] != 0xFFFFFFFF) {
			W_APB_REG(comp_top_reg_default[i][0], comp_top_reg_default[i][1]);
			i++;
		}
		/** write 7740 register **/
		tvafe_adc_configure(TVIN_SIG_FMT_COMP_720P_59HZ_D940);

		tvafe_top_config(TVIN_SIG_FMT_COMP_720P_59HZ_D940);
	}
	else if ((port >= TVIN_PORT_CVBS0) && (port <= TVIN_PORT_SVIDEO7))
	{
#ifdef CRYSTAL_25M
                WRITE_CBUS_REG(HHI_VAFE_CLKIN_CNTL, 0x703);//can't write !!!
#endif
		if((port >= TVIN_PORT_CVBS0) && (port <= TVIN_PORT_CVBS7)){			
#if (defined(CONFIG_ADC_DOUBLE_SAMPLING_FOR_CVBS) && defined(CRYSTAL_24M) && (MESON_CPU_TYPE >= MESON_CPU_TYPE_MESONG9TV))
			
		#if (MESON_CPU_TYPE >= MESON_CPU_TYPE_MESONG9TV)
			if ((port != TVIN_PORT_CVBS3) && (port != TVIN_PORT_CVBS0))
		#else
			if (port != TVIN_PORT_CVBS0)
		#endif

			{
				WRITE_CBUS_REG(HHI_ADC_PLL_CNTL3,0xa92a2110);
				WRITE_CBUS_REG(HHI_ADC_PLL_CNTL4,0x02973800);
				WRITE_CBUS_REG(HHI_ADC_PLL_CNTL,0x08664220);
				WRITE_CBUS_REG(HHI_ADC_PLL_CNTL2,0x34e0bf80);
				WRITE_CBUS_REG(HHI_ADC_PLL_CNTL3,0x292a2110);//reset clcok
			}
			else
#endif
			{
				WRITE_CBUS_REG(HHI_ADC_PLL_CNTL3,0xce7a2110);
				WRITE_CBUS_REG(HHI_ADC_PLL_CNTL4,0x2933800);
				WRITE_CBUS_REG(HHI_ADC_PLL_CNTL,0x0484680);
				WRITE_CBUS_REG(HHI_ADC_PLL_CNTL2,0x34e0bf84);
				WRITE_CBUS_REG(HHI_ADC_PLL_CNTL3,0x4e7a2110);//reset clcok
			}
		}
		tvafe_set_cvbs_default(cvd2, mem, port, pinmux);
		/*turn on/off av out*/
		tvafe_enable_avout(port,enableavout);
#if (MESON_CPU_TYPE >= MESON_CPU_TYPE_MESONG9TV)
		/*Enable AFE Bandgap*/
		WRITE_CBUS_REG_BITS(HHI_VDAC_CNTL0, 0, 9, 1);//CDAC_CTRL_RESV2<1>=0
#endif
	}

	/* must reload mux if you change adc reg table!!! */
	tvafe_set_source_muxing(port, pinmux);

	pr_info("[tvafe..] %s ok.\n", __func__);

}

/*
 * tvafe set APB bus register accessing error exception
 */
void tvafe_set_apb_bus_err_ctrl(void)
{
	W_APB_REG(TVFE_APB_ERR_CTRL_MUX1, 0x8fff8fff);
	W_APB_REG(TVFE_APB_ERR_CTRL_MUX2, 0x00008fff);
}

/*
 * tvafe reset the whole module
 */
static void tvafe_reset_module(void)
{
	W_APB_BIT(TVFE_RST_CTRL, 1, ALL_CLK_RST_BIT, ALL_CLK_RST_WID);
	W_APB_BIT(TVFE_RST_CTRL, 0, ALL_CLK_RST_BIT, ALL_CLK_RST_WID);
}

/*
 * tvafe power control of the module
 */
void tvafe_enable_module(bool enable)
{
	// enable

	//main clk up
	WRITE_CBUS_REG(HHI_VAFE_CLKXTALIN_CNTL, 0x100);
	WRITE_CBUS_REG(HHI_VAFE_CLKOSCIN_CNTL, 0x100);
	WRITE_CBUS_REG(HHI_VAFE_CLKIN_CNTL, 0x100);
	WRITE_CBUS_REG(HHI_VAFE_CLKPI_CNTL, 0x100);
	WRITE_CBUS_REG(HHI_TVFE_AUTOMODE_CLK_CNTL, 0x100);

	//tvfe power up
	W_APB_BIT(TVFE_TOP_CTRL, 1, COMP_CLK_ENABLE_BIT, COMP_CLK_ENABLE_WID);
	W_APB_BIT(TVFE_TOP_CTRL, 1, EDID_CLK_EN_BIT, EDID_CLK_EN_WID);
	W_APB_BIT(TVFE_TOP_CTRL, 1, DCLK_ENABLE_BIT, DCLK_ENABLE_WID);
	W_APB_BIT(TVFE_TOP_CTRL, 1, VAFE_MCLK_EN_BIT, VAFE_MCLK_EN_WID);
	W_APB_BIT(TVFE_TOP_CTRL, 3, TVFE_ADC_CLK_DIV_BIT, TVFE_ADC_CLK_DIV_WID);
	//adc power up
#if (MESON_CPU_TYPE < MESON_CPU_TYPE_MESONG9TV)
	W_APB_BIT(ADC_REG_21, 1, FULLPDZ_BIT, FULLPDZ_WID);
#endif
	/*reset module*/
	tvafe_reset_module();

	// disable
	if (!enable)
	{
		//adc power down
#if (MESON_CPU_TYPE < MESON_CPU_TYPE_MESONG9TV)
		W_APB_BIT(ADC_REG_21, 0, FULLPDZ_BIT, FULLPDZ_WID);
#endif
		//tvfe power down
		W_APB_BIT(TVFE_TOP_CTRL, 0, COMP_CLK_ENABLE_BIT, COMP_CLK_ENABLE_WID);
		W_APB_BIT(TVFE_TOP_CTRL, 0, EDID_CLK_EN_BIT, EDID_CLK_EN_WID);
		W_APB_BIT(TVFE_TOP_CTRL, 0, DCLK_ENABLE_BIT, DCLK_ENABLE_WID);
		W_APB_BIT(TVFE_TOP_CTRL, 0, VAFE_MCLK_EN_BIT, VAFE_MCLK_EN_WID);
		W_APB_BIT(TVFE_TOP_CTRL, 0, TVFE_ADC_CLK_DIV_BIT, TVFE_ADC_CLK_DIV_WID);

		//main clk down
		WRITE_CBUS_REG(HHI_VAFE_CLKXTALIN_CNTL, 0);
		WRITE_CBUS_REG(HHI_VAFE_CLKOSCIN_CNTL, 0);
		WRITE_CBUS_REG(HHI_VAFE_CLKIN_CNTL, 0);
		WRITE_CBUS_REG(HHI_VAFE_CLKPI_CNTL, 0);
		WRITE_CBUS_REG(HHI_TVFE_AUTOMODE_CLK_CNTL, 0);
	}
}

